@@ -1,7 +1,7 @@
 <?php
 /*
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004-2017 The Cacti Group                                 |
+ | Copyright (C) 2004-2020 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU General Public License             |
@@ -23,6 +23,7 @@
 */
 
 include('./include/auth.php');
+include_once('./lib/poller.php');
 include_once('./lib/utility.php');
 
 $at_actions = array(
@@ -77,50 +78,21 @@ function automation_template_dnd() {
 	get_filter_request_var('id');
 	/* ================= Input validation ================= */
 
-	if (!isset_request_var('template_ids') || !is_array(get_nfilter_request_var('template_ids'))) exit;
+	if (isset_request_var('template_ids') && is_array(get_nfilter_request_var('template_ids'))) {
+		$aids     = get_nfilter_request_var('template_ids');
+		$sequence = 1;
 
-	/* template table contains one row defined as 'nodrag&nodrop' */
-//	unset($_REQUEST['template_ids'][0]);
+		foreach($aids as $id) {
+			$id = str_replace('line', '', $id);
+			input_validate_input_number($id);
 
-	/* delivered template ids has to be exactly the same like we have stored */
-	$old_order = array();
+			db_execute_prepared('UPDATE automation_templates
+				SET sequence = ?
+				WHERE id = ?',
+				array($sequence, $id));
 
-	$i = 1;
-	foreach(get_nfilter_request_var('template_ids') as $sequence => $id) {
-		if (empty($id)) continue;
-		$new_order[$i] = str_replace('line', '', $id);
-		$i++;
-	}
-
-	$items = db_fetch_assoc('SELECT id, sequence FROM automation_templates ORDER BY sequence');
-
-	if (sizeof($items)) {
-		$i = 1;
-		foreach($items as $item) {
-			$old_order[$i] = $item['id'];
-			$i++;
+			$sequence++;
 		}
-	} else {
-		header('Location: automation_templates.php?header=false');
-		exit;
-	}
-
-	/* perform some sanity checks */
-	if (sizeof(array_diff($new_order, $old_order)) > 0) {
-		header('Location: automation_templates.php?header=false');
-		exit;
-	}
-
-	if (sizeof(array_diff_key($new_order, $old_order))>0) {
-		header('Location: automation_templates.php?header=false');
-		exit;
-	}
-
-	foreach($new_order as $sequence => $id) {
-		input_validate_input_number($sequence);
-		input_validate_input_number($id);
-
-		db_execute_prepared('UPDATE automation_templates SET sequence = ? WHERE id = ?', array($sequence, $id));
 	}
 
 	header('Location: automation_templates.php?header=false');
@@ -183,7 +155,7 @@ function form_actions() {
 
 	html_start_box($at_actions[get_nfilter_request_var('drp_action')], '60%', '', '3', 'center', '');
 
-	if (isset($at_array) && sizeof($at_array)) {
+	if (isset($at_array) && cacti_sizeof($at_array)) {
 		if (get_nfilter_request_var('drp_action') == '1') { /* delete */
 			print "<tr>
 				<td class='textArea' class='odd'>
@@ -192,18 +164,19 @@ function form_actions() {
 				</td>
 			</tr>\n";
 
-			$save_html = "<input type='button' value='" . __esc('Cancel') . "' onClick='cactiReturnTo()'>&nbsp;<input type='submit' value='" . __esc('Continue') . "' title='" . __esc('Delete Automation Template(s)') . "'>";
+			$save_html = "<input type='button' class='ui-button ui-corner-all ui-widget' value='" . __esc('Cancel') . "' onClick='cactiReturnTo()'>&nbsp;<input type='submit' class='ui-button ui-corner-all ui-widget' value='" . __esc('Continue') . "' title='" . __esc('Delete Automation Template(s)') . "'>";
 		}
 	} else {
-		print "<tr><td class='odd'><span class='textError'>" . __('You must select at least one Automation Template.') . "</span></td></tr>\n";
-		$save_html = "<input type='button' value='" . __esc('Return') . "' onClick='cactiReturnTo()'>";
+		raise_message(40);
+		header('Location: automation_templates.php?header=false');
+		exit;
 	}
 
 	print "<tr>
 		<td class='saveRow'>
 			<input type='hidden' name='action' value='actions'>
 			<input type='hidden' name='selected_items' value='" . (isset($at_array) ? serialize($at_array) : '') . "'>
-			<input type='hidden' name='drp_action' value='" . get_nfilter_request_var('drp_action') . "'>
+			<input type='hidden' name='drp_action' value='" . html_escape(get_nfilter_request_var('drp_action')) . "'>
 			$save_html
 		</td>
 	</tr>\n";
@@ -260,7 +233,7 @@ function automation_get_child_branches($tree_id, $id, $spaces, $headers) {
 
 	$spaces .= '--';
 
-	if (sizeof($items)) {
+	if (cacti_sizeof($items)) {
 	foreach($items as $i) {
 		$headers['tr_' . $tree_id . '_bi_' . $i['id']] = $spaces . ' ' . $i['title'];
 		$headers = automation_get_child_branches($tree_id, $i['id'], $spaces, $headers);
@@ -289,7 +262,7 @@ function template_edit() {
 
 	$template_names = array();
 
-	if (sizeof($host_template_names)) {
+	if (cacti_sizeof($host_template_names)) {
 		foreach ($host_template_names as $ht) {
 			$template_names[$ht['id']] = $ht['name'];
 		}
@@ -353,7 +326,7 @@ function template_edit() {
 			array(get_request_var('id')));
 
 		if (isset($template_names[$host_template['host_template']])) {
-			$header_label = __('Automation Templates [edit: %s]', html_escape($template_names[$host_template['host_template']]));
+			$header_label = __esc('Automation Templates [edit: %s]', $template_names[$host_template['host_template']]);
 		} else {
 			$header_label = __('Automation Templates for [Deleted Template]');
 		}
@@ -393,10 +366,9 @@ function template() {
 			'default' => '1'
 			),
 		'filter' => array(
-			'filter' => FILTER_CALLBACK,
+			'filter' => FILTER_DEFAULT,
 			'pageset' => true,
-			'default' => '',
-			'options' => array('options' => 'sanitize_search_string')
+			'default' => ''
 			)
 	);
 
@@ -421,7 +393,7 @@ function template() {
 						<?php print __('Search');?>
 					</td>
 					<td>
-						<input id='filter' type='text' size='25' value='<?php print html_escape_request_var('filter');?>'>
+						<input type='text' class='ui-state-default ui-corner-all' id='filter' size='25' value='<?php print html_escape_request_var('filter');?>'>
 					</td>
 					<td>
 						<?php print __('Templates');?>
@@ -430,7 +402,7 @@ function template() {
 						<select id='rows' onChange='applyFilter()'>
 							<option value='-1'<?php print (get_request_var('rows') == '-1' ? ' selected>':'>') . __('Default');?></option>
 							<?php
-							if (sizeof($item_rows) > 0) {
+							if (cacti_sizeof($item_rows) > 0) {
 								foreach ($item_rows as $key => $value) {
 									print "<option value='" . $key . "'"; if (get_request_var('rows') == $key) { print ' selected'; } print '>' . html_escape($value) . "</option>\n";
 								}
@@ -440,8 +412,8 @@ function template() {
 					</td>
 					<td>
 						<span>
-							<input type='button' id='refresh' value='<?php print __esc('Go');?>' title='<?php print __esc('Set/Refresh Filters');?>'>
-							<input type='button' id='clear' value='<?php print __esc('Clear');?>' title='<?php print __esc('Clear Filters');?>'>
+							<input type='button' class='ui-button ui-corner-all ui-widget' id='refresh' value='<?php print __esc('Go');?>' title='<?php print __esc('Set/Refresh Filters');?>'>
+							<input type='button' class='ui-button ui-corner-all ui-widget' id='clear' value='<?php print __esc('Clear');?>' title='<?php print __esc('Clear Filters');?>'>
 						</span>
 					</td>
 				</tr>
@@ -485,10 +457,10 @@ function template() {
 
 	/* form the 'where' clause for our main sql query */
 	if (get_request_var('filter') != '') {
-		$sql_where = "WHERE (name LIKE '%" . get_request_var('filter') . "%' OR " .
-			"sysName LIKE '%" . get_request_var('filter') . "%' OR " .
-			"sysDescr LIKE '%" . get_request_var('filter') . "%' OR " .
-			"sysOID LIKE '%" . get_request_var('filter') . "%')";
+		$sql_where = 'WHERE (name LIKE ' . db_qstr('%' . get_request_var('filter') . '%') . ' OR ' .
+			'sysName LIKE ' . db_qstr('%' . get_request_var('filter') . '%') . ' OR ' .
+			'sysDescr LIKE ' . db_qstr('%' . get_request_var('filter') . '%') . ' OR ' .
+			'sysOID LIKE ' . db_qstr('%' . get_request_var('filter') . '%') . ')';
 	} else {
 		$sql_where = '';
 	}
@@ -499,7 +471,7 @@ function template() {
 		ON ht.id=at.host_template
 		$sql_where");
 
-	$dts = db_fetch_assoc("SELECT at.*, '' AS sysName, ht.name
+	$dts = db_fetch_assoc("SELECT at.*, ht.name
 		FROM automation_templates AS at
 		LEFT JOIN host_template AS ht
 		ON ht.id=at.host_template
@@ -527,11 +499,11 @@ function template() {
 		$display_text[] = array('display' => __('Order'), 'align' => 'center');
 	}
 
-	html_header_checkbox($display_text, get_request_var('sort_column'), get_request_var('sort_direction'), false);
+	html_header_checkbox($display_text, false);
 
 	$i = 1;
-	$total_items = sizeof($dts);
-	if (sizeof($dts)) {
+	$total_items = cacti_sizeof($dts);
+	if (cacti_sizeof($dts)) {
 		foreach ($dts as $dt) {
 			if ($dt['name'] == '') {
 				$name = __('Unknown Template');
@@ -569,12 +541,12 @@ function template() {
 			$i++;
 		}
 	} else {
-		print "<tr><td><em>" . __('No Automation Device Templates Found') . "</em></td></tr>\n";
+		print "<tr class='tableRow'><td colspan='" . (cacti_sizeof($display_text)+1) . "'><em>" . __('No Automation Device Templates Found') . "</em></td></tr>\n";
 	}
 
 	html_end_box(false);
 
-	if (sizeof($dts)) {
+	if (cacti_sizeof($dts)) {
 		print $nav;
 	}
 
