@@ -1,8 +1,8 @@
-#!/usr/bin/php -q
+#!/usr/bin/env php
 <?php
 /*
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004-2018 The Cacti Group                                 |
+ | Copyright (C) 2004-2020 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU General Public License             |
@@ -23,27 +23,19 @@
  +-------------------------------------------------------------------------+
 */
 
-/* do NOT run this script through a web browser */
-if (!isset($_SERVER['argv'][0]) || isset($_SERVER['REQUEST_METHOD'])  || isset($_SERVER['REMOTE_ADDR'])) {
-	die('<br><strong>This script is only meant to run at the command line.</strong>');
-}
-
-/* we are not talking to the browser */
-$no_http_headers = true;
-
-/* start initialization section */
-include(dirname(__FILE__) . '/../include/global.php');
-include_once($config['base_path'] . '/lib/poller.php');
-include_once($config['base_path'] . '/lib/data_query.php');
-include_once($config['base_path'] . '/lib/dsstats.php');
-include_once($config['base_path'] . '/lib/boost.php');
-include_once($config['base_path'] . '/lib/rrd.php');
+require(__DIR__ . '/../include/cli_check.php');
+require_once($config['base_path'] . '/lib/poller.php');
+require_once($config['base_path'] . '/lib/data_query.php');
+require_once($config['base_path'] . '/lib/dsstats.php');
+require_once($config['base_path'] . '/lib/dsdebug.php');
+require_once($config['base_path'] . '/lib/boost.php');
+require_once($config['base_path'] . '/lib/rrd.php');
 
 /* process calling arguments */
 $parms = $_SERVER['argv'];
 array_shift($parms);
 
-if (sizeof($parms)) {
+if (cacti_sizeof($parms)) {
 	foreach($parms as $parameter) {
 		if (strpos($parameter, '=')) {
 			list($arg, $value) = explode('=', $parameter);
@@ -53,20 +45,20 @@ if (sizeof($parms)) {
 		}
 
 		switch ($arg) {
-			case '--version':
-			case '-V':
-			case '-v':
-				display_help();
-				exit;
-			case '--help':
-			case '-H':
-			case '-h':
-				display_help();
-				exit;
-			default:
-				echo 'ERROR: Invalid Parameter ' . $parameter . "\n\n";
-				display_help();
-				exit;
+		case '--version':
+		case '-V':
+		case '-v':
+			display_version();
+			exit(0);
+		case '--help':
+		case '-H':
+		case '-h':
+			display_help();
+			exit(0);
+		default:
+			print 'ERROR: Invalid Parameter ' . $parameter . "\n\n";
+			display_help();
+			exit(1);
 		}
 	}
 }
@@ -83,20 +75,20 @@ while (db_fetch_cell('SELECT count(*) FROM poller_output') > 0) {
 	$rrds_processed = $rrds_processed + process_poller_output($rrdtool_pipe, false);
 }
 
-echo "There were $rrds_processed, RRD updates made this pass\n";
+print "There were $rrds_processed RRD updates made this pass\n";
 
 rrd_close($rrdtool_pipe);
 
 /*  display_version - displays version information */
 function display_version() {
-	$version = get_cacti_version();
-	echo "Cacti Process Poller Output Utility, Version $version, " . COPYRIGHT_YEARS . "\n";
+	$version = get_cacti_cli_version();
+	print "Cacti Process Poller Output Utility, Version $version, " . COPYRIGHT_YEARS . "\n";
 }
 
 /*	display_help - displays the usage of the function */
 function display_help () {
 	display_version();
 
-	echo "\nusage: poller_output_empty.php\n\n";
-	echo "A utility to process the poller output table.  This tool is deprecated but should work.\n\n";
+	print "\nusage: poller_output_empty.php\n\n";
+	print "A utility to process the poller output table.  This tool is deprecated but should work.\n\n";
 }
