@@ -43,7 +43,7 @@ get_filter_request_var('stdout');
 /* flush the headers now */
 ob_end_clean();
 
-session_write_close();
+cacti_session_close();
 
 $graph_data_array = array();
 
@@ -117,47 +117,82 @@ if (isset_request_var('format') && get_nfilter_request_var('format') == 'table')
 
 if (is_array($xport_array['meta']) && isset($xport_array['meta']['start'])) {
 	if (!$html) {
-		print '"' . __('Title') . '","'          . $xport_array['meta']['title_cache']    . '"' . "\n";
-		print '"' . __('Vertical Label') . '","' . $xport_array['meta']['vertical_label'] . '"' . "\n";
+		$output  = '"' . __('Title') . '","'          . $xport_array['meta']['title_cache']    . '"' . "\n";
+		$output .= '"' . __('Vertical Label') . '","' . $xport_array['meta']['vertical_label'] . '"' . "\n";
 
-		print '"' . __('Start Date') . '","'     . date('Y-m-d H:i:s', $xport_array['meta']['start']) . '"' . "\n";
-		print '"' . __('End Date') . '","'       . date('Y-m-d H:i:s', ($xport_array['meta']['end'] == $xport_array['meta']['start']) ? $xport_array['meta']['start'] + $xport_array['meta']['step']*($xport_array['meta']['rows']-1) : $xport_array['meta']['end']) . '"' . "\n";
-		print '"' . __('Step') . '","'           . $xport_array['meta']['step']                       . '"' . "\n";
-		print '"' . __('Total Rows') . '","'     . $xport_array['meta']['rows']                       . '"' . "\n";
-		print '"' . __('Graph ID') . '","'       . $xport_array['meta']['local_graph_id']             . '"' . "\n";
-		print '"' . __('Host ID') . '","'        . $xport_array['meta']['host_id']                    . '"' . "\n";
+		$output .= '"' . __('Start Date') . '","'     . date('Y-m-d H:i:s', $xport_array['meta']['start']) . '"' . "\n";
+		$output .= '"' . __('End Date') . '","'       . date('Y-m-d H:i:s', ($xport_array['meta']['end'] == $xport_array['meta']['start']) ? $xport_array['meta']['start'] + $xport_array['meta']['step']*($xport_array['meta']['rows']-1) : $xport_array['meta']['end']) . '"' . "\n";
+		$output .= '"' . __('Step') . '","'           . $xport_array['meta']['step']                       . '"' . "\n";
+		$output .= '"' . __('Total Rows') . '","'     . $xport_array['meta']['rows']                       . '"' . "\n";
+		$output .= '"' . __('Graph ID') . '","'       . $xport_array['meta']['local_graph_id']             . '"' . "\n";
+		$output .= '"' . __('Host ID') . '","'        . $xport_array['meta']['host_id']                    . '"' . "\n";
 
 		if (isset($xport_meta['NthPercentile'])) {
 			foreach($xport_meta['NthPercentile'] as $item) {
-				print '"' . __('Nth Percentile') . '","' . $item['value'] . '","' . $item['format'] . '"' . "\n";
+				$output .= '"' . __('Nth Percentile') . '","' . $item['value'] . '","' . $item['format'] . '"' . "\n";
 			}
 		}
 
 		if (isset($xport_meta['Summation'])) {
 			foreach($xport_meta['Summation'] as $item) {
-				print '"' . __('Summation') . '","' . $item['value'] . '","' . $item['format'] . '"' . "\n";
+				$output .= '"' . __('Summation') . '","' . $item['value'] . '","' . $item['format'] . '"' . "\n";
 			}
 		}
 
-		print '""' . "\n";
+		$output .= '""' . "\n";
 
 		$header = '"' . __('Date') . '"';
 		for ($i = 1; $i <= $xport_array['meta']['columns']; $i++) {
 			$header .= ',"' . $xport_array['meta']['legend']['col' . $i] . '"';
 		}
-		print $header . "\n";
+		$output .= $header . "\n";
 	} else {
-		$second = "class='right' colspan='2'";
 		print "<table class='cactiTable' class='center'>\n";
-		print "<tr class='tableHeader'><td colspan='2' class='linkOverDark' style='font-weight:bold;'>" . __('Summary Details') . "</td><td class='right'><a href='#' role='link' style='cursor:pointer;' class='download linkOverDark' id='graph_" . $xport_array['meta']['local_graph_id'] . "'>" . __('Download') . "</a></td></tr>\n";
-		print "<tr class='even'><td class='left'>" . __('Title') . "</td><td $second>"          . html_escape($xport_array['meta']['title_cache'])      . "</td></tr>\n";
-		print "<tr class='odd'><td class='left'>" . __('Vertical Label') . "</td><td $second>" . html_escape($xport_array['meta']['vertical_label'])    . "</td></tr>\n";
-		print "<tr class='even'><td class='left'>" . __('Start Date') . "</td><td $second>"     . date('Y-m-d H:i:s', $xport_array['meta']['start']) . "</td></tr>\n";
-		print "<tr class='odd'><td class='left'>" . __('End Date') . "</td><td $second>"       . date('Y-m-d H:i:s', ($xport_array['meta']['end'] == $xport_array['meta']['start']) ? $xport_array['meta']['start'] + $xport_array['meta']['step']*($xport_array['meta']['rows']-1) : $xport_array['meta']['end']) . "</td></tr>\n";
-		print "<tr class='even'><td class='left'>" . __('Step') . "</td><td $second>"           . $xport_array['meta']['step']                       . "</td></tr>\n";
-		print "<tr class='odd'><td class='left'>" . __('Total Rows') . "</td><td $second>"     . $xport_array['meta']['rows']                        . "</td></tr>\n";
-		print "<tr class='even'><td class='left'>" . __('Graph ID') . "</td><td $second>"       . $xport_array['meta']['local_graph_id']             . "</td></tr>\n";
-		print "<tr class='odd'><td class='left'>" . __('Host ID') . "</td><td $second>"        . $xport_array['meta']['host_id']                     . "</td></tr>\n";
+
+		print "<tr class='tableHeader'>
+			<td>" . __('Summary Details') . "</td>
+			<td class='right'><a href='#' role='link' style='cursor:pointer;' class='download linkOverDark' id='graph_" . $xport_array['meta']['local_graph_id'] . "'>" . __('Download') . "</a></td>
+		</tr>\n";
+
+		print "<tr class='even'>
+			<td class='left' style='width:40%;'>" . __('Title') . "</td>
+			<td class='right'>" . html_escape($xport_array['meta']['title_cache']) . "</td>
+		</tr>\n";
+
+		print "<tr class='odd'>
+			<td class='left'>" . __('Vertical Label') . "</td>
+			<td class='right'>" . html_escape($xport_array['meta']['vertical_label']) . "</td>
+		</tr>\n";
+
+		print "<tr class='even'>
+			<td class='left'>" . __('Start Date') . "</td>
+			<td class='right'>" . date('Y-m-d H:i:s', $xport_array['meta']['start']) . "</td>
+		</tr>\n";
+
+		print "<tr class='odd'>
+			<td class='left'>" . __('End Date') . "</td>
+			<td class='right'>" . date('Y-m-d H:i:s', ($xport_array['meta']['end'] == $xport_array['meta']['start']) ? $xport_array['meta']['start'] + $xport_array['meta']['step']*($xport_array['meta']['rows']-1) : $xport_array['meta']['end']) . "</td>
+		</tr>\n";
+
+		print "<tr class='even'>
+			<td class='left'>" . __('Step') . "</td>
+			<td class='right'>" . $xport_array['meta']['step'] . "</td>
+		</tr>\n";
+
+		print "<tr class='odd'>
+			<td class='left'>" . __('Total Rows') . "</td>
+			<td class='right'>" . $xport_array['meta']['rows'] . "</td>
+		</tr>\n";
+
+		print "<tr class='even'>
+			<td class='left'>" . __('Graph ID') . "</td>
+			<td class='right'>" . $xport_array['meta']['local_graph_id'] . "</td>
+		</tr>\n";
+
+		print "<tr class='odd'>
+			<td class='left'>"  . __('Host ID') . "</td>
+			<td class='right'>" . $xport_array['meta']['host_id'] . "</td>
+		</tr>\n";
 
 		$class = 'even';
 		if (isset($xport_meta['NthPercentile'])) {
@@ -167,9 +202,14 @@ if (is_array($xport_array['meta']) && isset($xport_array['meta']['start'])) {
 				} else {
 					$class = 'even';
 				}
-				print "<tr class='$class'><td>" . __('Nth Percentile') . "</td><td class='left'>" . $item['value'] . "</td><td class='right'>" . $item['format'] . "</td></tr>\n";
+
+				print "<tr class='$class'>
+					<td class='left'>" . __('Nth Percentile') . ' [ ' . html_escape($item['format']) . " ]</td>
+					<td class='right'>" . html_escape($item['value']) . "</td>
+				</tr>\n";
 			}
 		}
+
 		if (isset($xport_meta['Summation'])) {
 			foreach($xport_meta['Summation'] as $item) {
 				if ($class == 'even') {
@@ -177,7 +217,11 @@ if (is_array($xport_array['meta']) && isset($xport_array['meta']['start'])) {
 				} else {
 					$class = 'even';
 				}
-				print "<tr class='$class'><td>" . __('Summation') . "</td><td class='left'>" . $item['value'] . "</td><td class='right'>" . $item['format'] . "</td></tr>\n";
+
+				print "<tr class='$class'>
+					<td class='left'>" . __('Summation') . ' [ ' . html_escape($item['format']) . " ]</td>
+					<td class='right'>" . html_escape($item['value']) . "</td>
+				</tr>\n";
 			}
 		}
 
@@ -186,10 +230,13 @@ if (is_array($xport_array['meta']) && isset($xport_array['meta']['start'])) {
 		print "<div class='wrapperMain' style='display:none;'>\n";
 		print "<table id='csvExport' class='cactiTable'><thead>\n";
 
-		print "<tr class='tableHeader'><th class='tableSubHeaderColumn left ui-resizable'>Date</th>";
+		print "<tr class='tableHeader'>
+			<th class='tableSubHeaderColumn left ui-resizable'>" . __('Date') . "</th>\n";
+
 		for ($i = 1; $i <= $xport_array['meta']['columns']; $i++) {
-			print "<th class='{sorter: \"numberFormat\"} tableSubHeaderColumn right ui-resizable'>" . $xport_array['meta']['legend']['col' . $i] . "</th>";
+			print "<th class='{sorter: \"numberFormat\"} tableSubHeaderColumn right ui-resizable'>" . $xport_array['meta']['legend']['col' . $i] . "</th>\n";
 		}
+
 		print "</tr></thead>\n";
 	}
 }
@@ -202,27 +249,33 @@ if (isset($xport_array['data']) && is_array($xport_array['data'])) {
 			for ($i = 1; $i <= $xport_array['meta']['columns']; $i++) {
 				$data .= ',"' . $row['col' . $i] . '"';
 			}
-			print $data . "\n";
+			$output .= $data . "\n";
 			$j++;
 		}
+
+		// Full UTF-8 Output
+		print "\xEF\xBB\xBF";
+		print $output;
 	} else {
 		$j = 1;
 		foreach($xport_array['data'] as $row) {
 			print "<tr><td class='left'>" . date('Y-m-d H:i:s', (isset($row['timestamp']) ? $row['timestamp'] : $xport_array['meta']['start'] + $j*$xport_array['meta']['step'])) . "</td>";
+
 			for ($i = 1; $i <= $xport_array['meta']['columns']; $i++) {
 				if ($row['col' . $i] > 1) {
-					print "<td class='right'>" . trim(number_format_i18n(round($row['col' . $i],3))) . '</td>';
+					print "<td class='right'>" . trim(number_format_i18n(round($row['col' . $i],3),2,$graph_info['base_value'])) . '</td>';
 				} elseif($row['col' . $i] == 0) {
 					print "<td class='right'>-</td>";
 				} else {
 					print "<td class='right'>" . round($row['col' . $i],4) . '</td>';
 				}
 			}
+
 			print "</tr>\n";
 			$j++;
 		}
 
-		print "</table></div>\n";
+		print "<tr><td>\n";
 
 		?>
 		<script type='text/javascript'>
@@ -265,6 +318,8 @@ if (isset($xport_array['data']) && is_array($xport_array['data'])) {
 		}
 		</script>
 		<?php
+
+		print "</td></tr></table></div>\n";
 	}
 }
 
