@@ -114,6 +114,7 @@ switch (get_request_var('action')) {
 		get_filter_request_var('host_id');
 
 		host_reload_query();
+		raise_message('query_reloaded', __('Data Query Re-indexed.'), MESSAGE_LEVEL_INFO);
 
 		header('Location: host.php?header=false&action=edit&id=' . get_request_var('host_id'));
 		break;
@@ -121,6 +122,7 @@ switch (get_request_var('action')) {
 		get_filter_request_var('host_id');
 
 		host_reload_query();
+		raise_message('query_reloaded', __('Device Data Query Re-indexed.  Verbose output displayed.'), MESSAGE_LEVEL_INFO);
 
 		header('Location: host.php?header=' . (isset_request_var('header') && get_nfilter_request_var('header') == 'true' ? 'true':'false') . '&action=edit&id=' . get_request_var('host_id') . '&display_dq_details=true');
 		break;
@@ -139,11 +141,26 @@ switch (get_request_var('action')) {
 		break;
 	case 'enable_debug':
 		enable_device_debug(get_filter_request_var('host_id'));
+		raise_message('enable_debug', __('Device Debugging Enabled for Device.'), MESSAGE_LEVEL_INFO);
+
 		header('Location: host.php?header=false&action=edit&id=' . get_request_var('host_id'));
 
 		break;
 	case 'disable_debug':
 		disable_device_debug(get_filter_request_var('host_id'));
+		raise_message('disable_debug', __('Device Debugging Disabled for Device.'), MESSAGE_LEVEL_INFO);
+
+		header('Location: host.php?header=false&action=edit&id=' . get_request_var('host_id'));
+
+		break;
+	case 'repopulate':
+		if (get_filter_request_var('host_id') > 0) {
+			push_out_host(get_request_var('host_id'));
+			raise_message('repopulate_message', __('Poller Cache for Device Refreshed.'), MESSAGE_LEVEL_INFO);
+		} else {
+			raise_message('repopulate_error', __('ERROR: Invalid Device ID.'), MESSAGE_LEVEL_ERROR);
+		}
+
 		header('Location: host.php?header=false&action=edit&id=' . get_request_var('host_id'));
 
 		break;
@@ -192,15 +209,16 @@ function add_tree_names_to_actions_array() {
 
 	if (cacti_sizeof($trees)) {
 		foreach ($trees as $tree) {
-			$device_actions['tr_' . $tree['id']] = 'Place on a Tree (' . $tree['name'] . ')';
+			$device_actions['tr_' . $tree['id']] = __esc('Place on a Tree (%s)', $tree['name']);
 		}
 	}
 }
 
 function get_site_locations() {
 	$return  = array();
-	$term    = get_request_var('term');
+	$term    = get_nfilter_request_var('term');
 	$host_id = $_SESSION['cur_device_id'];
+
 	$site_id = db_fetch_cell_prepared('SELECT site_id
 		FROM host
 		WHERE id = ?',
@@ -354,7 +372,7 @@ function form_actions() {
 					<p>" . __('Click \'Continue\' to enable the following Device(s).') . "</p>
 					<div class='itemlist'><ul>$host_list</ul></div>
 				</td>
-			</tr>\n";
+			</tr>";
 
 			$save_html = "<input type='button' class='ui-button ui-corner-all ui-widget' value='" . __esc('Cancel') . "' onClick='cactiReturnTo()'>&nbsp;<input type='submit' class='ui-button ui-corner-all ui-widget' value='" . __esc('Continue') . "' title='" . __esc('Enable Device(s)') . "'>";
 		} elseif (get_nfilter_request_var('drp_action') == '3') { // Disable Devices
@@ -363,7 +381,7 @@ function form_actions() {
 					<p>" . __('Click \'Continue\' to disable the following Device(s).') . "</p>
 					<div class='itemlist'><ul>$host_list</ul></div>
 				</td>
-				</tr>\n";
+				</tr>";
 
 			$save_html = "<input type='button' class='ui-button ui-corner-all ui-widget' value='" . __esc('Cancel') . "' onClick='cactiReturnTo()'>&nbsp;<input type='submit' class='ui-button ui-corner-all ui-widget' value='" . __esc('Continue') . "' title='" . __esc('Disable Device(s)') ."'>";
 		} elseif (get_nfilter_request_var('drp_action') == '4') { // Change Device options
@@ -372,7 +390,7 @@ function form_actions() {
 					<p>" . __('Click \'Continue\' to change the Device options below for multiple Device(s).  Please check the box next to the fields you want to update, and then fill in the new value.') . "</p>
 					<div class='itemlist'><ul>$host_list</ul></div>
 				</td>
-			</tr>\n";
+			</tr>";
 
 			$form_array = array();
 
@@ -384,6 +402,7 @@ function form_actions() {
 					($field_name == 'host_template_id') ||
 					($field_name == 'availability_method') ||
 					($field_name == 'device_threads') ||
+					($field_name == 'location') ||
 					($field_name == 'max_oids')) {
 					$form_array += array($field_name => $fields_host_edit[$field_name]);
 
@@ -400,6 +419,11 @@ function form_actions() {
 						'class' => 'ui-state-disabled',
 						'value' => ''
 					);
+
+					$form_array['location']['method']     = 'textbox';
+					$form_array['location']['default']    = '';
+					$form_array['location']['size']       = '20';
+					$form_array['location']['max_length'] = '40';
 				}
 			}
 
@@ -421,7 +445,7 @@ function form_actions() {
 					<p>" . __('Click \'Continue\' to clear the counters for the following Device(s).') . "</p>
 					<div class='itemlist'><ul>$host_list</ul></div>
 				</td>
-			</tr>\n";
+			</tr>";
 
 			$save_html = "<input type='button' class='ui-button ui-corner-all ui-widget' value='" . __esc('Cancel') . "' onClick='cactiReturnTo()'>&nbsp;<input type='submit' class='ui-button ui-corner-all ui-widget' value='" . __esc('Continue') . "' title='" . __esc('Clear Statistics on Device(s)') . "'>";
 		} elseif (get_nfilter_request_var('drp_action') == '7') { // sync device template
@@ -430,21 +454,21 @@ function form_actions() {
 					<p>" . __('Click \'Continue\' to Synchronize the following Device(s) to their Device Template.') . "</p>
 					<div class='itemlist'><ul>$host_list</ul></div>
 				</td>
-				</tr>\n";
+				</tr>";
 
 			$save_html = "<input type='button' class='ui-button ui-corner-all ui-widget' value='" . __esc('Cancel') . "' onClick='cactiReturnTo()'>&nbsp;<input type='submit' class='ui-button ui-corner-all ui-widget' value='" . __esc('Continue') . "' title='" . __esc('Synchronize Device(s)') ."'>";
 		} elseif (get_request_var('drp_action') == '1') { // Delete
 			print "<tr>
 				<td class='textArea'>
 					<p>" . __('Click \'Continue\' to delete the following Device(s).') . "</p>
-					<div class='itemlist'><ul>$host_list</ul></div>\n";
+					<div class='itemlist'><ul>$host_list</ul></div>";
 
 					form_radio_button('delete_type', '2', '1', __('Leave all Graph(s) and Data Source(s) untouched.  Data Source(s) will be disabled however.'), '1'); print '<br>';
 					form_radio_button('delete_type', '2', '2', __('Delete all associated Graph(s) and Data Source(s).'), '1'); print '<br>';
 
 			print "</td></tr>
 				</td>
-			</tr>\n";
+			</tr>";
 
 			$save_html = "<input type='button' class='ui-button ui-corner-all ui-widget' value='" . __esc('Cancel') . "' onClick='cactiReturnTo()'>&nbsp;<input type='submit' class='ui-button ui-corner-all ui-widget' value='" . __esc('Continue') . "' title='" . __esc('Delete Device(s)') . "'>";
 		} elseif (preg_match('/^tr_([0-9]+)$/', get_request_var('drp_action'), $matches)) { // place on tree
@@ -452,13 +476,13 @@ function form_actions() {
 				<td class='textArea'>
 					<p>" . __('Click \'Continue\' to place the following Device(s) under the branch selected below.') . "</p>
 					<div class='itemlist'><ul>$host_list</ul></div>
-					<p>" . __('Destination Branch:') . "<br>\n";
+					<p>" . __('Destination Branch:') . "<br>";
 					grow_dropdown_tree($matches[1], '0', 'tree_item_id', '0');
 
 			print "</p>
 				</td>
 			</tr>
-			<input type='hidden' name='tree_id' value='" . $matches[1] . "'>\n";
+			<input type='hidden' name='tree_id' value='" . $matches[1] . "'>";
 
 			$save_html = "<input type='button' class='ui-button ui-corner-all ui-widget' value='" . __esc('Cancel') . "' onClick='cactiReturnTo()'>&nbsp;<input type='submit' class='ui-button ui-corner-all ui-widget' value='" . __esc('Continue') . "' title='" . __esc('Place Device(s) on Tree') . "'>";
 		} elseif (get_request_var('drp_action') == 6) { // automation
@@ -467,7 +491,7 @@ function form_actions() {
 					<p>" . __('Click \'Continue\' to apply Automation Rules to the following Devices(s).'). "</p>
 					<div class='itemlist'><ul>$host_list</ul></div>
 				</td>
-			</tr>\n";
+			</tr>";
 
 			$save_html = "<input type='button' class='ui-button ui-corner-all ui-widget' value='" . __esc('Cancel'). "' onClick='cactiReturnTo()'>&nbsp;<input type='submit' class='ui-button ui-corner-all ui-widget' value='" . __esc('Continue') . "' title='" . __esc('Run Automation on Device(s)') . "'>";
 		} else {
@@ -492,7 +516,7 @@ function form_actions() {
 			<input type='hidden' name='drp_action' value='" . html_escape(get_nfilter_request_var('drp_action')) . "'>
 			$save_html
 		</td>
-	</tr>\n";
+	</tr>";
 
 	html_end_box();
 
@@ -609,6 +633,7 @@ function host_edit() {
 
 	$header_label = __('Device [new]');
 	$debug_link   = '';
+	$repop_link   = '';
 	if (!isempty_request_var('id')) {
 		$_SESSION['cur_device_id'] = get_request_var('id');
 
@@ -624,6 +649,8 @@ function host_edit() {
 			} else {
 				$debug_link = "<span class='linkMarker'>*</span><a class='hyperLink' href='" . html_escape('host.php?action=enable_debug&host_id=' . $host['id']) . "'>" . __('Enable Device Debug') . "</a><br>";
 			}
+
+			$repop_link = "<span class='linkMarker'>*</span><a class='hyperLink' href='" . html_escape('host.php?action=repopulate&host_id=' . $host['id']) . "'>" . __('Repopulate Poller Cache') . "</a><br>";
 		}
 	} else {
 		$_SESSION['cur_device_id'] = 0;
@@ -641,6 +668,7 @@ function host_edit() {
 					<span class='linkMarker'>*</span><a class='hyperLink' href='<?php print html_escape('graphs_new.php?reset=true&host_id=' . $host['id']);?>'><?php print __('Create Graphs for this Device');?></a><br>
 					<span class='linkMarker'>*</span><a class='hyperLink' href='<?php print html_escape('host.php?action=reindex&host_id=' . $host['id']);?>'><?php print __('Re-Index Device');?></a><br>
 					<?php print $debug_link;?>
+					<?php print $repop_link;?>
 					<span class='linkMarker'>*</span><a class='hyperLink' href='<?php print html_escape('data_sources.php?reset=true&host_id=' . $host['id'] . '&ds_rows=30&filter=&template_id=-1&method_id=-1&page=1');?>'><?php print __('Data Source List');?></a><br>
 					<span class='linkMarker'>*</span><a class='hyperLink' href='<?php print html_escape('graphs.php?reset=true&host_id=' . $host['id'] . '&graph_rows=30&filter=&template_id=-1&page=1');?>'><?php print __('Graph List');?></a>
 					<?php api_plugin_hook('device_edit_top_links'); ?>
@@ -675,32 +703,6 @@ function host_edit() {
 
 	device_javascript();
 
-	if ((isset_request_var('display_dq_details')) && (isset($_SESSION['debug_log']['data_query']))) {
-		$dbg_copy_uid = generate_hash();
-		?>
-		<div id='dqdebug' class='cactiTable'>
-			<div id='clipboardHeader<?php print $dbg_copy_uid;?>'>
-				<div class='cactiTableTitle'>
-					<span style='padding:3px;'><?php print __('Data Query Debug Information');?></span>
-				</div>
-				<div class='cactiTableButton'>
-					<span>
-						<a class='linkCopyDark cactiTableCopy' id='copyToClipboard<?php print $dbg_copy_uid;?>'><?php print __('Copy');?></a>
-						<a id='dbghide' class='fa fa-times' href='#'><?php print __('Hide');?></a>
-					</span>
-				</div>
-				<table class='cactiTable' id='clipboardData<?php print $dbg_copy_uid;?>'>
-					<tr class='tableRow'>
-						<td class='debug'>
-							<span><?php print debug_log_return('data_query');?></span>
-						</td>
-					</tr>
-				</table>
-			</div>
-		</div>
-		<?php
-	}
-
 	if (!empty($host['id'])) {
 		html_start_box(__('Associated Graph Templates'), '100%', '', '3', 'center', '');
 
@@ -800,6 +802,32 @@ function host_edit() {
 		<?php
 		html_end_box();
 
+		if ((isset_request_var('display_dq_details')) && (isset($_SESSION['debug_log']['data_query']))) {
+			$dbg_copy_uid = generate_hash();
+			?>
+			<div id='dqdebug' class='cactiTable'>
+				<div id='clipboardHeader<?php print $dbg_copy_uid;?>'>
+					<div class='cactiTableTitle'>
+						<span style='padding:3px;'><?php print __('Data Query Debug Information');?></span>
+					</div>
+					<div class='cactiTableButton'>
+						<span>
+							<a class='linkCopyDark cactiTableCopy' id='copyToClipboard<?php print $dbg_copy_uid;?>'><?php print __('Copy');?></a>
+							<a id='dbghide' class='fa fa-times' href='#'><?php print __('Hide');?></a>
+						</span>
+					</div>
+					<table class='cactiTable' id='clipboardData<?php print $dbg_copy_uid;?>'>
+						<tr class='tableRow'>
+							<td class='debug'>
+								<span><?php print debug_log_return('data_query');?></span>
+							</td>
+						</tr>
+					</table>
+				</div>
+			</div>
+			<?php
+		}
+
 		html_start_box(__('Associated Data Queries'), '100%', '', '3', 'center', '');
 
 		html_header(
@@ -935,10 +963,10 @@ function device_reindex_methods($item, $host) {
 	foreach($reindex_types as $key => $type) {
 		if ($selectedTheme != 'classic') {
 			if ($i == 0) {
-				print "<fieldset class='reindex_methods'>\n";
+				print "<fieldset class='reindex_methods'>";
 			}
-			print "<input name='reindex_radio_" . $item['id'] . "' type='radio' data-device-id='" . $host['id'] . "' data-query-id='" . $item['id'] . "' data-reindex-method='" . $key . "' id='reindex_" . $item['id'] . "_" . $key . "'" . ($item['reindex_method'] == $key ? ' checked="checked"':'') . " />\n";
-			print "<label title='" . html_escape($reindex_types_tips[$key]) . "' for='reindex_" . $item['id'] . "_" . $key . "'>" . $type . "</label>\n";
+			print "<input name='reindex_radio_" . $item['id'] . "' type='radio' data-device-id='" . $host['id'] . "' data-query-id='" . $item['id'] . "' data-reindex-method='" . $key . "' id='reindex_" . $item['id'] . "_" . $key . "'" . ($item['reindex_method'] == $key ? ' checked="checked"':'') . " />";
+			print "<label title='" . html_escape($reindex_types_tips[$key]) . "' for='reindex_" . $item['id'] . "_" . $key . "'>" . $type . "</label>";
 		} else {
 			print $reindex_types[$item['reindex_method']];
 			break;
@@ -948,7 +976,7 @@ function device_reindex_methods($item, $host) {
 	}
 
 	if ($selectedTheme != 'classic') {
-		print "</fieldset>\n";
+		print "</fieldset>";
 	}
 }
 
@@ -1135,6 +1163,15 @@ function device_javascript() {
 		setPing();
 	}
 
+	function hostPageLoad(strURL) {
+		var scrollTop = $(window).scrollTop();
+		$.get(strURL, function(data) {
+			$('#main').html(data);
+			applySkin();
+			$(window).scrollTop(scrollTop);
+		});
+	}
+
 	$(function() {
 		// Need to set this for global snmpv3 functions to remain sane between edits
 		snmp_security_initialized = false;
@@ -1168,23 +1205,22 @@ function device_javascript() {
 		$('[id^="reload"]').click(function(data) {
 			$(this).addClass('fa-spin');
 			strURL = 'host.php?action=query_reload&id='+$(this).attr('data-id')+'&host_id='+$('#id').val()+'&nostate=true';
-			loadPageNoHeader(strURL, true);
+			hostPageLoad(strURL);
 		});
 
 		$('[id^="verbose"]').click(function(data) {
-			$(this).addClass('fa-spin');
-			strURL = 'host.php?action=query_verbose&id='+$(this).attr('data-id')+'&host_id='+$('#id').val()+'&nostate=true';
+			var strURL = 'host.php?action=query_verbose&id='+$(this).attr('data-id')+'&host_id='+$('#id').val()+'&nostate=true';
 			loadPageNoHeader(strURL, true);
 		});
 
 		$('[id^="remove"]').click(function(data) {
-			strURL = 'host.php?action=query_remove&id='+$(this).attr('data-id')+'&host_id='+$('#id').val()+'&nostate=true';
-			loadPageNoHeader(strURL, true);
+			var strURL = 'host.php?action=query_remove&id='+$(this).attr('data-id')+'&host_id='+$('#id').val()+'&nostate=true';
+			hostPageLoad(strURL);
 		});
 
 		$('[id^="gtremove"]').click(function(data) {
 			strURL = 'host.php?action=gt_remove&id='+$(this).attr('data-id')+'&host_id='+$('#id').val()+'&nostate=true';
-			loadPageNoHeader(strURL, true);
+			hostPageLoad(strURL);
 		});
 
 		$('#add_dq').click(function() {
@@ -1210,6 +1246,11 @@ function device_javascript() {
 			$('#dqdebug').empty().fadeOut('fast');
 		});
 
+		if ($('#dbghide').length) {
+			var dbgloc = parseInt($('#dbghide').offset().top - $('.breadCrumbBar').outerHeight() - $('.cactiPageHead').outerHeight());
+			$('.cactiConsoleContentArea').scrollTop(dbgloc);
+		}
+
 		$('[id$="spacer"]').click(function() {
 			changeHostForm();
 		});
@@ -1229,7 +1270,6 @@ function device_javascript() {
 			.done(function(data) {
 				$('#ping_results').html(data);
 				hostInfoHeight = $('.hostInfoHeader').height();
-				$('#navigation_right, #main').scrollTop(0);
 			})
 			.fail(function(data) {
 				getPresentHTTPError(data);
@@ -1270,6 +1310,11 @@ function host_validate_vars() {
 			'pageset' => true,
 			'default' => ''
 			),
+		'location' => array(
+			'filter' => FILTER_CALLBACK,
+			'default' => '-1',
+			'options' => array('options' => 'sanitize_search_string')
+			),
 		'sort_column' => array(
 			'filter' => FILTER_CALLBACK,
 			'default' => 'description',
@@ -1319,6 +1364,12 @@ function get_device_records(&$total_rows, $rows) {
 		$sql_where = "WHERE deleted = ''";
 	}
 
+	if (get_request_var('location') == __('Undefined') || get_request_var('location') == '') {
+		$sql_where .= ($sql_where != '' ? ' AND':' WHERE') . ' IFNULL(host.location,"") = ""';
+	} elseif (get_request_var('location') != '-1') {
+		$sql_where .= ($sql_where != '' ? ' AND':' WHERE') . ' host.location = ' . db_qstr(get_request_var('location'));;
+	}
+
 	if (get_request_var('host_status') == '-1') {
 		/* Show all items */
 	} elseif (get_request_var('host_status') == '-2') {
@@ -1327,7 +1378,7 @@ function get_device_records(&$total_rows, $rows) {
 		$sql_where .= ($sql_where != '' ? " AND host.disabled=''" : " WHERE host.disabled=''");
 	} elseif (get_request_var('host_status') == '-4') {
 		$sql_where .= ($sql_where != '' ? " AND (host.status!='3' OR host.disabled='on')" : " WHERE (host.status!='3' OR host.disabled='on')");
-	}else {
+	} else {
 		$sql_where .= ($sql_where != '' ? ' AND (host.status=' . get_request_var('host_status') . " AND host.disabled = '')" : 'where (host.status=' . get_request_var('host_status') . " AND host.disabled = '')");
 	}
 
@@ -1407,10 +1458,12 @@ function host() {
 	<script type='text/javascript'>
 
 	function applyFilter() {
-		strURL  = 'host.php?host_status=' + $('#host_status').val();
+		strURL  = 'host.php';
+		strURL += '?host_status=' + $('#host_status').val();
 		strURL += '&host_template_id=' + $('#host_template_id').val();
 		strURL += '&site_id=' + $('#site_id').val();
 		strURL += '&poller_id=' + $('#poller_id').val();
+		strURL += '&location=' + $('#location').val();
 		strURL += '&rows=' + $('#rows').val();
 		strURL += '&filter=' + $('#filter').val();
 		strURL += '&header=false';
@@ -1425,10 +1478,11 @@ function host() {
 	function exportRecords() {
 		strURL = 'host.php?action=export';
 		document.location = strURL;
+		Pace.stop();
 	}
 
 	$(function() {
-		$('#rows, #site_id, #poller_id, #host_template_id, #host_status').change(function() {
+		$('#rows, #site_id, #poller_id, #location, #host_template_id, #host_status').change(function() {
 			applyFilter();
 		});
 
@@ -1479,7 +1533,7 @@ function host() {
 
 							if (cacti_sizeof($sites)) {
 								foreach ($sites as $site) {
-									print "<option value='" . $site['id'] . "'"; if (get_request_var('site_id') == $site['id']) { print ' selected'; } print '>' . html_escape($site['name']) . "</option>\n";
+									print "<option value='" . $site['id'] . "'"; if (get_request_var('site_id') == $site['id']) { print ' selected'; } print '>' . html_escape($site['name']) . "</option>";
 								}
 							}
 							?>
@@ -1496,7 +1550,7 @@ function host() {
 
 							if (cacti_sizeof($pollers)) {
 								foreach ($pollers as $poller) {
-									print "<option value='" . $poller['id'] . "'"; if (get_request_var('poller_id') == $poller['id']) { print ' selected'; } print '>' . html_escape($poller['name']) . "</option>\n";
+									print "<option value='" . $poller['id'] . "'"; if (get_request_var('poller_id') == $poller['id']) { print ' selected'; } print '>' . html_escape($poller['name']) . "</option>";
 								}
 							}
 							?>
@@ -1514,7 +1568,33 @@ function host() {
 
 							if (cacti_sizeof($host_templates)) {
 								foreach ($host_templates as $host_template) {
-									print "<option value='" . $host_template['id'] . "'"; if (get_request_var('host_template_id') == $host_template['id']) { print ' selected'; } print '>' . html_escape($host_template['name']) . "</option>\n";
+									print "<option value='" . $host_template['id'] . "'"; if (get_request_var('host_template_id') == $host_template['id']) { print ' selected'; } print '>' . html_escape($host_template['name']) . "</option>";
+								}
+							}
+							?>
+						</select>
+					</td>
+					<td>
+						<?php print __('Location');?>
+					</td>
+					<td>
+						<select id='location'>
+							<option value='-1'<?php if (get_request_var('location') == '-1') {?> selected<?php }?>><?php print __('All');?></option>
+							<?php
+
+							if (get_request_var('site_id') >= '0') {
+								$sql_where = 'WHERE site_id = ' . db_qstr(get_request_var('site_id'));
+							} else {
+								$sql_where = '';
+							}
+							$locations = db_fetch_assoc("SELECT DISTINCT IF(IFNULL(host.location,'') = '', '" . __('Undefined') . "', location) AS location
+								FROM host
+								$sql_where
+								ORDER BY location");
+
+							if (cacti_sizeof($locations)) {
+								foreach ($locations as $l) {
+									print "<option value='" . $l['location'] . "'"; if (get_request_var('location') == $l['location']) { print ' selected'; } print '>' . html_escape($l['location']) . "</option>";
 								}
 							}
 							?>
@@ -1561,7 +1641,7 @@ function host() {
 							<?php
 							if (cacti_sizeof($item_rows)) {
 								foreach ($item_rows as $key => $value) {
-									print "<option value='" . $key . "'"; if (get_request_var('rows') == $key) { print ' selected'; } print '>' . html_escape($value) . "</option>\n";
+									print "<option value='" . $key . "'"; if (get_request_var('rows') == $key) { print ' selected'; } print '>' . html_escape($value) . "</option>";
 								}
 							}
 							?>
