@@ -79,8 +79,6 @@ $url_path = '/cacti/';
 /* disable log rotation setting */
 $disable_log_rotation = false;
 
-/* allow upto 5000 items to be selected */
-ini_set('max_input_vars', '5000');
 $config = array();
 
 /* Include configuration, or use the defaults */
@@ -176,7 +174,6 @@ $no_http_header_files = array(
 	'query_host_cpu.php',
 	'query_host_partitions.php',
 	'rebuild_poller_cache.php',
-	'remote_agent.php',
 	'repair_database.php',
 	'script_server.php',
 	'snmpagent_mibcachechild.php',
@@ -222,7 +219,7 @@ if (empty($database_port)) {
 }
 
 /* set URL path */
-if (! isset($url_path)) {
+if (!isset($url_path)) {
 	$url_path = '';
 }
 $config['url_path'] = $url_path;
@@ -391,32 +388,56 @@ if ($config['is_web']) {
 
 	/* set the maximum post size */
 	ini_set('post_max_size', '8M');
-	ini_set('max_input_vars', '5000');
-	ini_set('session.cookie_httponly', '1');
-	if (!empty($_SERVER['HTTPS']) && $_SERVER['HTTPS'] != 'off') {
-		ini_set('session.cookie_secure', '1');
+
+	/* add additional cookie directives */
+	ini_set('session.cookie_httponly', true);
+	ini_set('session.cookie_path', $config['url_path']);
+	ini_set('session.use_strict_mode', true);
+
+	$options = array(
+		'cookie_httponly' => true,
+		'cookie_path'     => $config['url_path'],
+		'use_strict_mode' => true
+	);
+
+	if (isset($cacti_cookie_domain) && $cacti_cookie_domain != '') {
+		ini_set('session.cookie_domain', $cacti_cookie_domain);
+		$options['cookie_domain'] = $cacti_cookie_domain;
 	}
 
+	// SameSite php7.3+ behavior
+	if (version_compare(PHP_VERSION, '7.3', '>=')) {
+		ini_set('session.cookie_samesite', 'Strict');
+		$options['cookie_samesite'] = 'Strict';
+	}
+
+	if (!empty($_SERVER['HTTPS']) && $_SERVER['HTTPS'] != 'off') {
+		ini_set('session.cookie_secure', true);
+		$options['cookie_secure'] = true;
+	}
+
+	$config['cookie_options']     = $options;
+	$config['cacti_session_name'] = $cacti_session_name;
+
 	/* we don't want these pages cached */
-	header('Expires: Mon, 26 Jul 1997 05:00:00 GMT');
 	header('Last-Modified: ' . gmdate('D, d M Y H:i:s') . ' GMT');
-	header('Cache-Control: no-store, no-cache, must-revalidate');
-	header('Cache-Control: post-check=0, pre-check=0', false);
-	header('Pragma: no-cache');
 	header('X-Frame-Options: SAMEORIGIN');
 
 	/* increased web hardening */
-	header("Content-Security-Policy: default-src *; img-src 'self' data: blob:; style-src 'self' 'unsafe-inline'; script-src 'self' 'unsafe-inline'; frame-ancestors 'self';");
+	$script_policy = read_config_option('content_security_policy_script');
+	if ($script_policy != '0' && $script_policy != '') {
+		$script_policy = "'$script_policy'";
+	}
+	$alternates = read_config_option('content_security_alternate_sources');
+
+	header("Content-Security-Policy: default-src *; img-src 'self' $alternates data: blob:; style-src 'self' 'unsafe-inline' $alternates; script-src 'self' $script_policy 'unsafe-inline' $alternates; frame-ancestors 'self'; worker-src 'self'");
 
 	/* prevent IE from silently rejects cookies sent from third party sites. */
 	header('P3P: CP="CAO PSA OUR"');
+	header('Cache-Control: no-store, no-cache, must-revalidate');
+	header('Cache-Control: max-age=31536000');
 
-	/* initialize php session */
-	if (!function_exists('session_name')) {
-		die('PHP Session Management is missing, please install PHP Session module');
-	}
-	session_name($cacti_session_name);
-	if (!session_id()) session_start();
+	cacti_session_start();
 
 	/* we never run with magic quotes on */
 	if (version_compare(PHP_VERSION, '5.4', '<=')) {
@@ -442,8 +463,7 @@ if ($config['is_web']) {
 		$_SESSION['cacti_cwd'] = $config['base_path'];
 	} else {
 		if ($_SESSION['cacti_cwd'] != $config['base_path']) {
-			session_unset();
-			session_destroy();
+			cacti_session_destroy();
 		}
 	}
 }
@@ -503,6 +523,20 @@ if ($config['is_web']) {
 	if (isset_request_var('csrf_timeout')) {
 		raise_message('csrf_ptimeout');
 	}
+
+	/* check for save actions using GET */
+	if (isset_request_var('action')) {
+		$action = get_nfilter_request_var('action');
+
+		$bad_actions = array('save', 'update_data', 'changepassword');
+
+		foreach($bad_actions as $bad) {
+			if ($action == $bad && !isset($_POST['__csrf_magic'])) {
+				cacti_log('WARNING: Attempt to use GET method for POST operations from IP ' . get_client_addr(), false, 'WEBUI');
+				exit;
+			}
+		}
+	}
 }
 
 api_plugin_hook('config_insert');
