@@ -180,7 +180,7 @@ if (empty($l10n_handler)) {
 		$l10n_handler = CACTI_LANGUAGE_HANDLER_OSCAROTERO;
 	} elseif (file_exists($config['include_path'] . '/vendor/phpgettext/streams.php')) {
 		$l10n_handler = CACTI_LANGUAGE_HANDLER_PHPGETTEXT;
-	} elseif (file_exists($config['include_path'] . '/vendor/motanslator/src/Translator.php')) {
+	} elseif (file_exists($config['include_path'] . '/vendor/motranslator/src/Translator.php')) {
 		$l10n_handler = CACTI_LANGUAGE_HANDLER_MOTRANSLATOR;
 	}
 }
@@ -705,8 +705,16 @@ function number_format_i18n($number, $decimals = 0, $baseu = 1024) {
 	setlocale(LC_ALL, $cacti_locale);
 	$locale = localeconv();
 
+	if (!isset($locale['decimal_point']) || $locale['decimal_point'] == '') {
+		$locale['decimal_point'] = '.';
+	}
+
+	if (!isset($locale['thousands_sep']) || $locale['thousands_sep'] == '') {
+		$locale['thousands_sep'] = ',';
+	}
+
 	if ($decimals == -1) {
-		$number =  number_format($number, $decimals, $locale['decimal_point'], $locale['thousands_sep']);
+		$number =  number_format($number, 0, $locale['decimal_point'], $locale['thousands_sep']);
 	} elseif ($number>=pow($baseu, 4)) {
 		$number =  number_format($number/pow($baseu, 4), $decimals, $locale['decimal_point'], $locale['thousands_sep']) . __(' T');
 	} elseif ($number>=pow($baseu, 3)) {
