@@ -622,11 +622,8 @@ function boost_process_poller_output($local_data_id = '', $rrdtool_pipe = '') {
 	/* install the boost error handler */
 	set_error_handler('boost_error_handler');
 
-	/* we can simplify the delete process if only one local_data_id */
-	$orig_local_data_id   = $local_data_id;
-
 	/* aquire lock in order to prevent race conditions */
-	while (!db_fetch_cell("SELECT GET_LOCK('boost.single_ds.$orig_local_data_id', 1)")) {
+	while (!db_fetch_cell("SELECT GET_LOCK('boost.single_ds.$local_data_id', 1)")) {
 		usleep(50000);
 	}
 
@@ -640,16 +637,18 @@ function boost_process_poller_output($local_data_id = '', $rrdtool_pipe = '') {
 
 	if (cacti_count($arch_tables)) {
 		foreach($arch_tables as $table) {
-			$rows = db_fetch_cell('SELECT COUNT(local_data_id
+			$rows = db_fetch_cell_prepared('SELECT COUNT(local_data_id)
 				FROM ' . $table['name'] . '
-				WHERE local_data_id = ' . $local_data_id, '', false);
+				WHERE local_data_id = ?',
+				array($local_data_id), '', false);
 
 			if (db_table_exists($table['name']) && is_numeric($rows) && intval($rows) > 0) {
 				if ($query_string != '') {
 					$query_string .= ' UNION ';
 				}
 
-				$query_string .= ' (SELECT local_data_id, UNIX_TIMESTAMP(time) AS timestamp, rrd_name, output
+				$query_string .= ' (SELECT local_data_id,
+					UNIX_TIMESTAMP(time) AS timestamp, rrd_name, output
 					FROM ' . $table['name'] . "
 					WHERE local_data_id = $local_data_id) ";
 			}
@@ -662,9 +661,10 @@ function boost_process_poller_output($local_data_id = '', $rrdtool_pipe = '') {
 
 	$timestamp = time();
 
-	$query_string .= " (SELECT local_data_id, UNIX_TIMESTAMP(time) AS timestamp, rrd_name, output
+	$query_string .= " (SELECT local_data_id,
+		UNIX_TIMESTAMP(time) AS timestamp, rrd_name, output
 		FROM poller_output_boost
-		WHERE local_data_id = $orig_local_data_id
+		WHERE local_data_id = $local_data_id
 		AND time < FROM_UNIXTIME($timestamp))";
 
 	$query_string .= ' ORDER BY timestamp ASC, rrd_name ASC ';
@@ -709,7 +709,6 @@ function boost_process_poller_output($local_data_id = '', $rrdtool_pipe = '') {
 		/* initialize some variables */
 		$rrd_tmpl     = '';
 		$rrd_path     = '';
-		$initial_time = 0;
 		$outlen       = 0;
 
 		boost_timer('results_cycle', BOOST_TIMER_START);
@@ -730,7 +729,7 @@ function boost_process_poller_output($local_data_id = '', $rrdtool_pipe = '') {
 				/* update the rrd for the previous local_data_id */
 				if ($vals_in_buffer) {
 					boost_timer('rrdupdate', BOOST_TIMER_START);
-					$return_value = boost_rrdtool_function_update($local_data_id, $rrd_path, $rrd_tmpl, $initial_time, $outbuf, $rrdtool_pipe);
+					$return_value = boost_rrdtool_function_update($local_data_id, $rrd_path, $rrd_tmpl, $outbuf, $rrdtool_pipe);
 					boost_timer('rrdupdate', BOOST_TIMER_END);
 
 					$vals_in_buffer = 0;
@@ -754,7 +753,6 @@ function boost_process_poller_output($local_data_id = '', $rrdtool_pipe = '') {
 
 				$local_data_id  = $item['local_data_id'];
 				$time           = $item['timestamp'];
-				$initial_time   = $time;
 
 				if ($time < $last_update && cacti_version_compare(get_rrdtool_version(), '1.5', '<')) {
 					$output = '';
@@ -778,7 +776,7 @@ function boost_process_poller_output($local_data_id = '', $rrdtool_pipe = '') {
 			if ($time != $item['timestamp']) {
 				if ($outlen > $upd_string_len) {
 					boost_timer('rrdupdate', BOOST_TIMER_START);
-					$return_value = boost_rrdtool_function_update($local_data_id, $rrd_path, $rrd_tmpl, $initial_time, $outbuf, $rrdtool_pipe);
+					$return_value = boost_rrdtool_function_update($local_data_id, $rrd_path, $rrd_tmpl, $outbuf, $rrdtool_pipe);
 					boost_timer('rrdupdate', BOOST_TIMER_END);
 
 					$outbuf         = '';
@@ -878,7 +876,7 @@ function boost_process_poller_output($local_data_id = '', $rrdtool_pipe = '') {
 		/* process the last rrdupdate if applicable */
 		if ($vals_in_buffer) {
 			boost_timer('rrdupdate', BOOST_TIMER_START);
-			$return_value = boost_rrdtool_function_update($local_data_id, $rrd_path, $rrd_tmpl, $initial_time, $outbuf, $rrdtool_pipe);
+			$return_value = boost_rrdtool_function_update($local_data_id, $rrd_path, $rrd_tmpl, $outbuf, $rrdtool_pipe);
 			boost_timer('rrdupdate', BOOST_TIMER_END);
 
 			/* check return status for delete operation */
@@ -894,16 +892,18 @@ function boost_process_poller_output($local_data_id = '', $rrdtool_pipe = '') {
 
 		if (cacti_count($arch_tables)) {
 			foreach($arch_tables as $table) {
-				db_execute_prepared('DELETE FROM ' . $table['name'] . '
+				db_execute_prepared('DELETE IGNORE FROM ' . $table['name'] . '
 					WHERE local_data_id = ?',
 					array($local_data_id));
 			}
 		}
 
-		db_execute_prepared('DELETE FROM poller_output_boost
-			WHERE local_data_id = ?
-			AND time < FROM_UNIXTIME(?)',
-			array($local_data_id, $timestamp));
+		if (cacti_sizeof($results)) {
+			db_execute_prepared('DELETE FROM poller_output_boost
+				WHERE local_data_id = ?
+				AND time < FROM_UNIXTIME(?)',
+				array($local_data_id, $timestamp));
+		}
 
 		boost_timer('delete', BOOST_TIMER_END);
 
@@ -912,7 +912,7 @@ function boost_process_poller_output($local_data_id = '', $rrdtool_pipe = '') {
 		}
 	}
 
-	db_execute("SELECT RELEASE_LOCK('boost.single_ds.$orig_local_data_id')");
+	db_execute("SELECT RELEASE_LOCK('boost.single_ds.$local_data_id')");
 
 	/* restore original error handler */
 	restore_error_handler();
@@ -923,6 +923,16 @@ function boost_process_poller_output($local_data_id = '', $rrdtool_pipe = '') {
 function boost_rrdtool_get_last_update_time($rrd_path, &$rrdtool_pipe) {
 	$return_value = 0;
 
+	/* check if the rrd_path is empty
+	 * It can become empty if someone has removed
+	 * a Data Source while boost is running, or a Re-Index
+	 * found the Data Source invalid, so it was removed
+	 * from the poller_item table
+	 */
+	if ($rrd_path == '') {
+		return time();
+	}
+
 	if (read_config_option('storage_location')) {
 		$file_exists = rrdtool_execute("file_exists $rrd_path" , true, RRDTOOL_OUTPUT_BOOLEAN, $rrdtool_pipe, 'BOOST');
 	} else {
@@ -1012,7 +1022,7 @@ function boost_get_rrd_filename_and_template($local_data_id) {
 	return array('rrd_path' => $rrd_path, 'rrd_template' => trim($rrd_template));
 }
 
-function boost_rrdtool_function_create($local_data_id, $initial_time, $show_source, &$rrdtool_pipe) {
+function boost_rrdtool_function_create($local_data_id, $show_source, &$rrdtool_pipe) {
 	global $config;
 
 	/**
@@ -1028,7 +1038,7 @@ function boost_rrdtool_function_create($local_data_id, $initial_time, $show_sour
 	if ($show_source != true) {
 		if (read_config_option('storage_location')) {
 			$file_exists = rrdtool_execute("file_exists $data_source_path" , true, RRDTOOL_OUTPUT_BOOLEAN, $rrdtool_pipe, 'POLLER');
-		}else {
+		} else {
 			$file_exists = file_exists($data_source_path);
 		}
 
@@ -1066,11 +1076,8 @@ function boost_rrdtool_function_create($local_data_id, $initial_time, $show_sour
 		return false;
 	}
 
-	/* back off the initial time to allow updates */
-	$initial_time -= 300;
-
 	/* create the "--step" line */
-	$create_ds = RRD_NL . '--start ' . $initial_time . ' --step '. $rras[0]['rrd_step'] . ' ' . RRD_NL;
+	$create_ds = RRD_NL . '--start 0 --step '. $rras[0]['rrd_step'] . ' ' . RRD_NL;
 
 	/* query the data sources to be used in this .rrd file */
 	$data_sources = db_fetch_assoc_prepared('SELECT
@@ -1171,7 +1178,7 @@ function boost_rrdtool_function_create($local_data_id, $initial_time, $show_sour
    @arg $rrd_path      - the path to the RRD file
    @arg $rrd_update_template  - the order in which values need to be added
    @arg $rrd_update_values    - values to include in the database */
-function boost_rrdtool_function_update($local_data_id, $rrd_path, $rrd_update_template, $initial_time, &$rrd_update_values, &$rrdtool_pipe) {
+function boost_rrdtool_function_update($local_data_id, $rrd_path, $rrd_update_template, &$rrd_update_values, &$rrdtool_pipe) {
 
 	/* lets count the number of rrd files processed */
 	$rrds_processed = 0;
@@ -1179,6 +1186,14 @@ function boost_rrdtool_function_update($local_data_id, $rrd_path, $rrd_update_te
 	/* let's check for deleted Data Sources */
 	$valid_entry = true;
 
+	/* check for an empty rrd_path
+	 * this can happen when you've removed a data source
+	 * while boost is running
+	 */
+	if ($rrd_path == '') {
+		return 'OK';
+	}
+
 	/* create the rrd if one does not already exist */
 	if (read_config_option('storage_location')) {
 		$file_exists = rrdtool_execute("file_exists $rrd_path" , true, RRDTOOL_OUTPUT_BOOLEAN, $rrdtool_pipe, 'BOOST');
@@ -1191,7 +1206,7 @@ function boost_rrdtool_function_update($local_data_id, $rrd_path, $rrd_update_te
 
 		// Check for a Data Source that has been removed
 		if ($ds_exists) {
-			$valid_entry = boost_rrdtool_function_create($local_data_id, $initial_time, false, $rrdtool_pipe);
+			$valid_entry = boost_rrdtool_function_create($local_data_id, false, $rrdtool_pipe);
 		} else {
 			return 'OK';
 		}
@@ -1261,21 +1276,23 @@ function boost_update_snmp_statistics () {
 	$engine = '';
 	$max_data_length = 0;
 
-	foreach($boost_table_status as $table) {
-		if ($table['TABLE_NAME'] == 'poller_output_boost') {
-			$pending_records += $table['TABLE_ROWS'];
-		} else {
-			$arch_records += $table['TABLE_ROWS'];
+	if (cacti_sizeof($boost_table_status)) {
+		foreach($boost_table_status as $table) {
+			if ($table['TABLE_NAME'] == 'poller_output_boost') {
+				$pending_records += $table['TABLE_ROWS'];
+			} else {
+				$arch_records += $table['TABLE_ROWS'];
+			}
+			$data_length += $table['DATA_LENGTH'];
+			$data_length -= $table['DATA_FREE'];
+			$engine = $table['ENGINE'];
+			$max_data_length = $table['MAX_DATA_LENGTH'];
 		}
-		$data_length += $table['DATA_LENGTH'];
-		$data_length -= $table['DATA_FREE'];
-		$engine = $table['ENGINE'];
-		$max_data_length = $table['MAX_DATA_LENGTH'];
 	}
+
 	$total_records = $pending_records + $arch_records;
 	$avg_row_length = ($total_records ? intval($data_length / $total_records) : 0);
 
-
 	if (strcmp($engine, 'MEMORY') == 0) {
 		$max_length = db_fetch_cell('SELECT MAX(LENGTH(output)) FROM poller_output_boost');
 		$max_table_allowed = $max_data_length;
