@@ -22,7 +22,7 @@
  +-------------------------------------------------------------------------+
 */
 
-function run_data_query($host_id, $snmp_query_id) {
+function run_data_query($host_id, $snmp_query_id, $automation = false, $force = false) {
 	global $config, $input_types;
 
 	if (read_config_option('data_source_trace') == 'on') {
@@ -172,7 +172,7 @@ function run_data_query($host_id, $snmp_query_id) {
 
 	/* recalculate/change sort order */
 	$local_data = db_fetch_assoc_prepared('SELECT dl.id AS local_data_id, dl.host_id,
-		dl.snmp_query_id, dl.snmp_index,
+		dl.snmp_query_id, dl.snmp_index, dl.orphan,
 		did.data_input_field_id, did.data_template_data_id,
 		did.value AS query_index, "' . $old_sort_field . '" AS sort_field
 		FROM data_local AS dl
@@ -189,8 +189,10 @@ function run_data_query($host_id, $snmp_query_id) {
 		array($snmp_query_id, $host_id));
 
 	$changed_ids = array();
-	$removed_ids = array();
+	$orphaned_ids = array();
 	if (cacti_sizeof($local_data)) {
+		query_debug_timer_offset('data_query', __('Found %s Local Data ID\'s to Verify', cacti_sizeof($local_data)));
+
 		foreach($local_data as $data_source) {
 			// Just in case there is a forced type from the data source page
 			$forced_type = false;
@@ -286,7 +288,8 @@ function run_data_query($host_id, $snmp_query_id) {
 					query_debug_timer_offset('data_query', __('Index Change Detected! CurrentIndex: %s, PreviousIndex: %s', $current_index, $data_source['query_index']));
 
 					db_execute_prepared('UPDATE data_local
-						SET snmp_index = ?
+						SET snmp_index = ?,
+						orphan = 0
 						WHERE id = ?',
 						array($current_index, $data_source['local_data_id']));
 
@@ -302,16 +305,11 @@ function run_data_query($host_id, $snmp_query_id) {
 
 					// Set the index to Null, note that the Data Source still has the value
 					db_execute_prepared('UPDATE data_local
-						SET snmp_index = ""
+						SET orphan = 1
 						WHERE id = ?',
 						array($data_source['local_data_id']));
 
-					// Remove this item from the poller cache as it will cause wranings
-					db_execute_prepared('DELETE FROM poller_item
-						WHERE local_data_id = ?',
-						array($data_source['local_data_id']));
-
-					$removed_ids[] = $data_source['local_data_id'];
+					$orphaned_ids[] = $data_source['local_data_id'];
 				}
 			}
 
@@ -330,15 +328,30 @@ function run_data_query($host_id, $snmp_query_id) {
 			}
 		}
 
-		if (cacti_sizeof($changed_ids) || sizeof($removed_ids)) {
-			query_debug_timer_offset('data_query', __('Remapping Graphs to their new Indexes'));
+		query_debug_timer_offset('data_query', __('Verification of %s Local Data ID\'s Complete', cacti_sizeof($local_data)));
+
+		if (cacti_sizeof($changed_ids) || cacti_sizeof($orphaned_ids)) {
+			query_debug_timer_offset('data_query', __('Found Changed %s and %s Orphaned Local Data ID\'s to Re-map.', cacti_sizeof($changed_ids), cacti_sizeof($orphaned_ids)));
 			data_query_remap_indexes($changed_ids);
-			data_query_remap_indexes($removed_ids);
+			data_query_remap_indexes($orphaned_ids);
+			query_debug_timer_offset('data_query', __('Done remapping Graphs to their new Indexes'));
 		}
 
-		/* update title cache for graph and data source */
-		update_data_source_title_cache_from_host($host_id);
-		update_graph_title_cache_from_host($host_id);
+		if ((cacti_sizeof($changed_ids) || cacti_sizeof($orphaned_ids)) && !$force) {
+			/* update title cache for graph and data source */
+			update_data_source_title_cache_from_host($host_id, $snmp_query_id, array_merge($changed_ids, $orphaned_ids));
+			query_debug_timer_offset('data_query', __('Done updating Data Source Title Cache'));
+
+			update_graph_title_cache_from_host($host_id, $snmp_query_id, array_merge($changed_ids, $orphaned_ids));
+			query_debug_timer_offset('data_query', __('Done updating Graph Title Cache'));
+		} elseif ($force) {
+			/* update title cache for graph and data source */
+			update_data_source_title_cache_from_host($host_id, $snmp_query_id);
+			query_debug_timer_offset('data_query', __('Done updating Data Source Title Cache'));
+
+			update_graph_title_cache_from_host($host_id, $snmp_query_id);
+			query_debug_timer_offset('data_query', __('Done updating Graph Title Cache'));
+		}
 	}
 
 	query_debug_timer_offset('data_query', __('Index Association with Local Data complete'));
@@ -347,7 +360,7 @@ function run_data_query($host_id, $snmp_query_id) {
 
 	/* update the auto reindex cache */
 	if (cacti_sizeof($changed_ids)) {
-		query_debug_timer_offset('data_query', __('Update Re-Index Cache complete. There were ' . sizeof($changed_ids) . ' index changes, and ' . sizeof($removed_ids) . ' removed indexes.'));
+		query_debug_timer_offset('data_query', __('Update Re-Index Cache complete. There were ' . cacti_sizeof($changed_ids) . ' index changes, and ' . cacti_sizeof($orphaned_ids) . ' orphaned indexes.'));
 
 		/* update the poller cache */
 		update_poller_cache_from_query($host_id, $snmp_query_id, $changed_ids);
@@ -356,11 +369,12 @@ function run_data_query($host_id, $snmp_query_id) {
 		query_debug_timer_offset('data_query', __('No Index Changes Detected, Skipping Re-Index and Poller Cache Re-population'));
 	}
 
-	if (cacti_sizeof($removed_ids)) {
+	if (cacti_sizeof($orphaned_ids) &&
+		(isset($query_array['index_orphan_removal']) && $query_array['index_orphan_removal'] == 'true')) {
 		$poller_ids = array_rekey(
 			db_fetch_assoc_prepared('SELECT DISTINCT poller_id
 				FROM poller_item
-				WHERE local_data_id IN (' . implode(', ', $removed_ids) . ')'),
+				WHERE local_data_id IN (' . implode(', ', $orphaned_ids) . ')'),
 			'poller_id', 'poller_id'
 		);
 
@@ -369,9 +383,6 @@ function run_data_query($host_id, $snmp_query_id) {
 				api_data_source_cache_crc_update($poller_id);
 			}
 		}
-
-		db_execute_prepared('DELETE FROM poller_item
-			WHERE local_data_id IN (' . implode(', ', $removed_ids) . ')');
 	}
 
 	if ($config['poller_id'] == 1) {
@@ -382,7 +393,7 @@ function run_data_query($host_id, $snmp_query_id) {
 		api_plugin_hook_function('run_data_query', array('host_id' => $host_id, 'snmp_query_id' => $snmp_query_id));
 		query_debug_timer_offset('data_query', __('Plugin hooks complete'));
 	} elseif ($config['connection'] == 'online') {
-		poller_push_reindex_data_to_main($host_id, $snmp_query_id);
+		poller_push_reindex_data_to_poller($host_id, $snmp_query_id);
 
 		automation_execute_data_query($host_id, $snmp_query_id);
 		query_debug_timer_offset('data_query', __('Automation Execution for Data Query complete'));
@@ -401,9 +412,39 @@ function run_data_query($host_id, $snmp_query_id) {
 		}
 	}
 
+	// Remove orphans only if the data query expressly calls for it
+	if (cacti_sizeof($orphaned_ids) &&
+		(isset($query_array['index_orphan_removal']) && $query_array['index_orphan_removal'] == 'true')) {
+		data_query_remove_disabled_items($orphaned_ids);
+	}
+
 	return (isset($result) ? $result : true);
 }
 
+function data_query_remove_disabled_items($orphaned_ids) {
+	if (cacti_sizeof($orphaned_ids)) {
+		db_execute_prepared('DELETE FROM poller_item
+			WHERE local_data_id IN (' . implode(', ', $orphaned_ids) . ')');
+
+		db_execute_prepared('DELETE FROM poller_output_boost
+			WHERE local_data_id IN (' . implode(', ', $orphaned_ids) . ')');
+
+		$archive_tables = array_rekey(
+			db_fetch_assoc('SELECT TABLE_NAME
+				FROM information_schema.TABLES
+				WHERE TABLE_NAME LIKE "poller_output_boost_arch%"'),
+			'TABLE_NAME', 'TABLE_NAME'
+		);
+
+		if (cacti_sizeof($archive_tables)) {
+			foreach($archive_tables as $table) {
+				db_execute_prepared("DELETE FROM $table
+					WHERE local_data_id IN (" . implode(', ', $orphaned_ids) . ')', false);
+			}
+		}
+	}
+}
+
 function query_check_suitable($new_sort_field, $old_sort_field, $host_id, $snmp_query_id) {
 	if ($new_sort_field == $old_sort_field) {
 		query_debug_timer_offset('data_query', __('Checking for Sort Field change.  No changes detected.'));
@@ -584,7 +625,7 @@ function query_script_host($host_id, $snmp_query_id) {
 		$script_num_index_array = exec_into_array($script_path);
 
 		// if the number of indexes does not exist use emulation
-		if (!sizeof($script_num_index_array)) {
+		if (!cacti_sizeof($script_num_index_array)) {
 			query_debug_timer_offset('data_query', __('Data Query returned no indexes.'));
 			query_debug_timer_offset('data_query', __('&lt;arg_num_indexes&gt; exists in XML file but no data returned., \'Index Count Changed\' not supported'));
 		}
@@ -608,7 +649,7 @@ function query_script_host($host_id, $snmp_query_id) {
 	/* fetch specified index */
 	$script_index_array = exec_into_array($script_path);
 
-	if (!sizeof($script_index_array)) {
+	if (!cacti_sizeof($script_index_array)) {
 		query_debug_timer_offset('data_query', __('ERROR: Data Query returned no indexes.'));
 		return false;
 	}
@@ -623,6 +664,8 @@ function query_script_host($host_id, $snmp_query_id) {
 
 	debug_log_insert_section_end('data_query');
 
+	$empty_types = array();
+
 	/* set an array to host all updates */
 	$output_array = array();
 
@@ -636,6 +679,7 @@ function query_script_host($host_id, $snmp_query_id) {
 			$script_data_array = exec_into_array($script_path);
 
 			if (!cacti_sizeof($script_data_array) && $field_name == $sort_field) {
+				$empty_types[] = $field_name;
 				query_debug_timer_offset('data_query', __('Sort field returned no data for field name %s, skipping', $field_name));
 			} else {
 				debug_log_insert('data_query', __('Executing script query \'%s\'', $script_path));
@@ -659,7 +703,7 @@ function query_script_host($host_id, $snmp_query_id) {
 	}
 
 	if (cacti_sizeof($output_array)) {
-		data_query_update_host_cache_from_buffer($host_id, $snmp_query_id, $output_array);
+		data_query_update_host_cache_from_buffer($host_id, $snmp_query_id, $output_array, $empty_types);
 	}
 
 	return true;
@@ -812,6 +856,8 @@ function query_snmp_host($host_id, $snmp_query_id) {
 
 	$fields_processed = array();
 
+	$empty_types = array();
+
 	rewrite_snmp_enum_value(null);
 
 	foreach ($snmp_queries['fields'] as $field_name => $field_array) {
@@ -918,7 +964,7 @@ function query_snmp_host($host_id, $snmp_query_id) {
 						$oids[] = $value['oid'];
 					}
 
-					debug_log_insert('data_query', __('Executing SNMP get for %s oids (%s)' , count($oids), implode(', ', $oids)));
+					debug_log_insert('data_query', __('Executing SNMP get for %s oids (%s)' , cacti_count($oids), implode(', ', $oids)));
 
 					$value_output_format = SNMP_STRING_OUTPUT_GUESS;
 					if (isset($field_array['output_format'])) {
@@ -1035,6 +1081,7 @@ function query_snmp_host($host_id, $snmp_query_id) {
 						$snmp_data[$d['oid']] = $d['value'];
 					}
 				} elseif ($field_name == $sort_field) {
+					$empty_types[] = $field_name;
 					query_debug_timer_offset('data_query', __('Sort field returned no data for OID[%s], skipping.', $field_array['oid']));
 					continue;
 				}
@@ -1042,6 +1089,7 @@ function query_snmp_host($host_id, $snmp_query_id) {
 				$snmp_data = cacti_snmp_session_walk($session, $field_array['oid']);
 
 				if (!cacti_sizeof($snmp_data) && $field_name == $sort_field) {
+					$empty_types[] = $field_name;
 					query_debug_timer_offset('data_query', __('Sort field returned no data for OID[%s], skipping.', $field_array['oid']));
 					continue;
 				}
@@ -1116,7 +1164,7 @@ function query_snmp_host($host_id, $snmp_query_id) {
 						$oid = $field_array['oid'] .  '.' . $parse_value;
 
 						/* rewrite octet strings */
-						if (preg_match('/^\d{1,3}(\.\d{1,3}) {2,}$/', $parse_value)) {
+						if (preg_match('/^\d{1,3}(\.\d{1,3}){2,}$/', $parse_value)) {
 							$octets = explode('.', $parse_value);
 							$size = array_shift($octets);
 
@@ -1174,7 +1222,7 @@ function query_snmp_host($host_id, $snmp_query_id) {
 	$session->close();
 
 	if (cacti_sizeof($output_array)) {
-		data_query_update_host_cache_from_buffer($host_id, $snmp_query_id, $output_array);
+		data_query_update_host_cache_from_buffer($host_id, $snmp_query_id, $output_array, $empty_types);
 	}
 
 	return true;
@@ -1205,7 +1253,7 @@ function data_query_ctype_print_unicode($value) {
 	return preg_match($pattern, $value);
 }
 
-function data_query_update_host_cache_from_buffer($host_id, $snmp_query_id, &$output_array) {
+function data_query_update_host_cache_from_buffer($host_id, $snmp_query_id, &$output_array, &$empty_types) {
 	/* set all fields present value to 0, to mark the outliers when we are all done */
 	db_execute_prepared('UPDATE host_snmp_cache
 		SET present = 0
@@ -1252,8 +1300,23 @@ function data_query_update_host_cache_from_buffer($host_id, $snmp_query_id, &$ou
 		db_execute($sql_prefix . $buffer . $sql_suffix);
 	}
 
+	if (cacti_sizeof($empty_types)) {
+		foreach($empty_types as $field) {
+			db_execute_prepared('UPDATE host_snmp_cache
+				SET present = 1
+				WHERE host_id = ?
+				AND snmp_query_id = ?
+				AND field_name = ?',
+				array($host_id, $snmp_query_id, $field));
+		}
+	}
+
 	/* remove stale records from the host cache */
-	db_execute_prepared("DELETE FROM host_snmp_cache WHERE host_id = ? AND snmp_query_id = ? AND present='0'", array($host_id, $snmp_query_id));
+	db_execute_prepared("DELETE FROM host_snmp_cache
+		WHERE host_id = ?
+		AND snmp_query_id = ?
+		AND present='0'",
+		array($host_id, $snmp_query_id));
 }
 
 /* data_query_rewrite_indexes - returns array of rewritten indexes
@@ -1627,7 +1690,8 @@ function update_data_source_data_query_cache($local_data_id, $host_id = '', $dat
 		/* set the index to the new index */
 		db_execute_prepared('UPDATE data_local
 			SET snmp_query_id = ?,
-			snmp_index = ?
+			snmp_index = ?,
+			orphan = 0
 			WHERE id = ?',
 			array($data_query_id, $current_index, $local_data_id));
 
@@ -1930,7 +1994,7 @@ function get_ordered_index_type_list($host_id, $data_query_id) {
 	$return_array = array();
 
 	// Reorder the sort fields by desired order
-	if (sizeof($avail_indexes)) {
+	if (cacti_sizeof($avail_indexes)) {
 		foreach($avail_indexes as $index) {
 			if (array_search($index, $xml_outputs) !== false) {
 				$return_array[] = $index;
@@ -2015,7 +2079,7 @@ function get_best_data_query_index_type($host_id, $data_query_id) {
 	if ($index_type == '') {
 		$raw_xml = get_data_query_array($data_query_id);
 
-		if (sizeof($raw_xml)) {
+		if (cacti_sizeof($raw_xml)) {
 			if (isset($raw_xml['index_order']) && $raw_xml['index_order'] != '') {
 				$order = explode(':', $raw_xml['index_order']);
 
@@ -2026,7 +2090,7 @@ function get_best_data_query_index_type($host_id, $data_query_id) {
 					array($order[0], $host_id, $data_query_id));
 
 				return $order[0];
-			} elseif (isset($raw_xml['fields']) && sizeof($raw_xml['fields'])) {
+			} elseif (isset($raw_xml['fields']) && cacti_sizeof($raw_xml['fields'])) {
 				foreach($raw_xml['fields'] as $key => $attribs) {
 					break;
 				}
@@ -2194,7 +2258,7 @@ function api_data_query_errors($snmp_query_graph_id, $post) {
 
 	$errors = false;
 
-	if (sizeof($data_sources)) {
+	if (cacti_sizeof($data_sources)) {
 		foreach($data_sources as $ds) {
 			if (!isset($post['dsdt_' . $ds['data_template_id'] . '_' . $ds['id'] . '_check'])) {
 				raise_message('mapping_error', __('You must select an XML output column for Data Source \'%s\' and toggle the checkbox to its right', $ds['data_source_name']), MESSAGE_LEVEL_ERROR);
@@ -2209,4 +2273,3 @@ function api_data_query_errors($snmp_query_graph_id, $post) {
 
 	return $errors;
 }
-
