@@ -33,9 +33,21 @@ function nth_percentile($local_data_ids, $start_seconds, $end_seconds, $percenti
 	$stats = json_decode(rrdtool_function_stats($local_data_ids, $start_seconds, $end_seconds, $percentile, $resolution, $peak), true);
 
 	if ($peak) {
-		return $stats['peak'];
+		if (array_key_exists('peak', $stats)) {
+			return $stats['peak'];
+		} else if (array_key_exists('avg', $stats)) {
+			return $stats['avg'];
+		} else {
+			return $stats;
+		}
 	} else {
-		return $stats['avg'];
+		if (array_key_exists('avg', $stats)) {
+			return $stats['avg'];
+		} else if (array_key_exists('peak', $stats)) {
+			return $stats['peak'];
+		} else {
+			return $stats;
+		}
 	}
 }
 
@@ -150,7 +162,7 @@ function nth_percentile_fetch_statistics($percentile, &$local_data_ids, &$fetch_
 	foreach ($local_data_ids as $ldi => $data_source_name) {
 		if (cacti_sizeof($fetch_array[$ldi]['data_source_names'])) {
 			foreach ($fetch_array[$ldi]['data_source_names'] as $index => $ds_name) {
-				if (cacti_sizeof($fetch_array[$ldi]['values'][$index])) {
+				if (isset($fetch_array[$ldi]['values'][$index]) && cacti_sizeof($fetch_array[$ldi]['values'][$index])) {
 					foreach ($fetch_array[$ldi]['values'][$index] as $timestamp => $data) {
 						if (isset($asum_array[$ds_name]) && isset($asum_array[$ds_name][$timestamp])) {
 							$asum_array[$ds_name][$timestamp] += $data;
@@ -158,6 +170,8 @@ function nth_percentile_fetch_statistics($percentile, &$local_data_ids, &$fetch_
 							$asum_array[$ds_name][$timestamp]  = $data;
 						}
 					}
+				} else {
+					return false;
 				}
 			}
 		}
@@ -412,7 +426,7 @@ function variable_nth_percentile(&$regexp_match_array, &$graph, &$graph_item, &$
 	}
 
 	$gi = array();
-	if (sizeof($graph_items)) {
+	if (cacti_sizeof($graph_items)) {
 		foreach ($graph_items as $item) {
 			if ($item['local_data_id'] > 0 && $item['data_source_name'] != '') {
 				if (!empty($item['data_template_rrd_id']) &&
@@ -435,7 +449,7 @@ function variable_nth_percentile(&$regexp_match_array, &$graph, &$graph_item, &$
 	}
 
 	/* Get the Nth percentile values */
-	if (!sizeof($nth_cache)) {
+	if (!cacti_sizeof($nth_cache)) {
 		switch ($type) {
 			case 'current':
 				// Query data for the individual case
@@ -451,28 +465,28 @@ function variable_nth_percentile(&$regexp_match_array, &$graph, &$graph_item, &$
 				break;
 			case 'total':
 			case 'all_max_current':
-				if (sizeof($local_data_array)) {
+				if (cacti_sizeof($local_data_array)) {
 					$nth_cache = nth_percentile($local_data_array, $graph_start, $graph_end, $percentile);
 				}
 
 				break;
 			case 'total_peak':
 			case 'all_max_peak':
-				if (sizeof($local_data_array)) {
+				if (cacti_sizeof($local_data_array)) {
 					$nth_cache = nth_percentile($local_data_array, $graph_start, $graph_end, $percentile, 0, true);
 				}
 
 				break;
 			case 'aggregate':
 			case 'aggregate_sum':
-				if (sizeof($local_data_array)) {
+				if (cacti_sizeof($local_data_array)) {
 					$nth_cache = nth_percentile($local_data_array, $graph_start, $graph_end, $percentile);
 				}
 
 				break;
 			case 'aggregate_peak':
 			case 'aggregate_max':
-				if (sizeof($local_data_array)) {
+				if (cacti_sizeof($local_data_array)) {
 					$nth_cache = nth_percentile($local_data_array, $graph_start, $graph_end, $percentile, 0, true);
 				}
 
@@ -490,7 +504,7 @@ function variable_nth_percentile(&$regexp_match_array, &$graph, &$graph_item, &$
 						}
 					}
 
-					if (sizeof($local_data_array)) {
+					if (cacti_sizeof($local_data_array)) {
 						$nth_cache = nth_percentile($local_data_array, $graph_start, $graph_end, $percentile);
 					}
 				}
