@@ -76,7 +76,7 @@ function grow_dropdown_tree($tree_id, $parent = 0, $form_name = '', $selected_tr
 				$html_selected = '';
 			}
 
-			print "<option value='" . $leaf['id'] . "'$html_selected>$indent " . html_escape($leaf['title']) . "</option>\n";
+			print "<option value='" . $leaf['id'] . "'$html_selected>$indent " . html_escape($leaf['title']) . '</option>';
 
 			grow_dropdown_tree($tree_id, $leaf['id'], $form_name, $selected_tree_item_id, $tier);
 		}
@@ -206,6 +206,11 @@ function grow_dhtml_trees() {
 				$('.cactiTreeNavigationArea').css('overflow-x', '');
 			}
 		}
+
+		var navWidth = $('#navigation').width();
+		if (navWidth > 0) {
+			$('#searcher').css('width', navWidth-70);
+		}
 	}
 
 	function checkTreeForLogout() {
@@ -377,6 +382,7 @@ function get_tree_path() {
 	if (isset_request_var('node')) {
 		$nodes  = array();
 		$nnodes = array();
+		$rnodes = array();
 
 		if (strpos(get_request_var('node'), 'tbranch') !== false) {
 			$parts = explode('-', get_request_var('node'));
@@ -431,19 +437,18 @@ function get_tree_path() {
 							break;
 					}
 				}
-			}
 
-			$rnodes   = array();
-			while (true) {
-				if ($linknode['parent'] > 0) {
-					$rnodes[] = 'tbranch-' . $linknode['parent'];
+				while (true) {
+					if ($linknode['parent'] > 0) {
+						$rnodes[] = 'tbranch-' . $linknode['parent'];
 
-					$linknode = db_fetch_row_prepared('SELECT *
-						FROM graph_tree_items
-						WHERE id = ?',
-						array($linknode['parent']));
-				} else {
-					break;
+						$linknode = db_fetch_row_prepared('SELECT *
+							FROM graph_tree_items
+							WHERE id = ?',
+							array($linknode['parent']));
+					} else {
+						break;
+					}
 				}
 			}
 
@@ -711,10 +716,18 @@ function create_data_query_branch($leaf, $site_id = -1, $ht = -1) {
 	if (cacti_sizeof($data_queries)) {
 		if ($leaf['host_id'] > 0) {
 			$ntg = get_allowed_graphs('gl.host_id=' . $leaf['host_id'] . ' AND gl.snmp_query_id=0');
-			$agg = get_allowed_aggregate_graphs('gl.host_id=' . $leaf['host_id'] . ' AND gl.snmp_query_id=0');
+			if (read_user_setting('show_aggregates', 'on') == 'on') {
+				$agg = get_allowed_aggregate_graphs('gl.host_id=' . $leaf['host_id'] . ' AND gl.snmp_query_id=0');
+			} else {
+				$agg = array();
+			}
 		} else {
 			$ntg = get_allowed_graphs('gl.snmp_query_id=0');
-			$agg = get_allowed_aggregate_graphs('gl.snmp_query_id=0');
+			if (read_user_setting('show_aggregates', 'on') == 'on') {
+				$agg = get_allowed_aggregate_graphs('gl.snmp_query_id=0');
+			} else {
+				$agg = array();
+			}
 		}
 
 		$ntg = array_merge($ntg, $agg);
@@ -774,6 +787,13 @@ function create_dhtml_tree() {
 }
 
 function html_validate_tree_vars() {
+	static $count = false;
+
+	// prevent double calls in the same stack
+	if ($count) {
+		return false;
+	}
+
 	/* ================= input validation and session storage ================= */
 	$filters = array(
 		'graphs' => array(
@@ -841,6 +861,8 @@ function html_validate_tree_vars() {
 
 	validate_store_request_vars($filters, 'sess_grt');
 	/* ================= input validation ================= */
+
+	$count = true;
 }
 
 function grow_right_pane_tree($tree_id, $leaf_id, $host_group_data) {
@@ -1003,7 +1025,7 @@ function grow_right_pane_tree($tree_id, $leaf_id, $host_group_data) {
 	?>
 	<tr class='even noprint' id='search'>
 		<td class='noprint'>
-		<form name='form_graph_view' method='post' onSubmit='applyGraphFilter();return false'>
+		<form id='form_graph_view' method='post' onSubmit='applyGraphFilter();return false'>
 			<table class='filterTable'>
 				<tr>
 					<td>
@@ -1040,8 +1062,7 @@ function grow_right_pane_tree($tree_id, $leaf_id, $host_group_data) {
 												print ' selected';
 											}
 										}
-										print '>';
-										print $gt['name'] . "</option>\n";
+										print '>' . html_escape($gt['name']) . '</option>';
 									}
 								}
 							}
@@ -1050,10 +1071,10 @@ function grow_right_pane_tree($tree_id, $leaf_id, $host_group_data) {
 					</td>
 					<td>
 						<span>
-							<input type='button' class='ui-button ui-corner-all ui-widget' id='refresh' value='<?php print __esc('Go');?>' title='<?php print __esc('Set/Refresh Filter');?>' onClick='applyGraphFilter()'>
-							<input type='button' class='ui-button ui-corner-all ui-widget' id='clear' value='<?php print __esc('Clear');?>' title='<?php print __esc('Clear Filters');?>' onClick='clearGraphFilter()'>
+							<input type='submit' class='ui-button ui-corner-all ui-widget' id='go' value='<?php print __esc('Go');?>' title='<?php print __esc('Set/Refresh Filter');?>'>
+							<input type='button' class='ui-button ui-corner-all ui-widget' id='clear' value='<?php print __esc('Clear');?>' title='<?php print __esc('Clear Filters');?>'>
 							<?php if (is_view_allowed('graph_settings')) {?>
-							<input type='button' class='ui-button ui-corner-all ui-widget' id='save' value='<?php print __esc('Save');?>' title='<?php print __esc('Save the current Graphs, Columns, Thumbnail, Preset, and Timeshift preferences to your profile');?>' onClick='saveGraphFilter("tree")'>
+							<input type='button' class='ui-button ui-corner-all ui-widget' id='save' value='<?php print __esc('Save');?>' title='<?php print __esc('Save the current Graphs, Columns, Thumbnail, Preset, and Timeshift preferences to your profile');?>'>
 							<?php }?>
 						</span>
 					</td>
@@ -1070,7 +1091,7 @@ function grow_right_pane_tree($tree_id, $leaf_id, $host_group_data) {
 							<?php
 							if (cacti_sizeof($graphs_per_page)) {
 								foreach ($graphs_per_page as $key => $value) {
-									print "<option value='" . $key . "'"; if (get_request_var('graphs') == $key) { print ' selected'; } print '>' . $value . "</option>\n";
+									print "<option value='" . $key . "'"; if (get_request_var('graphs') == $key) { print ' selected'; } print '>' . $value . '</option>';
 								}
 							}
 							?>
@@ -1117,7 +1138,7 @@ function grow_right_pane_tree($tree_id, $leaf_id, $host_group_data) {
 
 							if (cacti_sizeof($graph_timespans)) {
 								foreach($graph_timespans as $value => $text) {
-									print "<option value='$value'"; if ($_SESSION['sess_current_timespan'] == $value) { print ' selected'; } print '>' . $text . "</option>\n";
+									print "<option value='$value'"; if ($_SESSION['sess_current_timespan'] == $value) { print ' selected'; } print '>' . $text . '</option>';
 								}
 							}
 							?>
@@ -1150,7 +1171,7 @@ function grow_right_pane_tree($tree_id, $leaf_id, $host_group_data) {
 								$end_val = cacti_sizeof($graph_timeshifts)+1;
 								if (cacti_sizeof($graph_timeshifts)) {
 									for ($shift_value=$start_val; $shift_value < $end_val; $shift_value++) {
-										print "<option value='$shift_value'"; if ($_SESSION['sess_current_timeshift'] == $shift_value) { print ' selected'; } print '>' . title_trim($graph_timeshifts[$shift_value], 40) . "</option>\n";
+										print "<option value='$shift_value'"; if ($_SESSION['sess_current_timeshift'] == $shift_value) { print ' selected'; } print '>' . title_trim($graph_timeshifts[$shift_value], 40) . '</option>';
 									}
 								}
 								?>
@@ -1272,6 +1293,19 @@ function grow_right_pane_tree($tree_id, $leaf_id, $host_group_data) {
 	}
 
 	$(function() {
+		$('#go').off('click').on('click', function(event) {
+			event.preventDefault();
+			applyGraphFilter();
+		});
+
+		$('#clear').off('click').on('click', function() {
+			clearGraphFilter();
+		});
+
+		$('#save').off('click').on('click', function() {
+			 saveGraphFilter('tree');
+		});
+
 		$.when(initPage())
 		.done(function() {
 			initializeGraphs();
@@ -1357,7 +1391,11 @@ function grow_right_pane_tree($tree_id, $leaf_id, $host_group_data) {
 		}
 
 		$graphs = get_allowed_graphs($sql_where);
-		$agg    = get_allowed_aggregate_graphs($sql_where);
+		if (read_user_setting('show_aggregates', 'on') == 'on') {
+			$agg = get_allowed_aggregate_graphs($sql_where);
+		} else {
+			$agg = array();
+		}
 
 		$graphs = array_merge($graphs, $agg);
 
@@ -1393,9 +1431,9 @@ function grow_right_pane_tree($tree_id, $leaf_id, $host_group_data) {
 	}
 
 	if (get_request_var('thumbnails') == 'true' || get_request_var('thumbnails') == 'on') {
-		html_graph_thumbnail_area($new_graph_list, '', 'view_type=tree&graph_start=' . get_current_graph_start() . '&graph_end=' . get_current_graph_end(), '', get_request_var('columns'));
+		html_graph_thumbnail_area($new_graph_list, '', 'view_type=tree&graph_start=' . get_current_graph_start() . '&graph_end=' . get_current_graph_end(), '', get_request_var('columns'), $tree_id, $leaf_id);
 	} else {
-		html_graph_area($new_graph_list, '', 'view_type=tree&graph_start=' . get_current_graph_start() . '&graph_end=' . get_current_graph_end(), '', get_request_var('columns'));
+		html_graph_area($new_graph_list, '', 'view_type=tree&graph_start=' . get_current_graph_start() . '&graph_end=' . get_current_graph_end(), '', get_request_var('columns'), $tree_id, $leaf_id);
 	}
 
 	if (!empty($leaf_id)) {
@@ -1469,12 +1507,20 @@ function get_host_graph_list($host_id, $graph_template_id, $data_query_id, $host
 			}
 
 			$graph_template_ids = [];
+
 			foreach ($final_templates as $graph_template) {
 				array_push($graph_template_ids, $graph_template['id']);
 			}
+
 			$sql_where .= ($sql_where != '' ? ' AND ':'') . 'gl.graph_template_id IN (' . implode(', ', $graph_template_ids) . ')';
 			$graphs = get_allowed_graphs($sql_where);
-			$agg    = get_allowed_aggregate_graphs($sql_where);
+
+			if (read_user_setting('show_aggregates', 'on') == 'on') {
+				$agg = get_allowed_aggregate_graphs($sql_where);
+			} else {
+				$agg = array();
+			}
+
 			$graphs = array_merge($graphs, $agg);
 
 			/* let's sort the graphs naturally */
@@ -1532,7 +1578,13 @@ function get_host_graph_list($host_id, $graph_template_id, $data_query_id, $host
 					' ' . ($data_query_index != '' ? ' AND gl.snmp_index = ' . db_qstr($data_query_index): '');
 
 				$graphs = get_allowed_graphs($sql_where);
-				$agg    = get_allowed_aggregate_graphs($sql_where);
+
+				if (read_user_setting('show_aggregates', 'on') == 'on') {
+					$agg = get_allowed_aggregate_graphs($sql_where);
+				} else {
+					$agg = array();
+				}
+
 				$graphs = array_merge($graphs, $agg);
 
 				/* re-key the results on data query index */
