@@ -23,7 +23,7 @@
 */
 
 function import_xml_data(&$xml_data, $import_as_new, $profile_id, $remove_orphans = false) {
-	global $config, $hash_type_codes, $cacti_version_codes, $preview_only, $remove_orphans, $import_debug_info, $legacy_template;
+	global $config, $hash_type_codes, $cacti_version_codes, $preview_only, $import_debug_info, $legacy_template;
 
 	include_once($config['library_path'] . '/xml.php');
 
@@ -277,7 +277,12 @@ function import_package($xmlfile, $profile_id = 1, $remove_orphans = false, $pre
 	if (is_resource($f)) {
 		while (!feof($f)) {
 			$x = fgets($f);
-			if (strpos($x, '<signature>') !== FALSE) {
+
+			if (strlen($x) == 0) {
+				cacti_log('FATAL: Unable to read Cacti Package ' . $filename, true, 'IMPORT', POLLER_VERBOSITY_LOW);
+				fclose($f);
+				return false;
+			} elseif (strpos($x, '<signature>') !== FALSE) {
 				$binary_signature =  base64_decode(trim(str_replace(array('<signature>', '</signature>'), array('', ''), $x)));
 				$x = "   <signature></signature>\n";
 
@@ -313,7 +318,6 @@ function import_package($xmlfile, $profile_id = 1, $remove_orphans = false, $pre
 
 	if ($info_only) {
 		return $data['info'];
-		exit;
 	}
 
 	cacti_log('Verifying each files signature', false, 'IMPORT', POLLER_VERBOSITY_LOW);
@@ -936,7 +940,9 @@ function xml_to_data_template($hash, &$xml_array, &$hash_cache, $import_as_new,
 }
 
 function xml_to_data_query($hash, &$xml_array, &$hash_cache) {
-	global $fields_data_query_edit, $fields_data_query_item_edit, $preview_only, $import_debug_info;
+	global $config, $fields_data_query_edit, $fields_data_query_item_edit, $preview_only, $import_debug_info;
+
+	static $counter = 0;
 
 	/* track changes */
 	$status = 0;
@@ -971,6 +977,15 @@ function xml_to_data_query($hash, &$xml_array, &$hash_cache) {
 		}
 	}
 
+	if (isset($save['xml_path'])) {
+		$path = str_replace('<path_cacti>', $config['base_path'], $save['xml_path']);
+
+		if (!file_exists($path) || !is_readable($path)) {
+			raise_message('resource_missing_' . $counter, __esc('Resource File: \'%s\' is missing or not readable.  Make sure you install it before using Data Query: \'%s\'', $path, $save['name']), MESSAGE_LEVEL_ERROR);
+			$counter++;
+		}
+	}
+
 	/* check for status changes */
 	$status += compare_data($save, $previous_data, 'snmp_query');
 
@@ -1649,6 +1664,12 @@ function xml_to_data_input_method($hash, &$xml_array, &$hash_cache) {
 	$save['hash'] = $hash;
 
 	foreach ($fields_data_input_edit as $field_name => $field_array) {
+		// Work around for templates exported during
+		// period where there was a bug in 1.2.13-1.2.15
+		if ($field_name == 'fname') {
+			$field_name = 'name';
+		}
+
 		/* make sure this field exists in the xml array first */
 		if (isset($xml_array[$field_name])) {
 			if ($field_name == 'input_string' && import_is_base64_encoded($xml_array[$field_name])) {
@@ -1718,6 +1739,12 @@ function xml_to_data_input_method($hash, &$xml_array, &$hash_cache) {
 			$save['data_input_id'] = $data_input_id;
 
 			foreach ($fields_data_input_field_edit as $field_name => $field_array) {
+				// Work around for templates exported during
+				// period where there was a bug in 1.2.13-1.2.15
+				if ($field_name == 'fname') {
+					$field_name = 'name';
+				}
+
 				/* make sure this field exists in the xml array first */
 				if (isset($item_array[$field_name])) {
 					$save[$field_name] = xml_character_decode($item_array[$field_name]);
@@ -1800,7 +1827,7 @@ function compare_data($save, $previous_data, $table) {
 				}
 
 				$different++;
-				$import_debug_info['differences'][] = 'Table: ' . $table . ', Column: ' . $column . ', New Value: ' . $value . ', Old Value: ' . $previous_data[$column];
+				$import_debug_info['differences'][] = 'Table: ' . $table . ', Column: ' . $column . ', New Value: ' . html_escape($value) . ', Old Value: ' . html_escape($previous_data[$column]);
 			}
 		}
 
@@ -1827,60 +1854,60 @@ function hash_to_friendly_name($hash, $display_type_name) {
 
 	switch ($parsed_hash['type']) {
 		case 'graph_template':
-			return $prepend . db_fetch_cell_prepared('SELECT name
+			return $prepend . html_escape(db_fetch_cell_prepared('SELECT name
 			FROM graph_templates
 			WHERE hash = ?',
-			array($parsed_hash['hash']));
+			array($parsed_hash['hash'])));
 		case 'data_template':
-			return $prepend . db_fetch_cell_prepared('SELECT name
+			return $prepend . html_escape(db_fetch_cell_prepared('SELECT name
 			FROM data_template
 			WHERE hash = ?',
-			array($parsed_hash['hash']));
+			array($parsed_hash['hash'])));
 		case 'data_template_item':
-			return $prepend . db_fetch_cell_prepared('SELECT data_source_name
+			return $prepend . html_escape(db_fetch_cell_prepared('SELECT data_source_name
 			FROM data_template_rrd
 			WHERE hash = ?',
-			array($parsed_hash['hash']));
+			array($parsed_hash['hash'])));
 		case 'host_template':
-			return $prepend . db_fetch_cell_prepared('SELECT name
+			return $prepend . html_escape(db_fetch_cell_prepared('SELECT name
 			FROM host_template
 			WHERE hash = ?',
-			array($parsed_hash['hash']));
+			array($parsed_hash['hash'])));
 		case 'data_input_method':
-			return $prepend . db_fetch_cell_prepared('SELECT name
+			return $prepend . html_escape(db_fetch_cell_prepared('SELECT name
 			FROM data_input
 			WHERE hash = ?',
-			array($parsed_hash['hash']));
+			array($parsed_hash['hash'])));
 		case 'data_input_field':
-			return $prepend . db_fetch_cell_prepared('SELECT name
+			return $prepend . html_escape(db_fetch_cell_prepared('SELECT name
 			FROM data_input_fields
 			WHERE hash = ?',
-			array($parsed_hash['hash']));
+			array($parsed_hash['hash'])));
 		case 'data_query':
-			return $prepend . db_fetch_cell_prepared('SELECT name
+			return $prepend . html_escape(db_fetch_cell_prepared('SELECT name
 			FROM snmp_query
 			WHERE hash = ?',
-			array($parsed_hash['hash']));
+			array($parsed_hash['hash'])));
 		case 'gprint_preset':
-			return $prepend . db_fetch_cell_prepared('SELECT name
+			return $prepend . html_escape(db_fetch_cell_prepared('SELECT name
 			FROM graph_templates_gprint
 			WHERE hash = ?',
-			array($parsed_hash['hash']));
+			array($parsed_hash['hash'])));
 		case 'cdef':
-			return $prepend . db_fetch_cell_prepared('SELECT name
+			return $prepend . html_escape(db_fetch_cell_prepared('SELECT name
 			FROM cdef
 			WHERE hash = ?',
-			array($parsed_hash['hash']));
+			array($parsed_hash['hash'])));
 		case 'vdef':
-			return $prepend . db_fetch_cell_prepared('SELECT name
+			return $prepend . html_escape(db_fetch_cell_prepared('SELECT name
 			FROM vdef
 			WHERE hash = ?',
-			array($parsed_hash['hash']));
+			array($parsed_hash['hash'])));
 		case 'data_source_profile':
-			return $prepend . db_fetch_cell_prepared('SELECT name
+			return $prepend . html_escape(db_fetch_cell_prepared('SELECT name
 			FROM data_source_profile
 			WHERE hash = ?',
-			array($parsed_hash['hash']));
+			array($parsed_hash['hash'])));
 		case 'round_robin_archive':
 			return $prepend;
 		default:
@@ -1891,7 +1918,7 @@ function hash_to_friendly_name($hash, $display_type_name) {
 
 			api_plugin_hook_function('get_friendly_name', $param);
 
-			return $param['prepend'];
+			return html_escape($param['prepend']);
 	}
 }
 
@@ -2075,7 +2102,7 @@ function import_display_results($import_debug_info, $filestatus, $web = false, $
 
 			print "<ul>";
 			foreach($filestatus as $filename => $status) {
-				print "<li>" . ($preview ? __("[preview] "):"") . $filename . " [" . $status . "]</li>\n";
+				print "<li>" . ($preview ? __("[preview] "):"") . html_escape($filename) . " [" . $status . "]</li>\n";
 			}
 			print "</ul>";
 		} else {
