@@ -372,7 +372,7 @@ class Ldap {
 		/* set an error handler for ldap */
 		set_error_handler(array($this, 'ErrorHandler'));
 
-		session_write_close();
+		cacti_session_close();
 	}
 
 	function RestoreCactiHandler() {
@@ -382,7 +382,7 @@ class Ldap {
 		/* set an error handler for Cacti */
 		set_error_handler('CactiErrorHandler');
 
-		session_start();
+		cacti_session_start();
 	}
 
 	function RecordError($output, $section = 'LDAP') {
@@ -466,6 +466,9 @@ class Ldap {
 					/* Process group membership if required */
 					if ($this->group_member_type == 1) {
 						$ldap_group_response = ldap_compare($ldap_conn, $this->group_dn, $this->group_attrib, $this->dn);
+						if (!$ldap_group_response) {
+							$ldap_group_response = Ldap::isUserInLDAPGroup($ldap_conn, $this->search_base, $this->group_dn, $this->dn);
+						}
 					} else if ($this->group_member_type == 2) {
 						/* Do a lookup to find this user's true DN. */
 						/* ldap_exop_whoami is not yet included in PHP. For reference, the
@@ -722,7 +725,7 @@ class Ldap {
 		/* validation */
 		if (empty($this->username)) {
 			$output = LdapError::GetErrorDetails(LdapError::Disabled);
-			Ldap::ReportError($output, 'LDAP_SEARCH');
+			Ldap::RecordError($output, 'LDAP_SEARCH');
 			return $output;
 		}
 
@@ -765,7 +768,7 @@ class Ldap {
 			cacti_log('LDAP: GetCN using ldaps://' . $this->host . ':' . $this->port_ssl, false, 'AUTH', POLLER_VERBOSITY_HIGH);
 			$ldap_conn = ldap_connect('ldaps://' . $this->host . ':' . $this->port_ssl);
 		} else {
-			cacti_log('LDAP: GetCN using ldap://' . $this->host . ':' . $this->port_ssl, false, 'AUTH', POLLER_VERBOSITY_HIGH);
+			cacti_log('LDAP: GetCN using ldap://' . $this->host . ':' . $this->port, false, 'AUTH', POLLER_VERBOSITY_HIGH);
 			$ldap_conn = ldap_connect($this->host, $this->port);
 		}
 
@@ -874,5 +877,14 @@ class Ldap {
 		$this->RestoreCactiHandler();
 		return $output;
 	}
+
+	function isUserInLDAPGroup($ldapConn, $ldapbasedn, $groupDN, $ldapUser) {
+		$query       = "(&(distinguishedName=$ldapUser)(memberOf:1.2.840.113556.1.4.1941:=$groupDN))";
+		$ldapSearch  = @ldap_search($ldapConn,$ldapbasedn,$query,array("dn"));
+		$ldapResults = @ldap_get_entries($ldapConn, $ldapSearch);
+
+		// user should only be returned once IF they're a member of the group
+		return isset($ldapResults['count']) && $ldapResults['count'] == 1 ? true:false;
+	}
 }
 
