@@ -623,7 +623,7 @@ function update_resource_cache($poller_id = 1) {
 					cache_in_path($path['path'], $type, $path['recursive']);
 				}
 			} else {
-				cacti_log("ERROR: Unable to read the " . $type . " path '" . $path['path'] . "'", false, 'POLLER');
+				cacti_log("ERROR: Unable to read the " . $type . " path '" . $path['path'] . "'", false, 'REPLICATE');
 			}
 		}
 
@@ -679,7 +679,7 @@ function update_resource_cache($poller_id = 1) {
 							}
 						}
 					} else {
-						cacti_log("WARNING: INFO file does not exist for plugin directory '" . $mpath . '/plugins/' . $path . "'", false, 'POLLER');
+						cacti_log("WARNING: INFO file does not exist for plugin directory '" . $mpath . '/plugins/' . $path . "'", false, 'REPLICATE');
 					}
 				} else {
 					cache_in_path($mpath . '/plugins/' . $path, 'plugins', false);
@@ -700,7 +700,7 @@ function update_resource_cache($poller_id = 1) {
 		}
 	} elseif ($poller_id > 1) {
 		if (read_config_option('disable_cache_replication') == 'on') {
-			cacti_log('NOTE: Resource Cache Replication is currently Disabled!  Skipping Replication.', true, 'POLLER');
+			cacti_log('NOTE: Resource Cache Replication is currently Disabled!  Skipping Replication.', true, 'REPLICATE');
 			return false;
 		}
 
@@ -717,14 +717,14 @@ function update_resource_cache($poller_id = 1) {
 
 		foreach($paths as $type => $path) {
 			if (!file_exists($path['path'])) {
-				cacti_log('INFO: Attempting to create directory \'' . $path['path'] . '\'', false, 'POLLER');
+				cacti_log('INFO: Attempting to create directory \'' . $path['path'] . '\'', false, 'REPLICATE');
 				@mkdir($path['path'], 0755, true);
 			}
 
 			if (is_writable($path['path'])) {
 				resource_cache_out($type, $path);
 			} else {
-				cacti_log("FATAL: Unable to write to the " . $type . " path '" . $path['path'] . "'", false, 'POLLER');
+				cacti_log("FATAL: Unable to write to the " . $type . " path '" . $path['path'] . "'", false, 'REPLICATE');
 			}
 		}
 	}
@@ -747,8 +747,8 @@ function cache_in_path($path, $type, $recursive = true) {
 		$last_md5      = read_config_option($settings_path);
 
 		if (empty($last_md5) || $last_md5 != $curr_md5) {
-			cacti_log('Type:' . $type . ', Path:' . $path . ', Last MD5:' . $last_md5 . ', Curr MD5:' . $curr_md5, false, 'POLLER', POLLER_VERBOSITY_MEDIUM);
-			cacti_log("NOTE: Detecting Resource Change.  Updating Resource Cache for '$path'", false, 'POLLER');
+			cacti_log('Type:' . $type . ', Path:' . $path . ', Last MD5:' . $last_md5 . ', Curr MD5:' . $curr_md5, false, 'REPLICATE', POLLER_VERBOSITY_MEDIUM);
+			cacti_log("NOTE: Detecting Resource Change.  Updating Resource Cache for '$path'", false, 'REPLICATE');
 			update_db_from_path($path, $type, $recursive);
 		}
 
@@ -792,7 +792,7 @@ function cache_in_path($path, $type, $recursive = true) {
 			}
 
 			if (empty($last_md5) || $last_md5 != $curr_md5) {
-				cacti_log("NOTE: Detecting Resource Change.  Updating Resource Cache for '$ppath'", false, 'POLLER');
+				cacti_log("NOTE: Detecting Resource Change.  Updating Resource Cache for '$ppath'", false, 'REPLICATE');
 				update_db_from_path($path, $type, $recursive);
 			}
 		}
@@ -839,6 +839,10 @@ function update_db_from_path($path, $type, $recursive = true) {
 						continue;
 					}
 
+					$entry_path = $path . DIRECTORY_SEPARATOR . $entry;
+					$attributes = fileperms($entry_path);
+					$attributes = empty($attributes) ? 33188 : $attributes;
+
 					$save         = array();
 					$save['path'] = $spath;
 					$save['id']   = db_fetch_cell_prepared('SELECT id
@@ -846,11 +850,10 @@ function update_db_from_path($path, $type, $recursive = true) {
 						WHERE `path` = ?',
 						array($save['path']));
 
-					$entry_path = $path. DIRECTORY_SEPARATOR . $entry;
 					$save['resource_type'] = $type;
 					$save['md5sum']        = md5_file($entry_path);
 					$save['update_time']   = date('Y-m-d H:i:s');
-					$save['attributes']    = fileperms($entry_path);
+					$save['attributes']    = $attributes;
 					$save['contents']      = base64_encode(file_get_contents($entry_path));
 
 					sql_save($save, 'poller_resource_cache');
@@ -872,6 +875,9 @@ function update_db_from_path($path, $type, $recursive = true) {
 			if (array_search($extension, $excluded_extensions, true) === false) {
 				$spath = ltrim(trim(str_replace($config['base_path'], '', $path), '/ \\'), '/ \\');
 
+				$attributes = fileperms($path);
+				$attributes = empty($attributes) ? 33188 : $attributes;
+
 				$save         = array();
 				$save['path'] = $spath;
 
@@ -883,6 +889,7 @@ function update_db_from_path($path, $type, $recursive = true) {
 				$save['resource_type'] = $type;
 				$save['md5sum']        = md5_file($path);
 				$save['update_time']   = date('Y-m-d H:i:s');
+				$save['attributes']    = $attributes;
 				$save['contents']      = base64_encode(file_get_contents($path));
 
 				sql_save($save, 'poller_resource_cache');
@@ -934,7 +941,9 @@ function resource_cache_out($type, $path) {
 
 				if (is_dir(dirname($mypath))) {
 					if ($md5sum != $e['md5sum'] && $e['path'] != 'include/config.php') {
-						$attributes = empty($e['attributes']) ? 0 : $e['attributes'];
+						// If for some reason, the attributes are empty, assume 0644
+						$attributes = empty($e['attributes']) ? 33188 : $e['attributes'];
+
 						$extension = substr(strrchr($e['path'], "."), 1);
 						$exit = -1;
 						$contents = base64_decode(db_fetch_cell_prepared('SELECT contents
@@ -944,6 +953,16 @@ function resource_cache_out($type, $path) {
 
 						/* if the file type is PHP check syntax */
 						if ($extension == 'php' && $contents != '') {
+							// Executable check
+							$executable = false;
+							if (strpos($e['path'], 'lib/poller.php') === false) {
+								if (strpos($contents, '#!/usr/bin/env php') !== false) {
+									$executable = true;
+								} elseif (strpos($contents, '#!/usr/bin/php') !== false) {
+									$executable = true;
+								}
+							}
+
 							$tmpdir = sys_get_temp_dir();
 							$tmpfile = tempnam($tmpdir,'ccp');
 
@@ -952,48 +971,53 @@ function resource_cache_out($type, $path) {
 									$output = system($php_path . ' -l ' . $tmpfile, $exit);
 
 									if ($exit == 0) {
-										cacti_log("INFO: Updating '$mypath' from Cache!", false, 'POLLER');
+										cacti_log("INFO: Updating '$mypath' from Cache!", false, 'REPLICATE');
 
 										if (is_writable($mypath) || (!file_exists($mypath) && is_writable(dirname($mypath)))) {
 											file_put_contents($mypath, $contents);
 
-											if ($attributes != 0) {
-												chmod($mypath, $attributes);
+											if ($executable) {
+												$attributes = 33261;
+											}
 
-												if (fileperms($mypath) != $attributes) {
-													cacti_log("WARNING: Cache cannot update permissions on '$mypath'", false, 'POLLER');
-												}
+											chmod($mypath, $attributes);
+
+											if (fileperms($mypath) != $attributes) {
+												cacti_log("WARNING: Cache cannot update permissions on '$mypath'", false, 'REPLICATE');
 											}
 										} else {
-											cacti_log("ERROR: Cache in cannot write to '$mypath', purge this location");
+											cacti_log("ERROR: Cache in cannot write to '$mypath', purge this location", false, 'REPLICATE');
 										}
 									} else {
-										cacti_log("ERROR: PHP Source File '$mypath' from Cache has an error while checking syntax ($exit) while executing: '$php_path -l $tmpfile'", false, 'POLLER');
-										cacti_log("ERROR: PHP Source File '$mypath'': " . str_replace("\n", ' ', str_replace("\t", ' ', $output)), false, 'POLLER');
+										cacti_log("ERROR: PHP Source File '$mypath' from Cache has an error while checking syntax ($exit) while executing: '$php_path -l $tmpfile'", false, 'REPLICATE');
+										cacti_log("ERROR: PHP Source File '$mypath'': " . str_replace("\n", ' ', str_replace("\t", ' ', $output)), false, 'REPLICATE');
 									}
 
 									unlink($tmpfile);
 								} else {
-									cacti_log("ERROR: Unable to write file '" . $tmpfile . "' for PHP Syntax verification", false, 'POLLER');
+									cacti_log("ERROR: Unable to write file '" . $tmpfile . "' for PHP Syntax verification", false, 'REPLICATE');
 								}
 							} else {
-								cacti_log("ERROR: Cache in cannot write to '" . $tmpfile . "', purge this location");
+								cacti_log("ERROR: Cache in cannot write to '" . $tmpfile . "', purge this location", false, 'REPLICATE');
 							}
 						} elseif (is_writeable($mypath) || (!file_exists($mypath) && is_writable(dirname($mypath)))) {
-							cacti_log("INFO: Updating '" . $mypath . "' from Cache!", false, 'POLLER');
+							cacti_log("INFO: Updating '" . $mypath . "' from Cache!", false, 'REPLICATE');
+
 							file_put_contents($mypath, $contents);
+
 							if ($attributes != 0) {
 								chmod($mypath, $attributes);
+
 								if (fileperms($mypath) != $attributes) {
-									cacti_log("WARNING: Cache cannot update permissions on '$mypath'", false, 'POLLER');
+									cacti_log("WARNING: Cache cannot update permissions on '$mypath'", false, 'REPLICATE');
 								}
 							}
 						} else {
-							cacti_log("ERROR: Cache in cannot write to '" . $mypath . "', purge this location");
+							cacti_log("ERROR: Cache in cannot write to '" . $mypath . "', purge this location", false, 'REPLICATE');
 						}
 					}
 				} else {
-					cacti_log("ERROR: Directory does not exist '" . dirname($mypath) . "'", false, 'POLLER');
+					cacti_log("ERROR: Directory does not exist '" . dirname($mypath) . "'", false, 'REPLICATE');
 				}
 			}
 		}
@@ -1037,7 +1061,7 @@ function md5sum_path($path, $recursive = true) {
 			} elseif (is_readable($path . DIRECTORY_SEPARATOR . $entry)) {
 				$filemd5s[] = md5_file($path . DIRECTORY_SEPARATOR . $entry);
 			} else {
-				cacti_log('WARNING: Unable to read file \'' . $path . DIRECTORY_SEPARATOR . $entry . '\' into Cacti resource cache.', false, 'POLLER');
+				cacti_log('WARNING: Unable to read file \'' . $path . DIRECTORY_SEPARATOR . $entry . '\' into Cacti resource cache.', false, 'REPLICATE');
 			}
          }
     }
@@ -1078,7 +1102,6 @@ function poller_push_to_remote_db_connect($device_or_poller, $is_poller = false)
 	return $rcnn_id;
 }
 
-
 function poller_connect_to_remote($poller_id) {
 	global $config, $local_db_cnn_id;
 
@@ -1097,7 +1120,7 @@ function poller_connect_to_remote($poller_id) {
 			array($poller_id));
 
 		if (!cacti_sizeof($cinfo)) {
-			cacti_log('ERROR: Remote Data Collector ID[' . $poller_id . '] to be Sync\'d does not exist!', false, 'POLLER');
+			cacti_log('ERROR: Remote Data Collector ID[' . $poller_id . '] to be Sync\'d does not exist!', false, 'REPLICATE');
 			raise_message('poller_notfound');
 			return false;
 		}
@@ -1119,7 +1142,7 @@ function poller_connect_to_remote($poller_id) {
 			$cinfo['dbsslca']);
 
 		if (!is_object($rcnn_id)) {
-			cacti_log('ERROR: Unable to connect to Remote Data Collector ' . $cinfo['name'], false, 'POLLER');
+			cacti_log('ERROR: Unable to connect to Remote Data Collector ' . $cinfo['name'], false, 'REPLICATE');
 			raise_message('poller_noconnect');
 			return false;
 		}
@@ -1164,6 +1187,30 @@ function replicate_out($remote_poller_id = 1, $class = 'all') {
 
 	// Core tables
 	if ($class == 'all') {
+		$data = db_fetch_assoc('SELECT * FROM automation_graph_rule_items');
+		replicate_out_table($rcnn_id, $data, 'automation_graph_rule_items', $remote_poller_id);
+
+		$data = db_fetch_assoc('SELECT * FROM automation_graph_rules');
+		replicate_out_table($rcnn_id, $data, 'automation_graph_rules', $remote_poller_id);
+
+		$data = db_fetch_assoc('SELECT * FROM automation_match_rule_items');
+		replicate_out_table($rcnn_id, $data, 'automation_match_rule_items', $remote_poller_id);
+
+		$data = db_fetch_assoc('SELECT * FROM automation_snmp');
+		replicate_out_table($rcnn_id, $data, 'automation_snmp', $remote_poller_id);
+
+		$data = db_fetch_assoc('SELECT * FROM automation_snmp_items');
+		replicate_out_table($rcnn_id, $data, 'automation_snmp_items', $remote_poller_id);
+
+		$data = db_fetch_assoc('SELECT * FROM automation_templates');
+		replicate_out_table($rcnn_id, $data, 'automation_templates', $remote_poller_id);
+
+		$data = db_fetch_assoc('SELECT * FROM automation_tree_rule_items');
+		replicate_out_table($rcnn_id, $data, 'automation_tree_rule_items', $remote_poller_id);
+
+		$data = db_fetch_assoc('SELECT * FROM automation_tree_rules');
+		replicate_out_table($rcnn_id, $data, 'automation_tree_rules', $remote_poller_id);
+
 		$data = db_fetch_assoc('SELECT * FROM data_input');
 		replicate_out_table($rcnn_id, $data, 'data_input', $remote_poller_id);
 
@@ -1221,7 +1268,8 @@ function replicate_out($remote_poller_id = 1, $class = 'all') {
 			FROM host_snmp_query AS hsq
 			INNER JOIN host AS h
 			ON h.id=hsq.host_id
-			WHERE h.poller_id = ?',
+			WHERE h.poller_id = ?
+			AND h.deleted = ""',
 			array($remote_poller_id));
 		replicate_out_table($rcnn_id, $data, 'host_snmp_query', $remote_poller_id);
 
@@ -1231,15 +1279,10 @@ function replicate_out($remote_poller_id = 1, $class = 'all') {
 			array($remote_poller_id));
 		replicate_out_table($rcnn_id, $data, 'poller_command', $remote_poller_id);
 
-		$data = db_fetch_assoc_prepared('SELECT pi.*
-			FROM poller_item AS pi
-			WHERE pi.poller_id = ?',
-			array($remote_poller_id));
-		replicate_out_table($rcnn_id, $data, 'poller_item', $remote_poller_id);
-
 		$data = db_fetch_assoc_prepared('SELECT h.*
 			FROM host AS h
-			WHERE h.poller_id = ?',
+			WHERE h.poller_id = ?
+			AND h.deleted = ""',
 			array($remote_poller_id));
 		replicate_out_table($rcnn_id, $data, 'host', $remote_poller_id);
 
@@ -1247,23 +1290,58 @@ function replicate_out($remote_poller_id = 1, $class = 'all') {
 			FROM host_snmp_cache AS hsc
 			INNER JOIN host AS h
 			ON h.id=hsc.host_id
-			WHERE h.poller_id = ?',
+			WHERE h.poller_id = ?
+			AND h.deleted = ""',
 			array($remote_poller_id));
 		replicate_out_table($rcnn_id, $data, 'host_snmp_cache', $remote_poller_id);
 
+		// Special class of 'update' for the table following tables 'poller_reindex', 'poller_item'
 		$data = db_fetch_assoc_prepared('SELECT pri.*
 			FROM poller_reindex AS pri
 			INNER JOIN host AS h
 			ON h.id=pri.host_id
-			WHERE h.poller_id = ?',
+			WHERE h.poller_id = ?
+			AND h.deleted = ""',
 			array($remote_poller_id));
-		replicate_out_table($rcnn_id, $data, 'poller_reindex', $remote_poller_id);
+		replicate_out_table($rcnn_id, $data, 'poller_reindex', $remote_poller_id, false, array('assert_value'));
+
+		// Since we are doing an update, remove stale data
+		db_execute('DELETE pr
+			FROM poller_reindex AS pr
+			LEFT JOIN host_snmp_query AS hsq
+			ON pr.host_id = hsq.host_id
+			AND pr.data_query_id = hsq.snmp_query_id
+			WHERE hsq.host_id IS NULL', false, $rcnn_id);
+
+		$data = db_fetch_assoc_prepared('SELECT pi.*
+			FROM poller_item AS pi
+			WHERE pi.poller_id = ?',
+			array($remote_poller_id));
+		replicate_out_table($rcnn_id, $data, 'poller_item', $remote_poller_id, false, array('last_updated'));
+
+		// Remove anything not updated recently
+		$time = db_fetch_cell('SELECT MAX(UNIX_TIMESTAMP(last_updated)) FROM poller_item', false, $rcnn_id);
+
+		if (!empty($time)) {
+			// You must update the last_updated locally
+			db_execute_prepared('UPDATE poller_item
+				SET last_updated = FROM_UNIXTIME(?)
+				WHERE poller_id = ?',
+				array($time, $remote_poller_id));
+
+			db_execute_prepared("DELETE FROM poller_item
+				WHERE last_updated < ?
+				AND last_updated > '0000-00-00'
+				AND last_updated NOT NULL",
+				array(date('Y-m-d H:i:s', $time)), false, $rcnn_id);
+		}
 
 		$data = db_fetch_assoc_prepared('SELECT dl.*
 			FROM data_local AS dl
 			INNER JOIN host AS h
 			ON h.id=dl.host_id
-			WHERE h.poller_id = ?',
+			WHERE h.poller_id = ?
+			AND h.deleted = ""',
 			array($remote_poller_id));
 		replicate_out_table($rcnn_id, $data, 'data_local', $remote_poller_id);
 
@@ -1271,7 +1349,8 @@ function replicate_out($remote_poller_id = 1, $class = 'all') {
 			FROM graph_local AS gl
 			INNER JOIN host AS h
 			ON h.id=gl.host_id
-			WHERE h.poller_id = ?',
+			WHERE h.poller_id = ?
+			AND h.deleted = ""',
 			array($remote_poller_id));
 		replicate_out_table($rcnn_id, $data, 'graph_local', $remote_poller_id);
 
@@ -1281,7 +1360,8 @@ function replicate_out($remote_poller_id = 1, $class = 'all') {
 			ON dtd.local_data_id=dl.id
 			INNER JOIN host AS h
 			ON h.id=dl.host_id
-			WHERE h.poller_id = ?',
+			WHERE h.poller_id = ?
+			AND h.deleted = ""',
 			array($remote_poller_id));
 		replicate_out_table($rcnn_id, $data, 'data_template_data', $remote_poller_id);
 
@@ -1291,7 +1371,8 @@ function replicate_out($remote_poller_id = 1, $class = 'all') {
 			ON dtr.local_data_id=dl.id
 			INNER JOIN host AS h
 			ON h.id=dl.host_id
-			WHERE h.poller_id = ?',
+			WHERE h.poller_id = ?
+			AND h.deleted = ""',
 			array($remote_poller_id));
 		replicate_out_table($rcnn_id, $data, 'data_template_rrd', $remote_poller_id);
 
@@ -1301,7 +1382,8 @@ function replicate_out($remote_poller_id = 1, $class = 'all') {
 			ON gti.local_graph_id=gl.id
 			INNER JOIN host AS h
 			ON h.id=gl.host_id
-			WHERE h.poller_id = ?',
+			WHERE h.poller_id = ?
+			AND h.deleted = ""',
 			array($remote_poller_id));
 		replicate_out_table($rcnn_id, $data, 'graph_templates_item', $remote_poller_id);
 
@@ -1313,7 +1395,8 @@ function replicate_out($remote_poller_id = 1, $class = 'all') {
 			ON dl.id=dtd.local_data_id
 			INNER JOIN host AS h
 			ON h.id=dl.host_id
-			WHERE h.poller_id = ?',
+			WHERE h.poller_id = ?
+			AND h.deleted = ""',
 			array($remote_poller_id));
 		replicate_out_table($rcnn_id, $data, 'data_input_data', $remote_poller_id);
 	}
@@ -1371,31 +1454,76 @@ function replicate_in() {
 	api_plugin_hook_function('replicate_in');
 }
 
-function replicate_out_table($conn, &$data, $table, $remote_poller_id) {
+/** replicate_out_table - replicate out an entire table to
+ *  remote database.  Optionally, performs update rather
+ *  than a truncate and excluding columns from the on duplicate
+ *  clause. By default, the remote table will be recreated
+ *  if it's table structure does not match the main table.
+ *
+ * @param object $conn          - Connection to remote database
+ * @param array $data           - Associative array of the table data
+ * @param string $table         - The remote table to replicate to
+ * @param int $remote_poller_id - The remote data collector's id
+ * @param boolean $truncate     - A flag that if true, truncates, otherwise updates
+ * @param array $exclude        - An array of column names to not update on replication
+ */
+function replicate_out_table($conn, &$data, $table, $remote_poller_id, $truncate = true, $exclude = false) {
 	if (cacti_sizeof($data)) {
-		/* check if the table structure changed */
+		/* check if the table structure changed, and if so, recreate */
 		$local_columns  = db_fetch_assoc('SHOW COLUMNS FROM ' . $table);
 		$remote_columns = db_fetch_assoc('SHOW COLUMNS FROM ' . $table, true, $conn);
+		$remote_rows    = db_fetch_cell('SELECT COUNT(*) FROM ' . $table, '', true, $conn);
+
+		if ($exclude !== false && !is_array($exclude)) {
+			$exclude = array($exclude);
+		}
+
+		if ($remote_rows == 0) {
+			$exclude = false;
+		}
 
 		if (cacti_sizeof($local_columns) != cacti_sizeof($remote_columns)) {
-			cacti_log('NOTE: Replicate Out Detected a Table Structure Change for ' . $table);
+			cacti_log('NOTE: Replicate Out Detected a Table Structure Change for ' . $table, false, 'REPLICATE');
 			$create = db_fetch_row('SHOW CREATE TABLE ' . $table);
 			if (isset($create['Create Table'])) {
-				cacti_log('NOTE: Replication Recreating Remote Table Structure for ' . $table);
+				cacti_log('NOTE: Replication Recreating Remote Table Structure for ' . $table, false, 'REPLICATE');
 				db_execute('DROP TABLE IF EXISTS ' . $table, true, $conn);
 				db_execute($create['Create Table'], true, $conn);
 			}
 		}
 
-		db_execute("TRUNCATE TABLE $table", true, $conn);
+		if ($truncate) {
+			$prefix = "REPLACE INTO $table (";
+			$suffix = '';
+
+			db_execute("TRUNCATE TABLE $table", true, $conn);
+		} else {
+			$prefix = "INSERT INTO $table (";
+			$suffix = ' ON DUPLICATE KEY UPDATE ';
+
+			$cols = array_rekey($local_columns, 'Field', 'Field');
+
+			$i = 0;
+			foreach($cols as $col) {
+				if ($exclude !== false) {
+					if (array_search($col, $exclude) === false) {
+						$suffix .= ($i > 0 ? ', ':'') . " $col=VALUES($col)";
+						$i++;
+					}
+				} else {
+					$suffix .= ($i > 0 ? ', ':'') . " $col=VALUES($col)";
+					$i++;
+				}
+			}
+		}
 
-		$prefix    = "REPLACE INTO $table (";
 		$sql       = '';
 		$colcnt    = 0;
 		$rows_done = 0;
 		$columns   = array_keys($data[0]);
 		$skipcols  = array();
 
+		// Find columns to skip, or exclude from updates
 		foreach($columns as $index => $c) {
 			if (!db_column_exists($table, $c, false, $conn)) {
 				$skipcols[$index] = $c;
@@ -1422,7 +1550,7 @@ function replicate_out_table($conn, &$data, $table, $remote_poller_id) {
 			$rowcnt++;
 
 			if ($rowcnt > 1000) {
-				db_execute($prefix . $sql, true, $conn);
+				db_execute($prefix . $sql . $suffix, true, $conn);
 				$rows_done += db_affected_rows($conn);
 				$sql = '';
 				$rowcnt = 0;
@@ -1430,15 +1558,15 @@ function replicate_out_table($conn, &$data, $table, $remote_poller_id) {
 		}
 
 		if ($rowcnt > 0) {
-			db_execute($prefix . $sql, true, $conn);
+			db_execute($prefix . $sql . $suffix, true, $conn);
 			$rows_done += db_affected_rows($conn);
 		}
 
-		cacti_log('NOTE: Table ' . $table . ' Replicated to Remote Poller ' . $remote_poller_id . ' With ' . $rows_done . ' Rows Updated');
+		cacti_log('NOTE: Table ' . $table . ' Replicated to Remote Poller ' . $remote_poller_id . ' With ' . $rows_done . ' Rows Updated', false, 'REPLICATE');
 	} else {
 		db_execute("TRUNCATE TABLE $table", true, $conn);
 
-		cacti_log('NOTE: Table ' . $table . ' Not Replicated to Remote Poller ' . $remote_poller_id . ' Due to No Rows Found');
+		cacti_log('NOTE: Table ' . $table . ' Not Replicated to Remote Poller ' . $remote_poller_id . ' Due to No Rows Found', false, 'REPLICATE');
 	}
 }
 
@@ -1461,8 +1589,17 @@ function poller_push_reindex_only_data_to_main($device_id, $data_query_id) {
 	}
 }
 
-function poller_push_reindex_data_to_main($device_id = 0, $data_query_id = 0, $force = false) {
-	global $remote_db_cnn_id;
+function poller_push_reindex_data_to_poller($device_id = 0, $data_query_id = 0, $force = false, $db_cnn_id = false) {
+	global $config, $remote_db_cnn_id, $local_db_cnn_id, $database_hostname, $rdatabase_hostname;
+
+	// If the hostnames are the same, replication is from main to remote
+	if ($db_cnn_id) {
+		// We are pushing forcibly to remote
+	} elseif (isset($rdatabase_hostname) && $database_hostname == $rdatabase_hostname) {
+		$db_cnn_id = $local_db_cnn_id;
+	} else {
+		$db_cnn_id = $remote_db_cnn_id;
+	}
 
 	$sql_where   = '';
 	$sql_where1  = '';
@@ -1499,27 +1636,27 @@ function poller_push_reindex_data_to_main($device_id = 0, $data_query_id = 0, $f
 		);
 	}
 
-	if (sizeof($recache_hosts)) {
+	if (cacti_sizeof($recache_hosts)) {
 		$local_data_ids = db_fetch_assoc("SELECT *
 			FROM data_local
 			WHERE host_id IN (" . implode(', ', $recache_hosts) . ")
 			$sql_where1");
 
-		replicate_table_to_poller($remote_db_cnn_id, $local_data_ids, 'data_local');
+		replicate_table_to_poller($db_cnn_id, $local_data_ids, 'data_local');
 
 		$local_graph_ids = db_fetch_assoc("SELECT *
 			FROM graph_local
 			WHERE host_id IN (" . implode(', ', $recache_hosts) . ")
 			$sql_where1");
 
-		replicate_table_to_poller($remote_db_cnn_id, $local_graph_ids, 'graph_local');
+		replicate_table_to_poller($db_cnn_id, $local_graph_ids, 'graph_local');
 
 		$host_snmp_cache = db_fetch_assoc("SELECT *
 			FROM host_snmp_cache
 			WHERE host_id IN (" . implode(', ', $recache_hosts) . ")
 			$sql_where1");
 
-		replicate_table_to_poller($remote_db_cnn_id, $host_snmp_cache, 'host_snmp_cache');
+		replicate_table_to_poller($db_cnn_id, $host_snmp_cache, 'host_snmp_cache');
 
 		// TODO: Make schema's equivalent renamed snmp_query_id to data_query_id everywhere
 		$sql_where1 = str_replace('snmp_query_id', 'data_query_id', $sql_where1);
@@ -1529,11 +1666,11 @@ function poller_push_reindex_data_to_main($device_id = 0, $data_query_id = 0, $f
 			WHERE host_id IN (" . implode(', ', $recache_hosts) . ")
 			$sql_where1");
 
-		replicate_table_to_poller($remote_db_cnn_id, $poller_reindex, 'poller_reindex');
+		replicate_table_to_poller($db_cnn_id, $poller_reindex, 'poller_reindex', array('assert_value'));
 	}
 }
 
-function replicate_table_to_poller($conn, &$data, $table) {
+function replicate_table_to_poller($conn, &$data, $table, $exclude = false) {
 	if (cacti_sizeof($data)) {
 		$prefix    = "INSERT INTO $table (";
 		$suffix    = ' ON DUPLICATE KEY UPDATE ';
@@ -1543,9 +1680,22 @@ function replicate_table_to_poller($conn, &$data, $table) {
 		$columns   = array_keys($data[0]);
 		$skipcols  = array();
 
+		$remote_rows    = db_fetch_cell('SELECT COUNT(*) FROM ' . $table, '', true, $conn);
+
+		if ($exclude !== false && !is_array($exclude)) {
+			$exclude = array($exclude);
+		}
+
+		if ($remote_rows == 0) {
+			$exclude = false;
+		}
+
+		// Find columns to skip from updates
 		foreach($columns as $index => $c) {
 			if (!db_column_exists($table, $c, false, $conn)) {
 				$skipcols[$index] = $c;
+			} elseif ($exclude !== false && array_search($c, $exclude) === true) {
+				// Do not update this column
 			} else {
 				$prefix .= ($colcnt > 0 ? ', ':'') . $c;
 				$suffix .= ($colcnt > 0 ? ', ':'') . "$c=VALUES($c)";
@@ -1582,7 +1732,7 @@ function replicate_table_to_poller($conn, &$data, $table) {
 			$rows_done += db_affected_rows($conn);
 		}
 
-		cacti_log('NOTE: Table ' . $table . ' Replicated to Poller With ' . $rows_done . ' Rows Updated', true, 'WEBUI', POLLER_VERBOSITY_MEDIUM);
+		cacti_log('NOTE: Table ' . $table . ' Replicated to Poller With ' . $rows_done . ' Rows Updated', true, 'REPLICATE', POLLER_VERBOSITY_MEDIUM);
 	}
 }
 
@@ -1670,7 +1820,7 @@ function poller_push_table($db_cnn, $records, $table, $ignore = false, $dupes =
 			$sql[] = $string;
 		}
 
-		if (sizeof($dupes)) {
+		if (cacti_sizeof($dupes)) {
 			$dupe = ' ON DUPLICATE KEY UPDATE ';
 			$i = 0;
 			foreach($dupes as $item) {
@@ -1687,7 +1837,7 @@ function poller_push_table($db_cnn, $records, $table, $ignore = false, $dupes =
 		}
 	}
 
-	return sizeof($records);
+	return cacti_sizeof($records);
 }
 
 function should_ignore_from_replication($path) {
@@ -1753,3 +1903,163 @@ function get_remote_poller_ids_from_devices(&$devices) {
 	}
 }
 
+/** register_process_start - public function to register a process
+ *  in Cacti's process table
+ *
+ * @param string $tasktype  - Mandatory task type
+ * @param string $taskname  - Mandatory task name
+ * @param int $taskid       - Optional task id
+ * @param int $timeout      - Optional timeout
+ * @return boolean success  - true if you can start running, else false if
+ *                            another version is running and has not ended.
+ */
+function register_process_start($tasktype, $taskname, $taskid = 0, $timeout = 300) {
+	$pid = getmypid();
+
+	if (!db_table_exists('processes')) {
+		// We return true here because we assume there is no process blocking start
+		return true;
+	}
+
+	$r = db_fetch_row_prepared('SELECT *
+		FROM processes
+		WHERE tasktype = ?
+		AND taskname = ?
+		AND taskid = ?',
+		array($tasktype, $taskname, $taskid));
+
+	if (!cacti_sizeof($r)) {
+		cacti_log(sprintf('NOTE: Registering process! (%s, %s, %s, %s)', $tasktype, $taskname, $taskid, $pid), false, 'POLLER', POLLER_VERBOSITY_MEDIUM);
+
+		register_process($tasktype, $taskname, $taskid, $pid, $timeout);
+	} elseif (strtotime($r['started']) + $r['timeout'] < time()) {
+		if ($r['pid'] > 0) {
+			cacti_log(sprintf('ERROR: Process being killed due to timeout! (%s, %s, %s, %s)', $tasktype, $taskname, $taskid, $r['pid']), false, 'POLLER');
+
+			posix_kill($r['pid'], SIGTERM);
+
+			unregister_process($tasktype, $taskname, $taskid);
+			register_process($tasktype, $taskname, $taskid, $pid, $timeout);
+		} else {
+			// Should never be reached
+			cacti_log(sprintf('ERROR: Failed registering process.  Invalid pid found.  Unable to kill! (%s, %s, %s, %s)', $tasktype, $taskname, $taskid, $r['pid']), false, 'POLLER');
+
+			return false;
+		}
+	} elseif ($r['pid'] > 0 && posix_kill($r['pid'], 0)) {
+		cacti_log(sprintf('NOTE: Failed registering process.  Old process still running and has not timed out! (%s, %s, %s, %s)', $tasktype, $taskname, $taskid, $pid), false, 'POLLER', POLLER_VERBOSITY_MEDIUM);
+
+		return false;
+	} else {
+		cacti_log(sprintf('WARNING: Detected process that is exited and did not unregister first! (%s, %s, %s, %s)', $tasktype, $taskname, $taskid, $pid), false, 'POLLER');
+
+		unregister_process($tasktype, $taskname, $taskid);
+		register_process($tasktype, $taskname, $taskid, $pid, $timeout);
+	}
+
+	return true;
+}
+
+/** register_process - register a process in Cacti's process table
+ *
+ * @param string $tasktype  - Mandatory task type
+ * @param string $taskname  - Mandatory task name
+ * @param int $taskid       - Mandatory task id
+ * @param int $pid          - Mandatory pid
+ * @param int $timeout      - Mandatory timeout
+ * @return null             - No data is returned
+ */
+function register_process($tasktype, $taskname, $taskid, $pid, $timeout) {
+	if (!db_table_exists('processes')) {
+		return true;
+	}
+
+	db_execute_prepared('INSERT INTO processes (tasktype, taskname, taskid, pid, timeout, last_update)
+		VALUES (?, ?, ?, ?, ?, ?)',
+		array($tasktype, $taskname, $taskid, $pid, $timeout, date('Y-m-d H:i:s')));
+}
+
+/** unregister_process - remove a process from Cacti's process table
+ *
+ * @param string $tasktype  - Mandatory task type
+ * @param string $taskname  - Mandatory task name
+ * @param int $taskid       - Optional task id
+ * @return null             - No data is returned
+ */
+function unregister_process($tasktype, $taskname, $taskid = 0) {
+	if (!db_table_exists('processes')) {
+		return true;
+	}
+
+	db_execute_prepared('DELETE FROM processes
+		WHERE tasktype = ?
+		AND taskname = ?
+		AND taskid = ?',
+		array($tasktype, $taskname, $taskid));
+}
+
+/** heartbeat_process - update the process table last_update timestamp
+ *
+ * @param string $tasktype  - Mandatory task type
+ * @param string $taskname  - Mandatory task name
+ * @param int $taskid       - Optional task id
+ * @return null             - No data is returned
+ */
+function heartbeat_process($tasktype, $taskname, $taskid = 0) {
+	if (!db_table_exists('processes')) {
+		return true;
+	}
+
+	db_execute_prepared('UPDATE processes
+		SET last_update = ?
+		WHERE tasktype = ?
+		AND taskname = ?
+		AND taskid = ?',
+		array(date('Y-m-d H:i:s'), $tasktype, $taskname, $taskid));
+}
+
+/** timeout_kill_registered_processes - allow a Cacti plugin or scheduled task to
+ *  be bulk cleaned.
+ *
+ * @param string $tasktype  - Optional task type
+ * @param string $taskname  - Optional task name
+ * @return null             - No data is returned
+ */
+function timeout_kill_registered_processes($tasktype = '', $taskname = '') {
+	if (!db_table_exists('processes')) {
+		return true;
+	}
+
+	if ($tasktype != '') {
+		$processes = db_fetch_assoc('SELECT *
+			FROM processes
+			WHERE UNIX_TIMESTAMP() > FROM_UNIXTIME(started) + timeout');
+	} elseif ($taskname == '') {
+		$processes = db_fetch_assoc_prepared('SELECT *
+			FROM processes
+			WHERE UNIX_TIMESTAMP() > FROM_UNIXTIME(started) + timeout
+			AND tasktype = ?',
+			array($tasktype));
+	} else {
+		$processes = db_fetch_assoc_prepared('SELECT *
+			FROM processes
+			WHERE UNIX_TIMESTAMP() > FROM_UNIXTIME(started) + timeout
+			AND tasktype = ?
+			AND taskname = ?',
+			array($tasktype, $taskname));
+	}
+
+	if (cacti_sizeof($processes)) {
+		foreach($processes as $r) {
+			if ($r['pid'] > 0 && posix_kill($r['pid'])) {
+				cacti_log(sprintf('ERROR: Process killed due to timeout! (%s, %s, %s, %s)', $r['tasktype'], $r['taskname'], $r['taskid'], $r['pid']), false, 'POLLER');
+				posix_kill($r['pid'], SIGTERM);
+			} else {
+				cacti_log(sprintf('ERROR: Detected process that is gone and did not unregister first! (%s, %s, %s, %s)', $r['tasktype'], $r['taskname'], $r['taskid'], $r['pid']), false, 'POLLER');
+			}
+
+			unregister_process($r['tasktype'], $r['taskname'], $r['taskid']);
+		}
+	}
+}
+
