@@ -1,7 +1,7 @@
 <?php
 /*
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004-2018 The Cacti Group                                 |
+ | Copyright (C) 2004-2020 The Cacti Group                                 |
  | Portions Copyright (C) 2010 Boris Lytochkin, Sponsored by Yandex LLC    |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
@@ -23,7 +23,9 @@
  +-------------------------------------------------------------------------+
 */
 
-define('REGEXP_SNMP_TRIM', '/(hex|counter(32|64)|gauge|gauge(32|64)|float|ipaddress|string|integer):|(up|down)\(|\)$/i');
+/* trim all but hex-string:, which will return 'hex-' */
+#define('REGEXP_SNMP_TRIM', '/(counter(32|64):|gauge:|gauge(32|64):|float:|ipaddress:|string:|integer:)$/i');
+define('REGEXP_SNMP_TRIM', '/(hex|counter(32|64)|gauge|gauge(32|64)|float|ipaddress|string|integer):/i');
 
 define('SNMP_METHOD_PHP', 1);
 define('SNMP_METHOD_BINARY', 2);
@@ -43,13 +45,17 @@ if (!defined('SNMP_STRING_OUTPUT_HEX')) {
 global $banned_snmp_strings;
 $banned_snmp_strings = array('End of MIB', 'No Such', 'No more');
 
-if (!class_exists('SNMP')) {
-	include_once($config['include_path'] . '/phpsnmp/classSNMP.php');
+if ($config['php_snmp_support']) {
+	include_once($config['include_path'] . '/vendor/phpsnmp/extension.php');
+} else {
+	include_once($config['include_path'] . '/vendor/phpsnmp/classSNMP.php');
 }
 
-function cacti_snmp_session($hostname, $community, $version, $auth_user, $auth_pass,
-	$auth_proto, $priv_pass, $priv_proto, $context, $engineid,
-	$port = 161, $timeout = 500, $retries = 0, $max_oids = 10) {
+use phpsnmp\SNMP;
+
+function cacti_snmp_session($hostname, $community, $version, $auth_user = '', $auth_pass = '',
+	$auth_proto = '', $priv_pass = '', $priv_proto = '', $context = '', $engineid = '',
+	$port = 161, $timeout_ms = 500, $retries = 0, $max_oids = 10) {
 
 	switch ($version) {
 		case '1':
@@ -63,17 +69,20 @@ function cacti_snmp_session($hostname, $community, $version, $auth_user, $auth_p
 			break;
 	}
 
+	$timeout_us = (int) ($timeout_ms * 1000);
+
 	try {
-		$session = new SNMP($version, $hostname . ':' . $port, ($version == 3 ? $auth_user : $community), $timeout * 1000, $retries);
+		$session = new SNMP($version, $hostname . ':' . $port, ($version == 3 ? $auth_user : $community), $timeout_us, $retries);
 	} catch (Exception $e) {
 		return false;
 	}
 
 	if (defined('SNMP_OID_OUTPUT_NUMERIC')) {
 		$session->oid_output_format = SNMP_OID_OUTPUT_NUMERIC;
-		$session->valueretrieval = SNMP_VALUE_LIBRARY;
+		$session->valueretrieval = SNMP_VALUE_PLAIN;
 	}
 
+	$session->quick_print = false;
 	$session->max_oids = $max_oids;
 
 	if (read_config_option('oid_increasing_check_disable') == 'on') {
@@ -105,9 +114,9 @@ function cacti_snmp_session($hostname, $community, $version, $auth_user, $auth_p
 	return $session;
 }
 
-function cacti_snmp_get($hostname, $community, $oid, $version, $auth_user, $auth_pass,
-	$auth_proto, $priv_pass, $priv_proto, $context,
-	$port = 161, $timeout = 500, $retries = 0, $environ = SNMP_POLLER,
+function cacti_snmp_get($hostname, $community, $oid, $version, $auth_user = '', $auth_pass = '',
+	$auth_proto = '', $priv_pass = '', $priv_proto = '', $context = '',
+	$port = 161, $timeout_ms = 500, $retries = 0, $environ = SNMP_POLLER,
 	$engineid = '', $value_output_format = SNMP_STRING_OUTPUT_GUESS) {
 
 	global $config, $snmp_error;
@@ -115,7 +124,7 @@ function cacti_snmp_get($hostname, $community, $oid, $version, $auth_user, $auth
 	$max_oids   = 1;
 	$snmp_error = '';
 
-	if (!cacti_snmp_options_sanitize($version, $community, $port, $timeout, $retries, $max_oids)) {
+	if (!cacti_snmp_options_sanitize($version, $community, $port, $timeout_ms, $retries, $max_oids)) {
 		return 'U';
 	}
 
@@ -128,10 +137,12 @@ function cacti_snmp_get($hostname, $community, $oid, $version, $auth_user, $auth
 			snmp_set_enum_print(true);
 		}
 
+		$timeout_us = (int) ($timeout_ms * 1000);
+
 		if ($version == '1') {
-			$snmp_value = snmpget($hostname . ':' . $port, $community, $oid, ($timeout * 1000), $retries);
+			$snmp_value = @snmpget($hostname . ':' . $port, $community, $oid, $timeout_us, $retries);
 		} elseif ($version == '2') {
-			$snmp_value = snmp2_get($hostname . ':' . $port, $community, $oid, ($timeout * 1000), $retries);
+			$snmp_value = @snmp2_get($hostname . ':' . $port, $community, $oid, $timeout_us, $retries);
 		} else {
 			if ($priv_proto == '[None]' || $priv_pass == '') {
 				if ($auth_pass == '' || $auth_proto == '[None]') {
@@ -145,7 +156,7 @@ function cacti_snmp_get($hostname, $community, $oid, $version, $auth_user, $auth
 				$sec_level = 'authPriv';
 			}
 
-			$snmp_value = snmp3_get($hostname . ':' . $port, $auth_user, $sec_level, $auth_proto, $auth_pass, $priv_proto, $priv_pass, $oid, ($timeout * 1000), $retries);
+			$snmp_value = @snmp3_get($hostname . ':' . $port, $auth_user, $sec_level, $auth_proto, $auth_pass, $priv_proto, $priv_pass, $oid, $timeout_us, $retries);
 		}
 
 		if ($snmp_value === false) {
@@ -158,7 +169,7 @@ function cacti_snmp_get($hostname, $community, $oid, $version, $auth_user, $auth
 		$snmp_value = '';
 
 		/* net snmp want the timeout in seconds */
-		$timeout = ceil($timeout / 1000);
+		$timeout_s = (int) ceil($timeout_ms / 1000);
 
 		if ($version == '1') {
 			$snmp_auth = '-c ' . snmp_escape_string($community); /* v1/v2 - community string */
@@ -177,7 +188,7 @@ function cacti_snmp_get($hostname, $community, $oid, $version, $auth_user, $auth
 		exec(cacti_escapeshellcmd(read_config_option('path_snmpget')) .
 			' -O fntevU' . ($value_output_format == SNMP_STRING_OUTPUT_HEX ? 'x ':' ') . $snmp_auth .
 			' -v ' . $version .
-			' -t ' . $timeout .
+			' -t ' . $timeout_s .
 			' -r ' . $retries .
 			' '    . cacti_escapeshellarg($hostname) . ':' . $port .
 			' '    . cacti_escapeshellarg($oid), $snmp_value);
@@ -187,8 +198,8 @@ function cacti_snmp_get($hostname, $community, $oid, $version, $auth_user, $auth
 			$snmp_value = implode(' ', $snmp_value);
 		}
 
-		if (substr_count($snmp_value, 'Timeout:')) {
-			cacti_log("WARNING: SNMP Error:'Timeout', Device:'$hostname', OID:'$oid'", false);
+		if (strpos($snmp_value, 'Timeout') !== false) {
+			cacti_log("WARNING: SNMP Error:'Timeout', Device:'$hostname', OID:'$oid'", false, 'SNMP', POLLER_VERBOSITY_HIGH);
 			$snmp_value = 'U';
 		} else {
 			$snmp_value = format_snmp_string($snmp_value, false, $value_output_format);
@@ -198,9 +209,9 @@ function cacti_snmp_get($hostname, $community, $oid, $version, $auth_user, $auth
 	return $snmp_value;
 }
 
-function cacti_snmp_get_raw($hostname, $community, $oid, $version, $auth_user, $auth_pass,
-	$auth_proto, $priv_pass, $priv_proto, $context,
-	$port = 161, $timeout = 500, $retries = 0, $environ = SNMP_POLLER,
+function cacti_snmp_get_raw($hostname, $community, $oid, $version, $auth_user = '', $auth_pass = '',
+	$auth_proto = '', $priv_pass = '', $priv_proto = '', $context = '',
+	$port = 161, $timeout_ms = 500, $retries = 0, $environ = SNMP_POLLER,
 	$engineid = '', $value_output_format = SNMP_STRING_OUTPUT_GUESS) {
 
 	global $config, $snmp_error;
@@ -208,7 +219,7 @@ function cacti_snmp_get_raw($hostname, $community, $oid, $version, $auth_user, $
 	$max_oids   = 1;
 	$snmp_error = '';
 
-	if (!cacti_snmp_options_sanitize($version, $community, $port, $timeout, $retries, $max_oids)) {
+	if (!cacti_snmp_options_sanitize($version, $community, $port, $timeout_ms, $retries, $max_oids)) {
 		return 'U';
 	}
 
@@ -217,14 +228,16 @@ function cacti_snmp_get_raw($hostname, $community, $oid, $version, $auth_user, $
 		we are getting back */
 		snmp_set_quick_print(0);
 
+		$timeout_us = (int) ($timeout_ms * 1000);
+
 		if (function_exists('snmp_set_enum_print')) {
 			snmp_set_enum_print(true);
 		}
 
 		if ($version == '1') {
-			$snmp_value = snmpget($hostname . ':' . $port, $community, $oid, ($timeout * 1000), $retries);
+			$snmp_value = @snmpget($hostname . ':' . $port, $community, $oid, $timeout_us, $retries);
 		} elseif ($version == '2') {
-			$snmp_value = snmp2_get($hostname . ':' . $port, $community, $oid, ($timeout * 1000), $retries);
+			$snmp_value = @snmp2_get($hostname . ':' . $port, $community, $oid, $timeout_us, $retries);
 		} else {
 			if ($priv_proto == '[None]' || $priv_pass == '') {
 				if ($auth_pass == '' || $auth_proto == '[None]') {
@@ -238,7 +251,7 @@ function cacti_snmp_get_raw($hostname, $community, $oid, $version, $auth_user, $
 				$sec_level = 'authPriv';
 			}
 
-			$snmp_value = snmp3_get($hostname . ':' . $port, $auth_user, $sec_level, $auth_proto, $auth_pass, $priv_proto, $priv_pass, $oid, ($timeout * 1000), $retries);
+			$snmp_value = @snmp3_get($hostname . ':' . $port, $auth_user, $sec_level, $auth_proto, $auth_pass, $priv_proto, $priv_pass, $oid, $timeout_us, $retries);
 		}
 
 		if ($snmp_value === false) {
@@ -249,7 +262,7 @@ function cacti_snmp_get_raw($hostname, $community, $oid, $version, $auth_user, $
 		$snmp_value = '';
 
 		/* net snmp want the timeout in seconds */
-		$timeout = ceil($timeout / 1000);
+		$timeout_s = (int) ceil($timeout_ms / 1000);
 
 		if ($version == '1') {
 			$snmp_auth = '-c ' . snmp_escape_string($community); /* v1/v2 - community string */
@@ -268,7 +281,7 @@ function cacti_snmp_get_raw($hostname, $community, $oid, $version, $auth_user, $
 		exec(cacti_escapeshellcmd(read_config_option('path_snmpget')) .
 			' -O fntev' . ($value_output_format == SNMP_STRING_OUTPUT_HEX ? 'x ':' ') . $snmp_auth .
 			' -v ' . $version .
-			' -t ' . $timeout .
+			' -t ' . $timeout_s .
 			' -r ' . $retries .
 			' '    . cacti_escapeshellarg($hostname) . ':' . $port .
 			' '    . cacti_escapeshellarg($oid), $snmp_value);
@@ -278,8 +291,8 @@ function cacti_snmp_get_raw($hostname, $community, $oid, $version, $auth_user, $
 			$snmp_value = implode(' ', $snmp_value);
 		}
 
-		if (substr_count($snmp_value, 'Timeout:')) {
-			cacti_log("WARNING: SNMP Error:'Timeout', Device:'$hostname', OID:'$oid'", false);
+		if (strpos($snmp_value, 'Timeout') !== false) {
+			cacti_log("WARNING: SNMP Error:'Timeout', Device:'$hostname', OID:'$oid'", false, 'SNMP', POLLER_VERBOSITY_HIGH);
 			$snmp_value = 'U';
 		}
 	}
@@ -287,9 +300,9 @@ function cacti_snmp_get_raw($hostname, $community, $oid, $version, $auth_user, $
 	return $snmp_value;
 }
 
-function cacti_snmp_getnext($hostname, $community, $oid, $version, $auth_user, $auth_pass,
-	$auth_proto, $priv_pass, $priv_proto, $context,
-	$port = 161, $timeout = 500, $retries = 0, $environ = SNMP_POLLER,
+function cacti_snmp_getnext($hostname, $community, $oid, $version, $auth_user = '', $auth_pass = '',
+	$auth_proto = '', $priv_pass = '', $priv_proto = '', $context = '',
+	$port = 161, $timeout_ms = 500, $retries = 0, $environ = SNMP_POLLER,
 	$engineid = '', $value_output_format = SNMP_STRING_OUTPUT_GUESS) {
 
 	global $config, $snmp_error;
@@ -297,7 +310,7 @@ function cacti_snmp_getnext($hostname, $community, $oid, $version, $auth_user, $
 	$max_oids   = 1;
 	$snmp_error = '';
 
-	if (!cacti_snmp_options_sanitize($version, $community, $port, $timeout, $retries, $max_oids)) {
+	if (!cacti_snmp_options_sanitize($version, $community, $port, $timeout_ms, $retries, $max_oids)) {
 		return 'U';
 	}
 
@@ -306,10 +319,12 @@ function cacti_snmp_getnext($hostname, $community, $oid, $version, $auth_user, $
 		we are getting back */
 		snmp_set_quick_print(0);
 
+		$timeout_us = (int) ($timeout_ms * 1000);
+
 		if ($version == '1') {
-			$snmp_value = snmpgetnext($hostname . ':' . $port, $community, $oid, ($timeout * 1000), $retries);
+			$snmp_value = @snmpgetnext($hostname . ':' . $port, $community, $oid, $timeout_us, $retries);
 		} elseif ($version == '2') {
-			$snmp_value = snmp2_getnext($hostname . ':' . $port, $community, $oid, ($timeout * 1000), $retries);
+			$snmp_value = @snmp2_getnext($hostname . ':' . $port, $community, $oid, $timeout_us, $retries);
 		} else {
 			if ($priv_proto == '[None]' || $priv_pass == '') {
 				if ($auth_pass == '' || $auth_proto == '[None]') {
@@ -322,7 +337,7 @@ function cacti_snmp_getnext($hostname, $community, $oid, $version, $auth_user, $
 				$sec_level = 'authPriv';
 			}
 
-			$snmp_value = snmp3_getnext($hostname . ':' . $port, $auth_user, $sec_level, $auth_proto, $auth_pass, $priv_proto, $priv_pass, $oid, ($timeout * 1000), $retries);
+			$snmp_value = @snmp3_getnext($hostname . ':' . $port, $auth_user, $sec_level, $auth_proto, $auth_pass, $priv_proto, $priv_pass, $oid, $timeout_us, $retries);
 		}
 
 		if ($snmp_value === false) {
@@ -335,7 +350,7 @@ function cacti_snmp_getnext($hostname, $community, $oid, $version, $auth_user, $
 		$snmp_value = '';
 
 		/* net snmp want the timeout in seconds */
-		$timeout = ceil($timeout / 1000);
+		$timeout_s = (int) ceil($timeout_ms / 1000);
 
 		if ($version == '1') {
 			$snmp_auth = '-c ' . snmp_escape_string($community); /* v1/v2 - community string */
@@ -354,7 +369,7 @@ function cacti_snmp_getnext($hostname, $community, $oid, $version, $auth_user, $
 		exec(cacti_escapeshellcmd(read_config_option('path_snmpgetnext')) .
 			' -O fntevU' . ($value_output_format == SNMP_STRING_OUTPUT_HEX ? 'x ':' ') . $snmp_auth .
 			' -v ' . $version .
-			' -t ' . $timeout .
+			' -t ' . $timeout_s .
 			' -r ' . $retries .
 			' '    . cacti_escapeshellarg($hostname) . ':' . $port .
 			' '    . cacti_escapeshellarg($oid), $snmp_value);
@@ -364,8 +379,8 @@ function cacti_snmp_getnext($hostname, $community, $oid, $version, $auth_user, $
 			$snmp_value = implode(' ', $snmp_value);
 		}
 
-		if (substr_count($snmp_value, 'Timeout:')) {
-			cacti_log("WARNING: SNMP Error:'Timeout', Device:'$hostname', OID:'$oid'", false);
+		if (strpos($snmp_value, 'Timeout') !== false) {
+			cacti_log("WARNING: SNMP Error:'Timeout', Device:'$hostname', OID:'$oid'", false, 'SNMP', POLLER_VERBOSITY_HIGH);
 		}
 
 		/* strip out non-snmp data */
@@ -416,7 +431,7 @@ function cacti_snmp_session_walk($session, $oid, $dummy = false, $max_repetition
 	$non_repeaters = NULL, $value_output_format = SNMP_STRING_OUTPUT_GUESS) {
 
 	$info = $session->info;
-	if (is_array($oid) && sizeof($oid) == 0) {
+	if (is_array($oid) && cacti_sizeof($oid) == 0) {
 		cacti_log('Empty OID!', false);
 		return array();
 	}
@@ -432,7 +447,7 @@ function cacti_snmp_session_walk($session, $oid, $dummy = false, $max_repetition
 		$max_repetitions = 10;
 
 	try {
-		$out = $session->walk($oid, false, $max_repetitions, $non_repeaters);
+		$out = @$session->walk($oid, false, $max_repetitions, $non_repeaters);
 	} catch (Exception $e) {
 		$out = false;
 	}
@@ -445,13 +460,13 @@ function cacti_snmp_session_walk($session, $oid, $dummy = false, $max_repetition
 			$oid == '.1.3.6.1.4.1.9.9.23.1.2.1.1.6') {
 			/* do nothing */
 		} elseif ($session->getErrno() == SNMP::ERRNO_TIMEOUT) {
-			cacti_log('WARNING: SNMP Error:\'Timeout (' . ($info['timeout']/1000) . " ms)', Device:'" . $info['hostname'] . "', OID:'$oid'", false);
+			cacti_log('WARNING: SNMP Error:\'Timeout (' . ($info['timeout']/1000) . " ms)', Device:'" . $info['hostname'] . "', OID:'$oid'", false, 'SNMP', POLLER_VERBOSITY_HIGH);
 		}
 
 		return array();
 	}
 
-	if (sizeof($out)) {
+	if (cacti_sizeof($out)) {
 		foreach($out as $oid => $value){
 			$out[$oid] = format_snmp_string($value, false, $value_output_format);
 		}
@@ -460,16 +475,16 @@ function cacti_snmp_session_walk($session, $oid, $dummy = false, $max_repetition
 	return $out;
 }
 
-function cacti_snmp_session_get($session, $oid) {
+function cacti_snmp_session_get($session, $oid, $strip_alpha = false) {
 	$info = $session->info;
 
-	if (is_array($oid) && sizeof($oid) == 0) {
+	if (is_array($oid) && cacti_sizeof($oid) == 0) {
 		cacti_log('Empty OID!', false);
 		return array();
 	}
 
 	try {
-		$out = $session->get($oid);
+		$out = @$session->get($oid);
 	} catch (Exception $e) {
 		$out = false;
 	}
@@ -480,17 +495,17 @@ function cacti_snmp_session_get($session, $oid) {
 
 	if ($out === false) {
 		if ($session->getErrno() == SNMP::ERRNO_TIMEOUT) {
-			cacti_log('WARNING: SNMP Error:\'Timeout (' . ($info['timeout']/1000) . " ms)', Device:'" . $info['hostname'] . "', OID:'$oid'", false);
+			cacti_log('WARNING: SNMP Error:\'Timeout (' . round($info['timeout']/1000,0) . " ms)', Device:'" . $info['hostname'] . "', OID:'$oid'", false, 'SNMP', POLLER_VERBOSITY_HIGH);
 		}
 		return false;
 	}
 
 	if (is_array($out)) {
 		foreach($out as $oid => $value){
-			$out[$oid] = format_snmp_string($value, false);
+			$out[$oid] = format_snmp_string($value, false, SNMP_STRING_OUTPUT_GUESS, $strip_alpha);
 		}
 	} else {
-		$out = format_snmp_string($out, false);
+		$out = format_snmp_string($out, false, SNMP_STRING_OUTPUT_GUESS, $strip_alpha);
 	}
 
 	return $out;
@@ -498,13 +513,13 @@ function cacti_snmp_session_get($session, $oid) {
 
 function cacti_snmp_session_getnext($session, $oid) {
 	$info = $session->info;
-	if (is_array($oid) && sizeof($oid) == 0) {
+	if (is_array($oid) && cacti_sizeof($oid) == 0) {
 		cacti_log('Empty OID!', false);
 		return array();
 	}
 
 	try {
-		$out = $session->getnext($oid);
+		$out = @$session->getnext($oid);
 	} catch (Exception $e) {
 		$out = false;
 	}
@@ -513,7 +528,7 @@ function cacti_snmp_session_getnext($session, $oid) {
 		$oid = implode(',', $oid);
 	} elseif ($out === false) {
 		if ($session->getErrno() == SNMP::ERRNO_TIMEOUT) {
-			cacti_log('WARNING: SNMP Error:\'Timeout (' . ($info['timeout']/1000) . " ms)', Device:'" . $info['hostname'] . "', OID:'$oid'", false);
+			cacti_log('WARNING: SNMP Error:\'Timeout (' . round($info['timeout']/1000, 0) . " ms)', Device:'" . $info['hostname'] . "', OID:'$oid'", false, 'SNMP', POLLER_VERBOSITY_HIGH);
 		}
 
 		return false;
@@ -530,9 +545,9 @@ function cacti_snmp_session_getnext($session, $oid) {
 	return $out;
 }
 
-function cacti_snmp_walk($hostname, $community, $oid, $version, $auth_user, $auth_pass,
-	$auth_proto, $priv_pass, $priv_proto, $context,
-	$port = 161, $timeout = 500, $retries = 0, $max_oids = 10, $environ = SNMP_POLLER,
+function cacti_snmp_walk($hostname, $community, $oid, $version, $auth_user = '', $auth_pass = '',
+	$auth_proto = '', $priv_pass = '', $priv_proto = '', $context = '',
+	$port = 161, $timeout_ms = 500, $retries = 0, $max_oids = 10, $environ = SNMP_POLLER,
 	$engineid = '', $value_output_format = SNMP_STRING_OUTPUT_GUESS) {
 
 	global $config, $banned_snmp_strings, $snmp_error;
@@ -543,7 +558,7 @@ function cacti_snmp_walk($hostname, $community, $oid, $version, $auth_user, $aut
 	$snmp_array        = array();
 	$temp_array        = array();
 
-	if (!cacti_snmp_options_sanitize($version, $community, $port, $timeout, $retries, $max_oids)) {
+	if (!cacti_snmp_options_sanitize($version, $community, $port, $timeout_ms, $retries, $max_oids)) {
 		return array();
 	}
 
@@ -553,6 +568,8 @@ function cacti_snmp_walk($hostname, $community, $oid, $version, $auth_user, $aut
 		/* make sure snmp* is verbose so we can see what types of data
 		we are getting back */
 
+		$timeout_us = (int) ($timeout_ms * 1000);
+
 		/* force php to return numeric oid's */
 		cacti_oid_numeric_format();
 
@@ -563,9 +580,9 @@ function cacti_snmp_walk($hostname, $community, $oid, $version, $auth_user, $aut
 		snmp_set_quick_print(0);
 
 		if ($version == '1') {
-			$temp_array = snmprealwalk($hostname . ':' . $port, $community, $oid, ($timeout * 1000), $retries);
+			$temp_array = snmprealwalk($hostname . ':' . $port, $community, $oid, $timeout_us, $retries);
 		} elseif ($version == 2) {
-			$temp_array = snmp2_real_walk($hostname . ':' . $port, $community, $oid, ($timeout * 1000), $retries);
+			$temp_array = snmp2_real_walk($hostname . ':' . $port, $community, $oid, $timeout_us, $retries);
 		} else {
 			if ($priv_proto == '[None]' || $priv_pass == '') {
 				if ($auth_pass == '') {
@@ -578,11 +595,11 @@ function cacti_snmp_walk($hostname, $community, $oid, $version, $auth_user, $aut
 				$sec_level = 'authPriv';
 			}
 
-			$temp_array = snmp3_real_walk($hostname . ':' . $port, $auth_user, $sec_level, $auth_proto, $auth_pass, $priv_proto, $priv_pass, $oid, ($timeout * 1000), $retries);
+			$temp_array = @snmp3_real_walk($hostname . ':' . $port, $auth_user, $sec_level, $auth_proto, $auth_pass, $priv_proto, $priv_pass, $oid, $timeout_us, $retries);
 		}
 
 		/* check for bad entries */
-		if ($temp_array !== false && sizeof($temp_array)) {
+		if ($temp_array !== false && cacti_sizeof($temp_array)) {
 			foreach($temp_array as $key => $value) {
 				foreach($banned_snmp_strings as $item) {
 					if (strstr($value, $item) != '') {
@@ -603,7 +620,7 @@ function cacti_snmp_walk($hostname, $community, $oid, $version, $auth_user, $aut
 		}
 	} else {
 		/* ucd/net snmp want the timeout in seconds */
-		$timeout = ceil($timeout / 1000);
+		$timeout_s = (int) ceil($timeout_ms / 1000);
 
 		if ($version == '1') {
 			$snmp_auth = '-c ' . snmp_escape_string($community); /* v1/v2 - community string */
@@ -620,11 +637,13 @@ function cacti_snmp_walk($hostname, $community, $oid, $version, $auth_user, $aut
 			$oidCheck = '';
 		}
 
+		$max_oids = read_config_option('snmp_bulk_walk_size');
+
 		if (file_exists($path_snmpbulkwalk) && ($version > 1) && ($max_oids > 1)) {
 			$temp_array = exec_into_array(cacti_escapeshellcmd($path_snmpbulkwalk) .
 				' -O QnU'  . ($value_output_format == SNMP_STRING_OUTPUT_HEX ? 'x ':' ') . $snmp_auth .
 				' -v '     . $version .
-				' -t '     . $timeout .
+				' -t '     . $timeout_s .
 				' -r '     . $retries .
 				' -Cr'     . $max_oids .
 				' '        . $oidCheck . ' ' .
@@ -632,21 +651,21 @@ function cacti_snmp_walk($hostname, $community, $oid, $version, $auth_user, $aut
 				cacti_escapeshellarg($oid));
 		} else {
 			$temp_array = exec_into_array(cacti_escapeshellcmd(read_config_option('path_snmpwalk')) .
-				' -O QnU ' . ($value_output_format == SNMP_STRING_OUTPUT_HEX ? 'x ':' ') . $snmp_auth .
+				' -O QnU' . ($value_output_format == SNMP_STRING_OUTPUT_HEX ? 'x ':' ') . $snmp_auth .
 				' -v '     . $version .
-				' -t '     . $timeout .
+				' -t '     . $timeout_s .
 				' -r '     . $retries .
 				' '        . $oidCheck . ' ' .
 				' '        . cacti_escapeshellarg($hostname) . ':' . $port .
 				' '        . cacti_escapeshellarg($oid));
 		}
 
-		if (substr_count(implode(' ', $temp_array), 'Timeout:')) {
-			cacti_log("WARNING: SNMP Error:'Timeout', Device:'$hostname', OID:'$oid'", false);
+		if (strpos(implode(' ', $temp_array), 'Timeout') !== false) {
+			cacti_log("WARNING: SNMP Error:'Timeout', Device:'$hostname', OID:'$oid'", false, 'SNMP', POLLER_VERBOSITY_HIGH);
 		}
 
 		/* check for bad entries */
-		if (is_array($temp_array) && sizeof($temp_array)) {
+		if (is_array($temp_array) && cacti_sizeof($temp_array)) {
 			foreach($temp_array as $key => $value) {
 				foreach($banned_snmp_strings as $item) {
 					if (strstr($value, $item) != '') {
@@ -672,7 +691,7 @@ function cacti_snmp_walk($hostname, $community, $oid, $version, $auth_user, $aut
 	return $snmp_array;
 }
 
-function format_snmp_string($string, $snmp_oid_included, $value_output_format = SNMP_STRING_OUTPUT_GUESS) {
+function format_snmp_string($string, $snmp_oid_included, $value_output_format = SNMP_STRING_OUTPUT_GUESS, $strip_alpha = false) {
 	global $banned_snmp_strings;
 
 	$string = preg_replace(REGEXP_SNMP_TRIM, '', trim($string));
@@ -681,7 +700,7 @@ function format_snmp_string($string, $snmp_oid_included, $value_output_format =
 		/* strip off all leading junk (the oid and stuff) */
 		$string_array = explode('=', $string);
 
-		if (sizeof($string_array) == 1) {
+		if (cacti_sizeof($string_array) == 1) {
 			/* trim excess first */
 			$string = trim($string);
 		} elseif ((substr($string, 0, 1) == '.') || (strpos($string, '::') !== false)) {
@@ -695,6 +714,9 @@ function format_snmp_string($string, $snmp_oid_included, $value_output_format =
 		$string = trim($string);
 	}
 
+	/* remove quotes and extranious data */
+	$string = trim($string, " \n\r\v\"'");
+
 	/* return the easiest value */
 	if ($string == '') {
 		return $string;
@@ -719,88 +741,133 @@ function format_snmp_string($string, $snmp_oid_included, $value_output_format =
 	}
 
 	/* Remove invalid chars, if the string output is to be numeric */
-	if ($value_output_format == SNMP_STRING_OUTPUT_GUESS) {
-		$k = strlen($string);
-		for ($i=0; $i < $k; $i++) {
-			if ((ord($string[$i]) <= 31) || (ord($string[$i]) >= 127)) {
-				$string[$i] = ' ';
+	if ($strip_alpha && $value_output_format == SNMP_STRING_OUTPUT_GUESS) {
+		$string = trim(str_ireplace('hex:', '', $string));
+		$len    = strlen($string);
+		$pos    = $len - 1;
+
+		while ($pos > 0) {
+			$value = ord($string[$pos]);
+
+			if (($value < 48 || $value > 57) && $value != 32) {
+				$string[$pos] = ' ';
+			} else {
+				break;
 			}
+
+			$pos--;
 		}
+
+		$string = trim($string);
+		$len    = strlen($string);
+		$pos    = 0;
+
+		while ($pos < $len) {
+			$value = ord($string[$pos]);
+
+			if (($value < 48 || $value > 57) && $value != 32) {
+				$string[$pos] = ' ';
+			} else {
+				break;
+			}
+
+			$pos++;
+		}
+
+		$string = trim($string);
+
+		if ($string == '') {
+			return 'U';
+		}
+	}
+
+	/* Remove non-printable characters, allow UTF-8 */
+	if ($value_output_format == SNMP_STRING_OUTPUT_GUESS) {
+		$string = preg_replace('/[^[:print:]\r\n]/', '', $string);
 	}
 
 	/* Trim the string of trailing and leading spaces */
 	$string = trim($string);
 
 	/* convert hex strings to numeric values */
-	if (is_hex_string($string) && $value_output_format == SNMP_STRING_OUTPUT_GUESS) {
+	if (is_hex_string($string)) {
+		/* the is_hex_string() function will remove the hex:
+		 * and hex-string: from the passed value
+		 */
 		$output = '';
+
 		$parts  = explode(' ', $string);
 
-		/* convert the hex string into an ascii string */
-		foreach($parts as $part) {
-			$output .= chr(hexdec($part));
-		}
+		if (cacti_sizeof($parts) == 4) {
+			$possible_ip = true;
 
-		if (is_numeric($output)) {
-			$string = number_format($output, 0, '', '');
-		} else {
-			$string = $output;
-		}
-	} elseif (preg_match('/hex-/i', $string)) {
-		$output = '';
+			$ip_address = '';
 
-		/* strip off the 'Hex-STRING:' */
-		$string = preg_replace('/hex- ?/i', '', $string);
+			/* convert the hex string into an ascii string */
+			foreach($parts as $part) {
+				if ($possible_ip && hexdec($part) >= 0 && hexdec($part) <= 255) {
+					$ip_address .= ($ip_address != '' ? '.':'') . hexdec($part);
+				} else {
+					$possible_ip = false;
+				}
 
-		/* normalize some forms */
-		$string = str_replace(array(' ', '-', '.'), ':', $string);
-		$parts  = explode(':', $string);
+				$output .= chr(hexdec($part));
+			}
 
-		if (is_mac_address($string)) {
-			$mac = true;
-		} else {
-			$mac = false;
-		}
+			if ($possible_ip && is_ipaddress($ip_address)) {
+				$string = $ip_address;
+			} else {
+				$string = $output;
+			}
+		/* hex string is mac-address */
+		} elseif (cacti_sizeof($parts) == 6) {
+			$possible_ip = false;
 
-		/* convert the hex string into an ascii string */
-		foreach($parts as $part) {
-			if ($mac == false) {
+			/* convert the hex string into an ascii string */
+			foreach($parts as $part) {
 				$output .= ($output != '' ? ':' : '');
 				if ($part == '00') {
 					$output .= '00';
 				} else  {
 					$output .= str_pad($part, 2, '0', STR_PAD_LEFT);
 				}
+			}
+
+			if (is_numeric($output)) {
+				$string = number_format($output, 0, '', '');
 			} else {
-				$output .= ($output != '' ? ':' : '') . $part;
+				$string = $output;
 			}
-		}
-
-		if (is_numeric($output)) {
-			$string = number_format($output, 0, '', '');
 		} else {
-			$string = $output;
+			$possible_ip = false;
 		}
-	} elseif (preg_match('/(hex:\?)?([a-fA-F0-9]{1,2}(:|\s)){5}/i', $string)) {
-		$octet = '';
+	} elseif (substr(strtolower($string), 0, 4) == 'hex:') {
+		$output = '';
 
-		/* strip off the 'hex:' */
-		$string = preg_replace('/hex: ?/i', '', $string);
+		/* strip off the 'Hex:' */
+		$string = trim(str_ireplace('hex:', '', $string));
 
-		/* split the hex on the delimiter */
-		$octets = preg_split('/\s|:/', $string);
+		/* normalize some forms */
+		$string = str_replace(array(' ', '-', '.'), ':', $string);
+		$parts  = explode(':', $string);
 
-		/* loop through each octet and format it accordingly */
-		for ($i=0;($i<count($octets));$i++) {
-			$octet .= str_pad($octets[$i], 2, '0', STR_PAD_LEFT);
+		if (!is_mac_address($string)) {
+			/* convert the hex string into an ascii string */
+			foreach($parts as $part) {
+				$output .= ($output != '' ? ':' : '');
+				if ($part == '00') {
+					$output .= '00';
+				} else  {
+					$output .= str_pad($part, 2, '0', STR_PAD_LEFT);
+				}
+			}
 
-			if (($i+1) < count($octets)) {
-				$octet .= ':';
+			if (is_numeric($output)) {
+				$string = number_format($output, 0, '', '');
+			} else {
+				$string = $output;
 			}
 		}
-
-		/* copy the final result and make it upper case */
-		$string = strtoupper($octet);
 	} elseif (preg_match('/Timeticks:\s\((\d+)\)\s/', $string, $matches)) {
 		$string = $matches[1];
 	}
@@ -834,9 +901,13 @@ function snmp_escape_string($string) {
 }
 
 function snmp_get_method($type = 'walk', $version = 1, $context = '', $engineid = '',
-	$value_output_format = SNMP_STRING_OUTPUT_GUESS) {
+    $value_output_format = SNMP_STRING_OUTPUT_GUESS) {
 
-	if ($value_output_format == SNMP_STRING_OUTPUT_HEX) {
+	global $config;
+
+	if (isset($config['php_snmp_support']) && !$config['php_snmp_support']) {
+		return SNMP_METHOD_BINARY;
+	} elseif ($value_output_format == SNMP_STRING_OUTPUT_HEX) {
 		return SNMP_METHOD_BINARY;
 	} elseif ($version == 3 && $context != '') {
 		return SNMP_METHOD_BINARY;
@@ -874,8 +945,6 @@ function cacti_snmp_options_sanitize($version, $community, &$port, &$timeout, &$
 		$port = '161';
 	}
 
-	//cacti_log('Version:' . $version . ', Port:' . $port . ', MaxOIDS:' . $max_oids . ', Retries:' . $retries . ', Timeout:' . $timeout);
-
 	/* do not attempt to poll invalid combinations */
 	if (($version == 0) || (!is_numeric($version)) ||
 		(!is_numeric($max_oids)) ||
