@@ -519,14 +519,18 @@ function poller_update_poller_cache_from_buffer($local_data_ids, &$poller_items,
 		$ids = implode(', ', $local_data_ids);
 
 		if ($ids != '') {
-			db_execute("UPDATE poller_item
+			db_execute_prepared("UPDATE poller_item
 				SET present=0
-				WHERE local_data_id IN ($ids)");
+				WHERE poller_id = ?
+				AND local_data_id IN ($ids)",
+				array($poller_id));
 
 			if (($rcnn_id = poller_push_to_remote_db_connect($poller_id, true)) !== false) {
-				db_execute("UPDATE poller_item
+				db_execute_prepared("UPDATE poller_item
 					SET present=0
-					WHERE local_data_id IN ($ids)", true, $rcnn_id);
+					WHERE poller_id = ?
+					AND local_data_id IN ($ids)",
+					array($poller_id), true, $rcnn_id);
 			}
 		}
 	} else {
@@ -602,14 +606,18 @@ function poller_update_poller_cache_from_buffer($local_data_ids, &$poller_items,
 
 	/* remove stale records FROM the poller cache */
 	if ($ids != '') {
-		db_execute("DELETE FROM poller_item
-			WHERE present=0
-			AND local_data_id IN ($ids)");
+		db_execute_prepared("DELETE FROM poller_item
+			WHERE present = 0
+			AND poller_id = ?
+			AND local_data_id IN ($ids)",
+			array($poller_id));
 
 		if (($rcnn_id = poller_push_to_remote_db_connect($poller_id, true)) !== false) {
-			db_execute("DELETE FROM poller_item
-				WHERE present=0
-				AND local_data_id IN ($ids)", true, $rcnn_id);
+			db_execute_prepared("DELETE FROM poller_item
+				WHERE present = 0
+				AND poller_id = ?
+				AND local_data_id IN ($ids)",
+				array($poller_id), true, $rcnn_id);
 		}
 	} else {
 		/* only handle explicitely given local_data_ids */
@@ -869,13 +877,19 @@ function utilities_get_mysql_recommendations() {
 				'value' => 'utf8mb4_unicode_ci',
 				'class' => 'warning',
 				'measure' => 'equal',
-				'comment' => __('When using Cacti with languages other than English, it is important to use the utf8_general_ci collation type as some characters take more than a single byte.  If you are first just now installing Cacti, stop, make the changes and start over again.  If your Cacti has been running and is in production, see the internet for instructions on converting your databases and tables if you plan on supporting other languages.')
+				'comment' => __('When using Cacti with languages other than English, it is important to use the utf8mb4_unicode_ci collation type as some characters take more than a single byte.')
+				),
+			'character_set_server' => array(
+				'value' => 'utf8mb4',
+				'class' => 'warning',
+				'measure' => 'equal',
+				'comment' => __('When using Cacti with languages other than English, it is important to use the utf8mb4 character set as some characters take more than a single byte.')
 				),
 			'character_set_client' => array(
 				'value' => 'utf8mb4',
 				'class' => 'warning',
 				'measure' => 'equal',
-				'comment' => __('When using Cacti with languages other than English, it is important to use the utf8 character set as some characters take more than a single byte. If you are first just now installing Cacti, stop, make the changes and start over again. If your Cacti has been running and is in production, see the internet for instructions on converting your databases and tables if you plan on supporting other languages.')
+				'comment' => __('When using Cacti with languages other than English, it is important to use the utf8mb4 character set as some characters take more than a single byte.')
 				)
 		);
 	} else {
@@ -900,6 +914,12 @@ function utilities_get_mysql_recommendations() {
 				'measure' => 'equal',
 				'comment' => __('When using Cacti with languages other than English, it is important to use the utf8mb4_unicode_ci collation type as some characters take more than a single byte.')
 				),
+			'character_set_server' => array(
+				'value' => 'utf8mb4',
+				'class' => 'warning',
+				'measure' => 'equal',
+				'comment' => __('When using Cacti with languages other than English, it is important to use the utf8mb4 character set as some characters take more than a single byte.')
+				),
 			'character_set_client' => array(
 				'value' => 'utf8mb4',
 				'class' => 'warning',
@@ -1004,7 +1024,7 @@ function utilities_get_mysql_recommendations() {
 					'comment' => __('With modern SSD type storage, having multiple io threads is advantageous for applications with high io characteristics.')
 					)
 			);
-		} else {
+		} elseif (version_compare($variables['innodb_version'], '10.5', '<')) {
 			$recommendations += array(
 				'innodb_flush_log_at_timeout' => array(
 					'value'   => '3',
@@ -1046,6 +1066,42 @@ function utilities_get_mysql_recommendations() {
 					'comment' => __('If you have SSD disks, use this suggestion. Otherwise, do not set this setting.')
 					)
 			);
+		} else {
+			$recommendations += array(
+				'innodb_flush_log_at_timeout' => array(
+					'value'   => '3',
+					'measure'  => 'ge',
+					'comment'  => __('As of %s %s, the you can control how often %s flushes transactions to disk.  The default is 1 second, but in high I/O systems setting to a value greater than 1 can allow disk I/O to be more sequential', $database, $version, $database),
+					),
+				'innodb_read_io_threads' => array(
+					'value'   => '32',
+					'measure' => 'ge',
+					'comment' => __('With modern SSD type storage, having multiple read io threads is advantageous for applications with high io characteristics.')
+					),
+				'innodb_write_io_threads' => array(
+					'value'   => '16',
+					'measure' => 'ge',
+					'comment' => __('With modern SSD type storage, having multiple write io threads is advantageous for applications with high io characteristics.')
+					),
+				'innodb_io_capacity' => array(
+					'value' => '5000',
+					'measure' => 'ge',
+					'class' => 'warning',
+					'comment' => __('If you have SSD disks, use this suggestion.  If you have physical hard drives, use 200 * the number of active drives in the array.  If using NVMe or PCIe Flash, much larger numbers as high as 100000 can be used.')
+					),
+				'innodb_io_capacity_max' => array(
+					'value' => '10000',
+					'measure' => 'ge',
+					'class' => 'warning',
+					'comment' => __('If you have SSD disks, use this suggestion.  If you have physical hard drives, use 2000 * the number of active drives in the array.  If using NVMe or PCIe Flash, much larger numbers as high as 200000 can be used.')
+					),
+				'innodb_flush_neighbor_pages' => array(
+					'value' => 'none',
+					'measure' => 'eq',
+					'class' => 'warning',
+					'comment' => __('If you have SSD disks, use this suggestion. Otherwise, do not set this setting.')
+					)
+			);
 
 			unset($recommendations['innodb_additional_mem_pool_size']);
 		}
@@ -1090,7 +1146,7 @@ function utilities_get_mysql_recommendations() {
 			switch($r['measure']) {
 			case 'gem':
 				$compare = '>=';
-				$value_display = ($variables[$name]/1024/1024) . 'M';
+				$value_display = ($variables[$name]/1024/1024) . ' M';
 				$value = trim($r['value'], 'M') * 1024 * 1024;
 				if ($variables[$name] < $value) {
 					$passed = false;
@@ -1130,8 +1186,8 @@ function utilities_get_mysql_recommendations() {
 
 				$compare = '>=';
 				$passed = ($variables[$name] >= ($r['value']*$totalMem/100));
-				$value_display = round($variables[$name]/1024/1024,0) . 'M';
-				$value_recommend = round($r['value']*$totalMem/100/1024/1024,0) . 'M';
+				$value_display = round($variables[$name]/1024/1024, 2) . ' M';
+				$value_recommend = round($r['value']*$totalMem/100/1024/1024, 2) . ' M';
 				break;
 			case 'pinst':
 				$compare = '>=';
@@ -1258,7 +1314,7 @@ function utilities_get_system_memory() {
 		exec('wmic os get TotalVisibleMemorySize', $memInfo['TotalVisibleMemorySize']);
 		if (cacti_sizeof($memInfo)) {
 			foreach ($memInfo as $key => $values) {
-				$memInfo[$key] = $values[1];
+				$memInfo[$key] = $values[1] * 1000;
 			}
 		}
 	} else {
@@ -1475,7 +1531,7 @@ function utility_php_verify_recommends(&$recommends, $source) {
 }
 
 function utility_php_set_recommends_text(&$recs) {
-	if (is_array($recs) && sizeof($recs)) {
+	if (is_array($recs) && cacti_sizeof($recs)) {
 		foreach ($recs as $name => $recommends) {
 			if (cacti_sizeof($recommends)) {
 				foreach ($recommends as $index => $recommend) {
