@@ -1,7 +1,7 @@
 <?php
 /*
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004-2018 The Cacti Group                                 |
+ | Copyright (C) 2004-2020 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU General Public License             |
@@ -62,10 +62,11 @@ case 'move_page_down':
 
 	break;
 case 'save':
-	$save['id']            = isset_request_var('id') ? get_filter_request_var('id'):0;
-	$save['title']         = form_input_validate(get_nfilter_request_var('title'), 'title', '', false, 3);
-	$save['style']         = get_nfilter_request_var('style');
-	$save['enabled']       = (isset_request_var('enabled') ? 'on':'');
+	$save['id']      = isset_request_var('id') ? get_filter_request_var('id'):0;
+	$save['title']   = form_input_validate(get_nfilter_request_var('title'), 'title', '', false, 3);
+	$save['style']   = get_nfilter_request_var('style');
+	$save['enabled'] = (isset_request_var('enabled') ? 'on':'');
+	$save['refresh'] = form_input_validate(get_nfilter_request_var('refresh'), 'refresh', '^[0-9]+$', false, 3);
 
 	if (preg_match('/^((((ht|f)tp(s?))\:\/\/){1}\S+)/i', get_nfilter_request_var('fileurl')) && get_nfilter_request_var('filename') == '0') {
 		$save['contentfile'] = get_nfilter_request_var('fileurl');
@@ -140,16 +141,16 @@ function form_actions() {
 		$selected_items = sanitize_unserialize_selected_items(get_nfilter_request_var('selected_items'));
 
 		if ($selected_items != false) {
-			if ($_POST['drp_action'] == '3') { // Enable Page
-				for ($i=0;($i<count($selected_items));$i++) {
+			if (get_request_var('drp_action') == '3') { // Enable Page
+				for ($i=0;($i<cacti_count($selected_items));$i++) {
 					db_execute_prepared("UPDATE external_links SET enabled='on' WHERE id = ?", array($selected_items[$i]));
 				}
-			} elseif ($_POST['drp_action'] == '2') { // Disable Page
-				for ($i=0;($i<count($selected_items));$i++) {
+			} elseif (get_request_var('drp_action') == '2') { // Disable Page
+				for ($i=0;($i<cacti_count($selected_items));$i++) {
 					db_execute_prepared("UPDATE external_links SET enabled='' WHERE id = ?", array($selected_items[$i]));
 				}
-			} elseif ($_POST['drp_action'] == '1') { // Delete Page
-				for ($i=0;($i<count($selected_items));$i++) {
+			} elseif (get_request_var('drp_action') == '1') { // Delete Page
+				for ($i=0;($i<cacti_count($selected_items));$i++) {
 					db_execute_prepared('DELETE FROM external_links WHERE id = ?', array($selected_items[$i]));
 					db_execute_prepared('DELETE FROM user_auth_realm WHERE realm_id = ?', array($selected_items[$i]+10000));
 					db_execute_prepared('DELETE FROM user_auth_group_realm WHERE realm_id = ?', array($selected_items[$i]+10000));
@@ -171,7 +172,7 @@ function form_actions() {
 			input_validate_input_number($matches[1]);
 			/* ==================================================== */
 
-			$page_list .= '<li>' . htmlspecialchars(db_fetch_cell_prepared('SELECT title FROM external_links WHERE id = ?', array($matches[1]))) . '</li>';
+			$page_list .= '<li>' . html_escape(db_fetch_cell_prepared('SELECT title FROM external_links WHERE id = ?', array($matches[1]))) . '</li>';
 			$pages[$i] = $matches[1];
 
 			$i++;
@@ -184,7 +185,7 @@ function form_actions() {
 
 	html_start_box($actions[get_request_var_post('drp_action')], '60%', '', '3', 'center', '');
 
-	if (isset($pages) && sizeof($pages)) {
+	if (isset($pages) && cacti_sizeof($pages)) {
 		if (get_request_var('drp_action') == '3') { // Enable Pages
 			print "<tr>
 				<td colspan='2' class='textArea'>
@@ -193,7 +194,7 @@ function form_actions() {
 				</td>
 			</tr>\n";
 
-			$save_html = "<input type='button' value='" . __('Cancel') . "' onClick='cactiReturnTo()'>&nbsp;<input type='submit' value='" . __('Continue') . "' title='" . __esc('Enable Page(s)') . "'>";
+			$save_html = "<input type='button' class='ui-button ui-corner-all ui-widget' value='" . __('Cancel') . "' onClick='cactiReturnTo()'>&nbsp;<input type='submit' class='ui-button ui-corner-all ui-widget' value='" . __('Continue') . "' title='" . __esc('Enable Page(s)') . "'>";
 		} elseif (get_request_var('drp_action') == '2') { // Disable Pages
 			print "<tr>
 				<td colspan='2' class='textArea'>
@@ -202,7 +203,7 @@ function form_actions() {
 				</td>
 			</tr>\n";
 
-			$save_html = "<input type='button' value='" . __('Cancel') . "' onClick='cactiReturnTo()'>&nbsp;<input type='submit' value='" . __('Continue') . "' title='" . __esc('Disable Page(s)') . "'>";
+			$save_html = "<input type='button' class='ui-button ui-corner-all ui-widget' value='" . __('Cancel') . "' onClick='cactiReturnTo()'>&nbsp;<input type='submit' class='ui-button ui-corner-all ui-widget' value='" . __('Continue') . "' title='" . __esc('Disable Page(s)') . "'>";
 		} elseif (get_request_var('drp_action') == '1') { // Delete Pages
 			print "<tr>
 				<td colspan='2' class='textArea'>
@@ -211,11 +212,12 @@ function form_actions() {
 				</td>
 			</tr>\n";
 
-			$save_html = "<input type='button' value='" . __('Cancel') . "' onClick='cactiReturnTo()'>&nbsp;<input type='submit' value='" . __('Continue') . "' title='" . __esc('Delete Page(s)') . "'>";
+			$save_html = "<input type='button' class='ui-button ui-corner-all ui-widget' value='" . __('Cancel') . "' onClick='cactiReturnTo()'>&nbsp;<input type='submit' class='ui-button ui-corner-all ui-widget' value='" . __('Continue') . "' title='" . __esc('Delete Page(s)') . "'>";
 		}
 	} else {
-		print "<tr><td><span class='textError'>" . __('You must select at least one page.') . "</span></td></tr>\n";
-		$save_html = "<input type='button' value='" . __('Return') . "' onClick='cactiReturnTo()'>";
+		raise_message(40);
+		header('Location: links.php?header=false');
+		exit;
 	}
 
 	print "<tr class='saveRow'>
@@ -249,10 +251,9 @@ function pages() {
 			'default' => '1'
 			),
 		'filter' => array(
-			'filter' => FILTER_CALLBACK,
+			'filter' => FILTER_DEFAULT,
 			'pageset' => true,
-			'default' => '',
-			'options' => array('options' => 'sanitize_search_string')
+			'default' => ''
 			),
 		'sort_column' => array(
 			'filter' => FILTER_CALLBACK,
@@ -309,7 +310,7 @@ function pages() {
 						<?php print __('Search');?>
 					</td>
 					<td>
-						<input type='textbox' id='filter' size='25' value='<?php print html_escape_request_var('filter');?>'>
+						<input type='text' class='ui-state-default ui-corner-all' id='filter' size='25' value='<?php print html_escape_request_var('filter');?>'>
 					</td>
 					<td>
 						<?php print __('Links');?>
@@ -326,8 +327,8 @@ function pages() {
 					</td>
 					<td>
 						<span>
-							<input type='button' id='refresh' value='<?php print __('Go');?>' title='<?php print __esc('Apply Filter');?>' onClick='applyFilter()'>
-							<input type='button' id='clear' value='<?php print __('Clear');?>' title='<?php print __esc('Reset filters');?>' onClick='clearFilter()'>
+							<input type='button' class='ui-button ui-corner-all ui-widget' id='refresh' value='<?php print __('Go');?>' title='<?php print __esc('Apply Filter');?>' onClick='applyFilter()'>
+							<input type='button' class='ui-button ui-corner-all ui-widget' id='clear' value='<?php print __('Clear');?>' title='<?php print __esc('Reset filters');?>' onClick='clearFilter()'>
 						</span>
 					</td>
 				</tr>
@@ -347,7 +348,7 @@ function pages() {
 	);
 
 	if (get_request_var('filter') != '') {
-		$sql_where = " WHERE title LIKE '%" . get_request_var('filter') . "%' OR contentfile LIKE '%" . get_request_var('filter') . "%'";
+		$sql_where = ' WHERE title LIKE ' . db_qstr('%' . get_request_var('filter') . '%') . ' OR contentfile LIKE ' . db_qstr('%' . get_request_var('filter') . '%');
 	} else {
 		$sql_where = '';
 	}
@@ -362,9 +363,11 @@ function pages() {
 		$sql_order
 		$sql_limit");
 
-	$total_rows = db_fetch_cell('SELECT COUNT(*) FROM external_links');
+	$total_rows = db_fetch_cell("SELECT COUNT(*)
+		FROM external_links
+		$sql_where");
 
-	form_start('links.php');
+	form_start('links.php', 'chk');
 
 	$nav = html_nav_bar('links.php', MAX_DISPLAY_PAGES, get_request_var_request('page'), $rows, $total_rows, 8, __('External Links'), 'page', 'main');
 
@@ -384,33 +387,33 @@ function pages() {
 	html_header_sort_checkbox($display_text, get_request_var('sort_column'), get_request_var('sort_direction'));
 
 	$i = 0;
-	if (sizeof($pages)) {
+	if (cacti_sizeof($pages)) {
 		foreach ($pages as $page) {
 			form_alternate_row('line' . $page['id']);
 
-			$actions = '<a class="pic"  href="' . htmlspecialchars('links.php?action=edit&id='.$page['id']) . '" title="' . __esc('Edit Page') . '"><img src="' . $config['url_path'] . 'images/application_edit.png" alt=""></a>';
+			$actions = '<a class="pic"  href="' . html_escape('links.php?action=edit&id='.$page['id']) . '" title="' . __esc('Edit Page') . '"><img src="' . $config['url_path'] . 'images/application_edit.png" alt=""></a>';
 
 			if ($page['enabled'] == 'on') {
-				$actions .= '<a class="pic" href="' . htmlspecialchars('link.php?id=' . $page['id']) . '" title="' . __esc('View Page') . '"><img src="' . $config['url_path'] . 'images/view_page.png" alt=""></a>';
+				$actions .= '<a class="pic" href="' . html_escape('link.php?id=' . $page['id']) . '" title="' . __esc('View Page') . '"><img src="' . $config['url_path'] . 'images/view_page.png" alt=""></a>';
 			}
 
 			form_selectable_cell($actions, $page['id'], '50');
-			form_selectable_cell(htmlspecialchars($page['contentfile']), $page['id']);
-			form_selectable_cell(htmlspecialchars($page['title']), $page['id']);
-			form_selectable_cell(htmlspecialchars($style_translate[$page['style']]) . ($page['style'] == 'CONSOLE' ? ' ( ' . ($page['extendedstyle'] == '' ? 'External Links':$page['extendedstyle']) . ' )':''), $page['id']);
-			form_selectable_cell(($page['enabled'] == 'on' ? 'Yes':'No'), $page['id']);
+			form_selectable_ecell($page['contentfile'], $page['id']);
+			form_selectable_ecell($page['title'], $page['id']);
+			form_selectable_ecell($style_translate[$page['style']] . ($page['style'] == 'CONSOLE' ? ' ( ' . ($page['extendedstyle'] == '' ? __('External Links'):$page['extendedstyle']) . ' )':''), $page['id']);
+			form_selectable_cell(($page['enabled'] == 'on' ? __('Yes'):__('No')), $page['id']);
 
 			if (get_request_var('sort_column') == 'sortorder') {
 				if ($i != 0) {
-					$sort = '<a class="pic fa fa-caret-up moveArrow" href="' . htmlspecialchars('links.php?action=move_page_up&order=' . $page['sortorder'] . '&id='.$page['id']) . '"></a>';
+					$sort = '<a class="pic fa fa-caret-up moveArrow" href="' . html_escape('links.php?action=move_page_up&order=' . $page['sortorder'] . '&id='.$page['id']) . '"></a>';
 				} else {
 					$sort = '<span class="moveArrowNone"></span>';
 				}
 
-				if ($i == sizeof($pages)-1) {
+				if ($i == cacti_sizeof($pages)-1) {
 					$sort .= '<span class="moveArrowNone"></span>';
 				} else {
-					$sort .= '<a class="pic fa fa-caret-down moveArrow" href="' . htmlspecialchars('links.php?action=move_page_down&order=' . $page['sortorder'] . '&id=' . $page['id']) . '"></a>';
+					$sort .= '<a class="pic fa fa-caret-down moveArrow" href="' . html_escape('links.php?action=move_page_down&order=' . $page['sortorder'] . '&id=' . $page['id']) . '"></a>';
 				}
 
 				form_selectable_cell($sort, $page['id'], '', 'center');
@@ -424,12 +427,12 @@ function pages() {
 			$i++;
 		}
 	} else {
-		print "<tr><td><em>" . __('No Pages Found') . "</em></td></tr>\n";
+		print "<tr><td colspan='" . (cacti_sizeof($nav) + 1) . "'><em>" . __('No Pages Found') . "</em></td></tr>\n";
 	}
 
 	html_end_box(false);
 
-	if (sizeof($pages)) {
+	if (cacti_sizeof($pages)) {
 		print $nav;
 	}
 
@@ -450,9 +453,9 @@ function page_resort() {
 	$pages = db_fetch_assoc("SELECT * FROM external_links ORDER BY sortorder");
 
 	$i = 1;
-	if (sizeof($pages)) {
+	if (cacti_sizeof($pages)) {
 		foreach ($pages as $page) {
-			db_execute_prepared('UPDATE external_links SET sortorder = ? WHERE id = ?' . array($id, $page['id']));
+			db_execute_prepared('UPDATE external_links SET sortorder = ? WHERE id = ?' . array($i, $page['id']));
 			$i++;
 		}
 	}
@@ -470,7 +473,7 @@ function page_move($pageid, $junk, $direction) {
 }
 
 function edit_page() {
-	global $config;
+	global $config, $poller_intervals;
 
 	$sections = db_fetch_assoc("SELECT extendedstyle
 		FROM external_links
@@ -494,6 +497,9 @@ function edit_page() {
 		$data = array();
 	}
 
+	$myrefresh[0] = __('Disabled');
+	$myrefresh   += $poller_intervals;
+
 	$field_array = array(
 		'id' => array(
 			'friendly_name' => __('Style'),
@@ -552,18 +558,25 @@ function edit_page() {
 			'value' => (isset($data['contentfile']) ? $data['contentfile']:'')
 		),
 		'enabled' => array(
-			'friendly_name' => 'Do you want this page enabled',
+			'friendly_name' => __('Enabled'),
 			'method' => 'checkbox',
-			'description' => "If you wish this page to be viewable immediately, please check the checkbox.",
+			'description' => __('If checked, the page will be available immediately to the admin user.'),
 			'default' => 'on',
 			'value' => (isset($data['enabled']) ? 'on':'')
 		),
+		'refresh' => array(
+			'friendly_name' => __('Automatic Page Refresh'),
+			'method' => 'drop_array',
+			'array' => $myrefresh,
+			'description' => __('How often do you wish this page to be refreshed automatically.'),
+			'value' => (isset($data['refresh']) ? $data['refresh']:'')
+		),
 	);
 
-	form_start('links.php');
+	form_start('links.php', 'link_edit');
 
 	if (isset($data['title'])) {
-		html_start_box(__('External Links [edit: %s]', $data['title']), '100%', true, '3', 'center', '');
+		html_start_box(__('External Links [edit: %s]', html_escape($data['title'])), '100%', true, '3', 'center', '');
 	} else {
 		html_start_box(__('External Links [new]'), '100%', true, '3', 'center', '');
 	}
