@@ -74,6 +74,8 @@ if (isset_request_var('mode') && in_array(get_nfilter_request_var('mode'), $mode
 				api_plugin_install($id);
 			}
 
+			define('IN_PLUGIN_INSTALL', 1);
+
 			if ($_SESSION['sess_plugins_state'] >= 0) {
 				header('Location: plugins.php?state=5' . ($option != '' ? '&' . $option:''));
 			} else {
@@ -83,7 +85,11 @@ if (isset_request_var('mode') && in_array(get_nfilter_request_var('mode'), $mode
 			break;
 		case 'uninstall':
 			if (!in_array($id, $pluginslist)) break;
+
+			define('IN_PLUGIN_INSTALL', 1);
+
 			api_plugin_uninstall($id);
+
 			header('Location: plugins.php' . ($option != '' ? '?' . $option:''));
 			exit;
 			break;
@@ -187,6 +193,7 @@ function plugins_load_temp_table() {
 					WHERE directory = ?",
 					array($file));
 
+				$infoname = ($cinfo[$file]['name'] == strtolower($cinfo[$file]['name']) ? ucfirst($cinfo[$file]['name']) : $cinfo[$file]['name']);
 				if (!$exists) {
 					db_execute_prepared("INSERT INTO $table
 						(directory, name, status, author, webpage, version, requires, infoname)
@@ -199,14 +206,14 @@ function plugins_load_temp_table() {
 							$cinfo[$file]['homepage'],
 							$cinfo[$file]['version'],
 							$cinfo[$file]['requires'],
-							$cinfo[$file]['name']
+							$infoname
 						)
 					);
 				} else {
 					db_execute_prepared("UPDATE $table
 						SET infoname = ?, requires = ?
 						WHERE directory = ?",
-						array($cinfo[$file]['name'], $cinfo[$file]['requires'], $file));
+						array($infoname, $cinfo[$file]['requires'], $file));
 				}
 			}
 		}
@@ -239,7 +246,7 @@ function plugins_load_temp_table() {
 							$plugin['homepage'],
 							$plugin['version'],
 							$plugin['requires'],
-							$plugin['name']
+							($plugin['name'] == strtolower($plugin['name']) ? ucfirst($plugin['name']) : $plugin['name'])
 						)
 					);
 				} else {
@@ -545,7 +552,7 @@ function format_plugin_row($plugin, $last_plugin, $include_ordering, $table) {
 
 	$row = plugin_actions($plugin, $table);
 
-	$row .= "<td><a href='" . html_escape($plugin['webpage']) . "' target='_blank'>" . (get_request_var('filter') != '' ? preg_replace('/(' . preg_quote(get_request_var('filter')) . ')/i', "<span class='filteredValue'>\\1</span>", ucfirst($plugin['directory'])) : ucfirst($plugin['directory'])) . '</a></td>';
+	$row .= "<td><a href='" . html_escape($plugin['webpage']) . "' target='_blank' rel='noopener'>" . (get_request_var('filter') != '' ? preg_replace('/(' . preg_quote(get_request_var('filter')) . ')/i', "<span class='filteredValue'>\\1</span>", $plugin['infoname']) : $plugin['infoname']) . '</a></td>';
 
 	$row .= "<td class='nowrap'>" . filter_value($plugin['name'], get_request_var('filter')) . "</td>\n";
 
