@@ -1,4 +1,4 @@
-#!/usr/bin/php -q
+#!/usr/bin/env php
 <?php
 /*
  +-------------------------------------------------------------------------+
@@ -62,7 +62,7 @@ function sig_handler($signo) {
         case SIGINT:
 			if ($thread > 0) {
 				clearTask($network_id, getmypid());
-				exit;
+				exit(0);
 			} elseif($thread == 0 && !$master) {
 				$pids = array_rekey(db_fetch_assoc_prepared("SELECT pid
 					FROM automation_processes
@@ -100,7 +100,7 @@ function sig_handler($signo) {
 				clearTask($network_id, getmypid());
 			}
 
-            exit;
+            exit(0);
 
             break;
         default:
@@ -167,15 +167,15 @@ if (cacti_sizeof($parms)) {
 			case '-v':
 			case '--version':
 				display_version();
-				exit;
+				exit(0);
 			case '-h':
 			case '--help':
 				display_help();
-				exit;
+				exit(0);
 			default:
 				print 'ERROR: Invalid Parameter ' . $parameter . "\n\n";
 				display_help();
-				exit;
+				exit(1);
 		}
 	}
 }
@@ -189,7 +189,7 @@ if (function_exists('pcntl_signal')) {
 // Let's ensure that we were called correctly
 if (!$master && !$network_id) {
 	print "FATAL: You must specify -M to Start the Master Control Process, or the Network ID using --network\n";
-	exit;
+	exit(1);
 }
 
 // Simple check for a disabled network
@@ -202,11 +202,15 @@ if (!$master && $thread == 0) {
 
 	if ($status != 'on' && !$force) {
 		cacti_log(automation_get_pid() . " WARNING: The Network ID: $network_id is disabled.  You must use the 'force' option to force it's execution.", true, 'AUTOM8');
-		exit;
+		exit(1);
 	}
 }
 
 if ($master) {
+	if (!db_table_exists('automation_networks')) {
+		exit(0);
+	}
+
 	$networks = db_fetch_assoc_prepared('SELECT *
 		FROM automation_networks
 		WHERE poller_id = ?',
@@ -225,7 +229,7 @@ if ($master) {
 		}
 	}
 
-	exit;
+	exit(0);
 }
 
 // Check for Network Master
@@ -342,7 +346,7 @@ if (!$master && $thread == 0) {
 			clearAllTasks($network_id);
 			reportNetworkStatus($network_id, $preexisting_devices);
 
-			exit;
+			exit(0);
 		} else {
 			$failcount++;
 		}
@@ -355,7 +359,7 @@ if (!$master && $thread == 0) {
 	endTask($network_id, getmypid());
 }
 
-exit;
+exit(0);
 
 function discoverDevices($network_id, $thread) {
 	$network = db_fetch_row_prepared('SELECT *
@@ -390,7 +394,7 @@ function discoverDevices($network_id, $thread) {
 		if ($command == 'cancel' || empty($command)) {
 			removeMyProcess(getmypid(), $network_id);
 			killProcess(getmypid());
-			exit;
+			exit(0);
 		}
 
 		// set and ip to be scanned
@@ -412,10 +416,19 @@ function discoverDevices($network_id, $thread) {
 		if (cacti_sizeof($device) && isset($device['ip_address'])) {
 			$count++;
 
-			cacti_log(automation_get_pid() . ' NOTE: Found device IP address \'' . $device['ip_address'] .'\' to check',false,'AUTOM8',POLLER_VERBOSITY_MEDIUM);
+			cacti_log(automation_get_pid() . ' NOTE: Found device IP address \'' . $device['ip_address'] .'\' to check', false, 'AUTOM8', POLLER_VERBOSITY_MEDIUM);
+
+			if (!filter_var($device['ip_address'], FILTER_VALIDATE_IP)) {
+				cacti_log(automation_get_pid() . ' WARNING: IP address \'' . $device['ip_address'] .'\' is not a valid IP address.', false, 'AUTOM8');
+
+				markIPDone($device['ip_address'], $network_id);
+
+				continue;
+			}
 
 			if ($dns != '') {
 				$dnsname = automation_get_dns_from_ip($device['ip_address'], $dns, 300);
+
 				if ($dnsname != $device['ip_address'] && $dnsname != 'timed_out') {
 					automation_debug("Device: " . $device['ip_address'] . ", Checking DNS: Found '" . $dnsname . "'");
 
@@ -429,7 +442,9 @@ function discoverDevices($network_id, $thread) {
 					$device['dnsname_short'] = preg_split('/[\.]+/', strtolower($dnsname), -1, PREG_SPLIT_NO_EMPTY);
 				} elseif ($network['enable_netbios'] == 'on') {
 					automation_debug("Device: " . $device['ip_address'] . ", Checking DNS: Not found, Checking NetBIOS:");
+
 					$netbios = ping_netbios_name($device['ip_address']);
+
 					if ($netbios === false) {
 						automation_debug(" Not found");
 						$device['hostname']      = $device['ip_address'];
@@ -448,6 +463,7 @@ function discoverDevices($network_id, $thread) {
 					}
 				} else {
 					automation_debug("Device: " . $device['ip_address'] . ", Checking DNS: Not found");
+
 					$device['hostname']      = $device['ip_address'];
 					$device['dnsname']       = '';
 					$device['dnsname_short'] = '';
@@ -455,6 +471,7 @@ function discoverDevices($network_id, $thread) {
 			} else {
 				$dnsname = @gethostbyaddr($device['ip_address']);
 				$device['hostname'] = $dnsname;
+
 				if ($dnsname != $device['ip_address']) {
 					automation_debug("Device: " . $device['ip_address'] . ", Checking DNS: Found '" . $dnsname . "'");
 
@@ -467,6 +484,7 @@ function discoverDevices($network_id, $thread) {
 					$device['dnsname_short'] = preg_split('/[\.]+/', strtolower($dnsname), -1, PREG_SPLIT_NO_EMPTY);
 				} elseif ($network['enable_netbios'] == 'on') {
 					automation_debug("Device: " . $device['ip_address'] . ", Checking DNS: Not found, Checking NetBIOS:");
+
 					$netbios = ping_netbios_name($device['ip_address']);
 					if ($netbios === false) {
 						automation_debug(" Not found");
@@ -486,6 +504,7 @@ function discoverDevices($network_id, $thread) {
 					}
 				} else {
 					automation_debug("Device: " . $device['ip_address'] . ", Checking DNS: Not found");
+
 					$device['hostname']      = $device['ip_address'];
 					$device['dnsname']       = '';
 					$device['dnsname_short'] = '';
@@ -825,8 +844,8 @@ function display_help () {
 
 	print "\nusage: poller_automation.php -M [--poller=ID] | --network=network_id [-T=thread_id]\n";
 	print "    [--debug] [--force]\n\n";
-	print "Cacti's automation poller.  This poller has two operating modes, Master and Slave.\n";
-	print "The Master process tracks and launches all Slaves based upon Cacti's automation\n";
+	print "Cacti's automation poller.  This poller has two operating modes, master and worker.\n";
+	print "The master process tracks and launches all workers based upon Cacti's automation\n";
 	print "settings.  If you only want to force a network to be collected, you only need to\n";
 	print "specify the Network ID and the force options.\n\n";
 	print "Master Process:\n";
