@@ -1,4 +1,4 @@
-#!/usr/bin/php -q
+#!/usr/bin/env php
 <?php
 /*
  +-------------------------------------------------------------------------+
@@ -164,7 +164,7 @@ function display_version() {
 function display_help() {
 	display_version();
 
-	print "\nusage: poller_rt.php --graph=ID [--interval=SEC] [--force] [--debug]\n\n";
+	print "\nusage: poller_realtime.php --graph=ID [--interval=SEC] [--force] [--debug]\n\n";
 	print "Cacti's Realtime graphing poller.  This poller behavies very similary\n";
 	print "to Cacti's main poller with the exception that it only polls data source\n";
 	print "that are specific to the graph being rendered in the Cacti UI.\n\n";
@@ -203,7 +203,7 @@ function process_poller_output_rt($rrdtool_pipe, $poller_id, $interval) {
 			/* create rt rrd */
 			if (!file_exists($rt_graph_path)) {
 				/* get the syntax */
-				$command = @rrdtool_function_create($item['local_data_id'], '-60', true);
+				$command = @rrdtool_function_create($item['local_data_id'], true);
 
 				/* replace path */
 				$command = str_replace($data_source_path, $rt_graph_path, $command);
