@@ -1,4 +1,4 @@
-#!/usr/bin/php -q
+#!/usr/bin/env php
 <?php
 /*
  +-------------------------------------------------------------------------+
@@ -59,7 +59,7 @@ function sig_handler($signo) {
 		case SIGINT:
 			reports_log('WARNING: Reports Poller terminated by user', false, 'REPORTS TRACE', POLLER_VERBOSITY_LOW);
 
-			exit;
+			exit(1);
 			break;
 		default:
 			/* ignore all other signals */
@@ -97,16 +97,16 @@ if (cacti_sizeof($parms)) {
 			case '-V':
 			case '-v':
 				display_version();
-				exit;
+				exit(0);
 			case '--help':
 			case '-H':
 			case '-h':
 				display_help();
-				exit;
+				exit(0);
 			default:
 				print 'ERROR: Invalid Parameter ' . $parameter . "\n\n";
 				display_help();
-				exit;
+				exit(1);
 		}
 	}
 }
@@ -126,7 +126,19 @@ ini_set('max_execution_time', '0');
 $t = time();
 $number_sent = 0;
 
-# fetch all enabled reports that have a stratime in the past
+if (!$force) {
+	/* silently end if the registered process is still running, or process table missing */
+	if (!register_process_start('reports', 'master', 0, read_config_option('reports_timeout'))) {
+		exit(0);
+	}
+}
+
+/* cacti upgrading */
+if (!db_table_exists('reports')) {
+	exit(0);
+}
+
+/* fetch all enabled reports that have a stratime in the past */
 if (!$force) {
 	$reports = db_fetch_assoc_prepared('SELECT * FROM reports WHERE mailtime < ? AND enabled="on"', array($t));
 } else {
@@ -134,7 +146,7 @@ if (!$force) {
 }
 reports_log('Cacti Reports reports found: ' . cacti_sizeof($reports), true, 'REPORTS', POLLER_VERBOSITY_MEDIUM);
 
-# execute each of those reports
+/* execute each of those reports */
 if (cacti_sizeof($reports)) {
 	foreach ($reports as $report) {
 		reports_log('Reports processing report: ' . $report['name'], true, 'REPORTS', POLLER_VERBOSITY_MEDIUM);
@@ -150,6 +162,13 @@ if (cacti_sizeof($reports)) {
 
 	/* log statistics */
 	$reports_stats = sprintf('Time:%01.4f Reports:%s', $end - $start, $number_sent);
-	reports_log('Reports STATS: ' . $reports_stats, true, 'REPORTS', POLLER_VERBOSITY_LOW);
+	reports_log('REPORTS STATS: ' . $reports_stats, true, 'REPORTS', POLLER_VERBOSITY_LOW);
 	db_execute_prepared('REPLACE INTO settings (name, value) VALUES ("stats_reports", ?)', array($reports_stats));
 }
+
+if (!$force) {
+	unregister_process('reports', 'master', 0);
+}
+
+exit(0);
+
