@@ -176,6 +176,19 @@ case 'save':
 					array($field_name));
 			}
 		} elseif (isset_request_var($field_name)) {
+			if ($field_array['method'] == 'textbox' && isset($field_array['filter'])) {
+				if (isset($field_array['options'])) {
+					$value = filter_var(get_nfilter_request_var($field_name), $field_array['filter'], $field_array['options']);
+				} else {
+					$value = filter_var(get_nfilter_request_var($field_name), $field_array['filter']);
+				}
+				if ($value === false) {
+					$_SESSION['sess_error_fields'][$field_name] = $field_name;
+					$_SESSION['sess_field_values'][$field_name] = get_nfilter_request_var($field_name);
+					$errors[3] = 3;
+					continue;
+				}
+			}
 			if (is_array(get_nfilter_request_var($field_name))) {
 				$inserts[] = '(' . db_qstr($field_name) . ', ' . db_qstr(implode(',', get_nfilter_request_var($field_name))) . ')';
 				db_execute_prepared('REPLACE INTO settings
@@ -847,21 +860,21 @@ default:
 	function initSearch() {
 		if ($('#auth_method').val() == 3) {
 			switch($('#ldap_mode').val()) {
-			case "0":
+			case '0':
 				$('#row_ldap_search_base_header').hide();
 				$('#row_ldap_search_base').hide();
 				$('#row_ldap_search_filter').hide();
 				$('#row_ldap_specific_dn').hide();
 				$('#row_ldap_specific_password').hide();
 				break;
-			case "1":
+			case '1':
 				$('#row_ldap_search_base_header').show();
 				$('#row_ldap_search_base').show();
 				$('#row_ldap_search_filter').show();
 				$('#row_ldap_specific_dn').hide();
 				$('#row_ldap_specific_password').hide();
 				break;
-			case "2":
+			case '2':
 				$('#row_ldap_search_base_header').show();
 				$('#row_ldap_search_base').show();
 				$('#row_ldap_search_filter').show();
@@ -901,7 +914,8 @@ default:
 
 	function initAuth() {
 		switch($('#auth_method').val()) {
-		case "0": // None
+		case '0': // None
+			$('#row_path_basic_mapfile').hide();
 			$('#row_special_users_header').hide();
 			$('#row_auth_cache_enabled').hide();
 			$('#row_guest_user').hide();
@@ -943,7 +957,8 @@ default:
 			$('#row_secpass_lockfailed').hide();
 			$('#row_secpass_unlocktime').hide();
 			break;
-		case "1": // Builtin
+		case '1': // Builtin
+			$('#row_path_basic_mapfile').hide();
 			$('#row_special_users_header').show();
 			$('#row_auth_cache_enabled').show();
 			$('#row_guest_user').show();
@@ -985,7 +1000,8 @@ default:
 			$('#row_secpass_lockfailed').show();
 			$('#row_secpass_unlocktime').show();
 			break;
-		case "2": // Web Basic
+		case '2': // Web Basic
+			$('#row_path_basic_mapfile').show();
 			$('#row_special_users_header').show();
 			$('#row_auth_cache_enabled').hide();
 			$('#row_guest_user').show();
@@ -1027,7 +1043,8 @@ default:
 			$('#row_secpass_lockfailed').hide();
 			$('#row_secpass_unlocktime').hide();
 			break;
-		case "4": // Multiple Domains
+		case '4': // Multiple Domains
+			$('#row_path_basic_mapfile').hide();
 			$('#row_special_users_header').show();
 			$('#row_auth_cache_enabled').show();
 			$('#row_guest_user').show();
@@ -1069,7 +1086,7 @@ default:
 			$('#row_secpass_lockfailed').show();
 			$('#row_secpass_unlocktime').show();
 			break;
-		case "3": // Single Domain
+		case '3': // Single Domain
 			$('#row_special_users_header').show();
 			$('#row_auth_cache_enabled').show();
 			$('#row_guest_user').show();
@@ -1160,28 +1177,28 @@ default:
 
 	function initAvail() {
 		switch($('#availability_method').val()) {
-		case "0":
+		case '0':
 			$('#row_ping_method').hide();
 			$('#row_ping_port').hide();
 			$('#row_ping_timeout').hide();
 			$('#row_ping_retries').hide();
 			break;
-		case "1":
-		case "4":
+		case '1':
+		case '4':
 			$('#row_ping_method').show();
 			$('#row_ping_port').show();
 			$('#row_ping_timeout').show();
 			$('#row_ping_retries').show();
 			break;
-		case "3":
+		case '3':
 			$('#row_ping_method').show();
 			$('#row_ping_port').show();
 			$('#row_ping_timeout').show();
 			$('#row_ping_retries').show();
 			break;
-		case "2":
-		case "5":
-		case "6":
+		case '2':
+		case '5':
+		case '6':
 			$('#row_ping_method').hide();
 			$('#row_ping_port').hide();
 			$('#row_ping_timeout').show();
