@@ -30,7 +30,8 @@ include_once('./lib/data_query.php');
 $tree_actions = array(
 	1 => __x('dropdown action', 'Delete'),
 	2 => __x('dropdown action', 'Publish'),
-	3 => __x('dropdown action', 'Un Publish')
+	3 => __x('dropdown action', 'Un-Publish'),
+	4 => __x('dropdown action', 'Un-Lock')
 );
 
 /* set default action */
@@ -485,6 +486,7 @@ function form_save() {
 	if (isset_request_var('save_component_tree')) {
 		/* ================= input validation ================= */
 		get_filter_request_var('id');
+		get_filter_request_var('sequence');
 		/* ==================================================== */
 
 		if (get_filter_request_var('id') > 0) {
@@ -503,8 +505,10 @@ function form_save() {
 		$save['enabled']       = get_nfilter_request_var('enabled') == 'true' ? 'on':'-';
 		$save['modified_by']   = $_SESSION['sess_user_id'];
 
-		if (empty($save['sequence'])) {
+		if (isempty_request_var('sequence')) {
 			$save['sequence'] = tree_get_max_sequence() + 1;
+		} else {
+			$save['sequence'] = get_request_var('sequence');
 		}
 
 		if (empty($save['id'])) {
@@ -602,6 +606,12 @@ function form_actions() {
 					last_modified=NOW(),
 					modified_by=" . $_SESSION['sess_user_id'] . '
 					WHERE ' . array_to_sql_or($selected_items, 'id'));
+			} elseif (get_nfilter_request_var('drp_action') == '4') { // un-lock
+				db_execute("UPDATE graph_tree
+					SET locked=0,
+					last_modified=NOW(),
+					modified_by=" . $_SESSION['sess_user_id'] . '
+					WHERE ' . array_to_sql_or($selected_items, 'id'));
 			}
 		}
 
@@ -660,6 +670,15 @@ function form_actions() {
 			</tr>\n";
 
 			$save_html = "<input type='button' class='ui-button ui-corner-all ui-widget' value='" . __esc('Cancel') . "' onClick='cactiReturnTo()'>&nbsp;<input type='submit' class='ui-button ui-corner-all ui-widget' value='" . __esc('Continue') . "' title='" . __n('Un-publish Tree', 'Un-publish Trees', cacti_sizeof($tree_array)) . "'>";
+		} elseif (get_nfilter_request_var('drp_action') == '4') { // un-lock
+			print "<tr>
+				<td class='textArea' class='odd'>
+					<p>" . __n('Click \'Continue\' to un-lock the following Tree.', 'Click \'Continue\' to un-lock following Trees.', cacti_sizeof($tree_array)) . "</p>
+					<div class='itemlist'><ul>$tree_list</ul></div>
+				</td>
+			</tr>\n";
+
+			$save_html = "<input type='button' class='ui-button ui-corner-all ui-widget' value='" . __esc('Cancel') . "' onClick='cactiReturnTo()'>&nbsp;<input type='submit' class='ui-button ui-corner-all ui-widget' value='" . __esc('Continue') . "' title='" . __n('Un-lock Tree', 'Un-lock Trees', cacti_sizeof($tree_array)) . "'>";
 		}
 	} else {
 		raise_message(40);
@@ -815,7 +834,7 @@ function tree_edit() {
 		html_start_box('', '100%', '', '3', 'center', '');
 		html_header($display_text);
 
-		print "<tr class='tableRow'><td style='padding:7px;'><div id='sites'>\n";
+		print "<tr class='tableRow'><td style='padding:7px;'><div id='sites' style='display:none;'>\n";
 		display_sites();
 		print "</div></td></tr>\n";
 
@@ -850,7 +869,7 @@ function tree_edit() {
 		html_start_box('', '100%', '', '3', 'center', '');
 		html_header($display_text);
 
-		print "<tr class='tableRow'><td style='padding:7px;'><div id='hosts'>\n";
+		print "<tr class='tableRow'><td style='padding:7px;'><div id='hosts' style='display:none;'>\n";
 		display_hosts();
 		print "</div></td></tr>\n";
 
@@ -884,7 +903,7 @@ function tree_edit() {
 		html_start_box('', '100%', '', '3', 'center', '');
 		html_header($display_text);
 
-		print "<tr class='tableRow'><td style='padding:7px;'><div id='graphs'>\n";
+		print "<tr class='tableRow'><td style='padding:7px;'><div id='graphs' style='display:none;'>\n";
 		display_graphs();
 		print "</div></td></tr>\n";
 
@@ -911,7 +930,7 @@ function tree_edit() {
 
 		function createNode() {
 			var ref = $('#ctree').jstree(true);
-			sel = ref.create_node('#', '<?php print __('New Node');?>', '0');
+			sel = ref.create_node('#', '<?php print __esc('New Node');?>', '0');
 			if (sel) {
 				ref.edit(sel);
 			}
@@ -1048,7 +1067,7 @@ function tree_edit() {
 				event.preventDefault();
 
 				if ($(this).attr('id') == 'tree_edit') {
-					$.post('tree.php', { action: 'save', name: $('#name').val(), sort_type: $('#sort_type').val(), enabled: $('#enabled').is(':checked'), id: $('#id').val(), save_component_tree: 1, __csrf_magic: csrfMagicToken } ).done(function(data) {
+					$.post('tree.php', { action: 'save', name: $('#name').val(), sort_type: $('#sort_type').val(), enabled: $('#enabled').is(':checked'), id: $('#id').val(), save_component_tree: 1, sequence: $('#sequence').val(), __csrf_magic: csrfMagicToken } ).done(function(data) {
 						$('#main').html(data);
 						applySkin();
 					});
@@ -1080,14 +1099,16 @@ function tree_edit() {
 
 			function resizer() {
 				if ($('#ctree').length) {
-					height  = parseInt($(window).height()-$('#ctree').offset().top-10)+'px';
-					sheight = parseInt($(window).height()-$('#sites').offset().top-10)+'px';
-					hheight = parseInt($(window).height()-$('#hosts').offset().top-10)+'px';
-					gheight = parseInt($(window).height()-$('#graphs').offset().top-10)+'px';
-					$('#ctree').css('height', height).css('overflow','auto');;
-					$('#hosts').css('height', hheight).css('overflow','auto');;
-					$('#sites').css('height', hheight).css('overflow','auto');;
-					$('#graphs').css('height', gheight).css('overflow','auto');;
+					var wheight = $(window).height();
+					var cTop    = $('#ctree').parent().offset().top;
+					var sTop    = $('#sites').parent().offset().top;
+					var height  = wheight - cTop - 10;
+					var sheight = wheight - sTop - 10;
+
+					$('#ctree').css('height', height).css('overflow','auto');
+					$('#hosts').css('height', sheight).css('overflow','auto');
+					$('#sites').css('height', sheight).css('overflow','auto');
+					$('#graphs').css('height', sheight).css('overflow','auto');
 
 					switchDisplay();
 				}
@@ -1216,7 +1237,7 @@ function tree_edit() {
 						data.instance.set_text(data.node, d.text);
 						data.instance.edit(data.node);
 
-						if (d.text != '<?php print __('New Node');?>') {
+						if (d.text != '<?php print __esc('New Node');?>') {
 							$('.jstree').jstree(true).refresh();
 						}
 					})
@@ -1341,6 +1362,7 @@ function tree_edit() {
 				$(element).children().bind('contextmenu', function(event) {
 					return false;
 				});
+				$(element).show();
 		}
 
 		function branchContext(nodeid) {
@@ -1350,7 +1372,7 @@ function tree_edit() {
 					'separator_after'	: true,
 					'icon'				: 'fa fa-folder',
 					'_disabled'			: false,
-					'label'				: '<?php print __('Create');?>',
+					'label'				: '<?php print __esc('Create');?>',
 					'action'			: function (data) {
 						var inst = $.jstree.reference(data.reference);
 						var obj = inst.get_node(data.reference);
@@ -1364,7 +1386,7 @@ function tree_edit() {
 					'separator_after'	: false,
 					'icon'				: 'fa fa-pencil-alt',
 					'_disabled'			: false,
-					'label'				: '<?php print __('Rename');?>',
+					'label'				: '<?php print __esc('Rename');?>',
 					'action'			: function (data) {
 						var inst = $.jstree.reference(data.reference);
 						var obj = inst.get_node(data.reference);
@@ -1376,7 +1398,7 @@ function tree_edit() {
 					'icon'				: 'fa fa-times',
 					'separator_after'	: false,
 					'_disabled'			: false,
-					'label'				: '<?php print __('Delete');?>',
+					'label'				: '<?php print __esc('Delete');?>',
 					'action'			: function (data) {
 						var inst = $.jstree.reference(data.reference);
 						var obj = inst.get_node(data.reference);
@@ -1391,14 +1413,14 @@ function tree_edit() {
 					'separator_before'	: true,
 					'icon'				: 'fa fa-sort',
 					'separator_after'	: false,
-					'label'				: '<?php print __('Branch Sorting');?>',
+					'label'				: '<?php print __esc('Branch Sorting');?>',
 					'action'			: false,
 					'submenu' : {
 						'inherit' : {
 							'separator_before'	: false,
 							'separator_after'	: false,
 							'icon'				: getBranchSortIcon('inherit', nodeid),
-							'label'				: '<?php print __('Inherit');?>',
+							'label'				: '<?php print __esc('Inherit');?>',
 							'action'			: function (data) {
 								setBranchSortOrder('inherit', nodeid);
 								var inst = $.jstree.reference(data.reference);
@@ -1412,7 +1434,7 @@ function tree_edit() {
 							'separator_before'	: false,
 							'separator_after'	: false,
 							'icon'				: getBranchSortIcon('manual', nodeid),
-							'label'				: '<?php print __('Manual');?>',
+							'label'				: '<?php print __esc('Manual');?>',
 							'action'			: function (data) {
 								setBranchSortOrder('manual', nodeid);
 								var inst = $.jstree.reference(data.reference);
@@ -1426,7 +1448,7 @@ function tree_edit() {
 							'separator_before'	: false,
 							'icon'				: getBranchSortIcon('alpha', nodeid),
 							'separator_after'	: false,
-							'label'				: '<?php print __('Alphabetic');?>',
+							'label'				: '<?php print __esc('Alphabetic');?>',
 							'action'			: function (data) {
 								setBranchSortOrder('alpha', nodeid);
 								var inst = $.jstree.reference(data.reference);
@@ -1440,7 +1462,7 @@ function tree_edit() {
 							'separator_before'	: false,
 							'icon'				: getBranchSortIcon('natural', nodeid),
 							'separator_after'	: false,
-							'label'				: '<?php print __('Natural');?>',
+							'label'				: '<?php print __esc('Natural');?>',
 							'action'			: function (data) {
 								setBranchSortOrder('natural', nodeid);
 								var inst = $.jstree.reference(data.reference);
@@ -1454,7 +1476,7 @@ function tree_edit() {
 							'separator_before'	: false,
 							'icon'				: getBranchSortIcon('numeric', nodeid),
 							'separator_after'	: false,
-							'label'				: '<?php print __('Numeric');?>',
+							'label'				: '<?php print __esc('Numeric');?>',
 							'action'			: function (data) {
 								setBranchSortOrder('numeric', nodeid);
 								var inst = $.jstree.reference(data.reference);
@@ -1470,14 +1492,14 @@ function tree_edit() {
 					'separator_before'	: true,
 					'icon'				: 'fa fa-edit',
 					'separator_after'	: false,
-					'label'				: '<?php print __('Edit');?>',
+					'label'				: '<?php print __esc('Edit');?>',
 					'action'			: false,
 					'submenu' : {
 						'cut' : {
 							'separator_before'	: false,
 							'separator_after'	: false,
 							'icon'				: 'fa fa-cut',
-							'label'				: '<?php print __('Cut');?>',
+							'label'				: '<?php print __esc('Cut');?>',
 							'action'			: function (data) {
 								var inst = $.jstree.reference(data.reference);
 								var obj = inst.get_node(data.reference);
@@ -1492,7 +1514,7 @@ function tree_edit() {
 							'separator_before'	: false,
 							'icon'				: 'fa fa-copy',
 							'separator_after'	: false,
-							'label'				: '<?php print __('Copy');?>',
+							'label'				: '<?php print __esc('Copy');?>',
 							'action'			: function (data) {
 								var inst = $.jstree.reference(data.reference);
 								var obj = inst.get_node(data.reference);
@@ -1510,7 +1532,7 @@ function tree_edit() {
 								return !$.jstree.reference(data.reference).can_paste();
 							},
 							'separator_after'	: false,
-							'label'				: '<?php print __('Paste');?>',
+							'label'				: '<?php print __esc('Paste');?>',
 							'action'			: function (data) {
 								var inst = $.jstree.reference(data.reference);
 								var obj = inst.get_node(data.reference);
@@ -1529,7 +1551,7 @@ function tree_edit() {
 					'icon'				: 'fa fa-times',
 					'separator_after'	: false,
 					'_disabled'			: false, //(this.check('delete_node', data.reference, this.get_parent(data.reference), '')),
-					'label'				: '<?php print __('Delete');?>',
+					'label'				: '<?php print __esc('Delete');?>',
 					'action'			: function (data) {
 						var inst = $.jstree.reference(data.reference);
 						var obj = inst.get_node(data.reference);
@@ -1544,14 +1566,14 @@ function tree_edit() {
 					'separator_before'	: true,
 					'icon'				: 'fa fa-edit',
 					'separator_after'	: false,
-					'label'				: '<?php print __('Edit');?>',
+					'label'				: '<?php print __esc('Edit');?>',
 					'action'			: false,
 					'submenu' : {
 						'cut' : {
 							'separator_before'	: false,
 							'separator_after'	: false,
 							'icon'				: 'fa fa-cut',
-							'label'				: '<?php print __('Cut');?>',
+							'label'				: '<?php print __esc('Cut');?>',
 							'action'			: function (data) {
 								var inst = $.jstree.reference(data.reference);
 								var obj = inst.get_node(data.reference);
@@ -1566,7 +1588,7 @@ function tree_edit() {
 							'separator_before'	: false,
 							'icon'				: 'fa fa-copy',
 							'separator_after'	: false,
-							'label'				: '<?php print __('Copy');?>',
+							'label'				: '<?php print __esc('Copy');?>',
 							'action'			: function (data) {
 								var inst = $.jstree.reference(data.reference);
 								var obj = inst.get_node(data.reference);
@@ -1589,7 +1611,7 @@ function tree_edit() {
 					'icon'				: 'fa fa-times',
 					'separator_after'	: false,
 					'_disabled'			: false,
-					'label'				: '<?php print __('Delete');?>',
+					'label'				: '<?php print __esc('Delete');?>',
 					'action'			: function (data) {
 						var inst = $.jstree.reference(data.reference);
 						var obj = inst.get_node(data.reference);
@@ -1610,7 +1632,7 @@ function tree_edit() {
 					'icon'				: 'fa fa-times',
 					'separator_after'	: false,
 					'_disabled'			: false,
-					'label'				: '<?php print __('Delete');?>',
+					'label'				: '<?php print __esc('Delete');?>',
 					'action'			: function (data) {
 						var inst = $.jstree.reference(data.reference);
 						var obj = inst.get_node(data.reference);
@@ -1625,14 +1647,14 @@ function tree_edit() {
 					'separator_before'	: true,
 					'separator_after'	: false,
 					'icon'				: 'fa fa-sort',
-					'label'				: '<?php print __('Sorting Type');?>',
+					'label'				: '<?php print __esc('Sorting Type');?>',
 					'action'			: false,
 					'submenu' : {
 						'hsgt' : {
 							'separator_before'	: false,
 							'icon'				: getHostSortIcon('hsgt', nodeid),
 							'separator_after'	: false,
-							'label'				: '<?php print __('Graph Template');?>',
+							'label'				: '<?php print __esc('Graph Template');?>',
 							'action'			: function (data) {
 								setHostSortOrder('hsgt', nodeid);
 							}
@@ -1641,7 +1663,7 @@ function tree_edit() {
 							'separator_before'	: false,
 							'icon'				: getHostSortIcon('hsdq', nodeid),
 							'separator_after'	: false,
-							'label'				: '<?php print __('Data Query Index');?>',
+							'label'				: '<?php print __esc('Data Query Index');?>',
 							'action'			: function (data) {
 								setHostSortOrder('hsdq', nodeid);
 							}
@@ -1652,14 +1674,14 @@ function tree_edit() {
 					'separator_before'	: true,
 					'icon'				: 'fa fa-edit',
 					'separator_after'	: false,
-					'label'				: '<?php print __('Edit');?>',
+					'label'				: '<?php print __esc('Edit');?>',
 					'action'			: false,
 					'submenu' : {
 						'cut' : {
 							'separator_before'	: false,
 							'separator_after'	: false,
 							'icon'				: 'fa fa-cut',
-							'label'				: '<?php print __('Cut');?>',
+							'label'				: '<?php print __esc('Cut');?>',
 							'action'			: function (data) {
 								var inst = $.jstree.reference(data.reference),
 									obj = inst.get_node(data.reference);
@@ -1674,7 +1696,7 @@ function tree_edit() {
 							'separator_before'	: false,
 							'icon'				: 'fa fa-copy',
 							'separator_after'	: false,
-							'label'				: '<?php print __('Copy');?>',
+							'label'				: '<?php print __esc('Copy');?>',
 							'action'			: function (data) {
 								var inst = $.jstree.reference(data.reference),
 									obj = inst.get_node(data.reference);
@@ -1724,7 +1746,7 @@ function display_sites() {
 
 	if (cacti_sizeof($sites)) {
 		foreach($sites as $s) {
-			print "<ul><li id='tsite:" . $s['id'] . "' data-jstree='{ \"type\" : \"site\"}'>" . $s['name'] . "</li></ul>\n";
+			print "<ul><li id='tsite:" . $s['id'] . "' data-jstree='{ \"type\" : \"site\"}'>" . html_escape($s['name']) . "</li></ul>\n";
 		}
 	}
 }
@@ -1741,11 +1763,11 @@ function display_hosts() {
 		$sql_where .= ($sql_where != '' ? ' AND ':'') . 'h.site_id = ' . get_filter_request_var('site_id');
 	}
 
-	$hosts = get_allowed_devices($sql_where, 'description', '20');
+	$hosts = get_allowed_devices($sql_where, 'description', read_config_option('autocomplete_rows'));
 
 	if (cacti_sizeof($hosts)) {
 		foreach($hosts as $h) {
-			print "<ul><li id='thost:" . $h['id'] . "' data-jstree='{ \"type\" : \"device\"}'>" . $h['description'] . ' (' . $h['hostname'] . ')' . "</li></ul>\n";
+			print "<ul><li id='thost:" . $h['id'] . "' data-jstree='{ \"type\" : \"device\"}'>" . html_escape($h['description']) . ' (' . html_escape($h['hostname']) . ')' . "</li></ul>\n";
 		}
 	}
 }
@@ -1783,7 +1805,7 @@ function display_graphs() {
 		ON gl.host_id = h.id
 		$sql_where
 		ORDER BY title_cache
-		LIMIT 20");
+		LIMIT " . read_config_option('autocomplete_rows'));
 
 	if (cacti_sizeof($graphs)) {
 		foreach($graphs as $g) {
@@ -1962,7 +1984,7 @@ function tree() {
 		$sql_order
 		$sql_limit");
 
-	$total_rows = db_fetch_cell("SELECT COUNT(DISTINCT(ti.graph_tree_id))
+	$total_rows = db_fetch_cell("SELECT COUNT(DISTINCT(t.id))
 		FROM graph_tree AS t
 		LEFT JOIN graph_tree_items AS ti
 		ON t.id=ti.graph_tree_id
@@ -1977,18 +1999,79 @@ function tree() {
 	html_start_box('', '100%', '', '3', 'center', '');
 
 	$display_text = array(
-		'name' => array('display' => __('Tree Name'), 'align' => 'left', 'sort' => 'ASC', 'tip' => __('The name by which this Tree will be referred to as.')),
-		'id' => array('display' => __('ID'), 'align' => 'right', 'sort' => 'ASC', 'tip' => __('The internal database ID for this Tree.  Useful when performing automation or debugging.')),
-		'enabled' => array('display' => __('Published'), 'align' => 'left', 'sort' => 'ASC', 'tip' => __('Unpublished Trees cannot be viewed from the Graph tab')),
-		'locked' => array('display' => __('Locked'), 'align' => 'left', 'sort' => 'ASC', 'tip' => __('A Tree must be locked in order to be edited.')),
-		'user_id' => array('display' => __('Owner'), 'align' => 'left', 'sort' => 'ASC', 'tip' => __('The original author of this Tree.')),
-		'sequence' => array('display' => __('Order'), 'align' => 'center', 'sort' => 'ASC', 'tip' => __('To change the order of the trees, first sort by this column, press the up or down arrows once they appear.')),
-		'last_modified' => array('display' => __('Last Edited'), 'align' => 'right', 'sort' => 'ASC', 'tip' => __('The date that this Tree was last edited.')),
-		'modified_by' => array('display' => __('Edited By'), 'align' => 'right', 'sort' => 'ASC', 'tip' => __('The last user to have modified this Tree.')),
-		'sites' => array('display' => __('Sites'), 'align' => 'right', 'sort' => 'DESC', 'tip' => __('The total number of Site Branches in this Tree.')),
-		'branches' => array('display' => __('Branches'), 'align' => 'right', 'sort' => 'DESC', 'tip' => __('The total number of Branches in this Tree.')),
-		'hosts' => array('display' => __('Devices'), 'align' => 'right', 'sort' => 'DESC', 'tip' => __('The total number of individual Devices in this Tree.')),
-		'graphs' => array('display' => __('Graphs'), 'align' => 'right', 'sort' => 'DESC', 'tip' => __('The total number of individual Graphs in this Tree.')));
+		'name' => array(
+			'display' => __('Tree Name'),
+			'align' => 'left',
+			'sort' => 'ASC',
+			'tip' => __('The name by which this Tree will be referred to as.')
+		),
+		'id' => array(
+			'display' => __('ID'),
+			'align' => 'right',
+			'sort' => 'ASC',
+			'tip' => __('The internal database ID for this Tree.  Useful when performing automation or debugging.')
+		),
+		'enabled' => array(
+			'display' => __('Published'),
+			'align' => 'left',
+			'sort' => 'ASC',
+			'tip' => __('Unpublished Trees cannot be viewed from the Graph tab')
+		),
+		'locked' => array(
+			'display' => __('Locked'),
+			'align' => 'left',
+			'sort' => 'ASC',
+			'tip' => __('A Tree must be locked in order to be edited.')
+		),
+		'user_id' => array(
+			'display' => __('Owner'),
+			'align' => 'left',
+			'sort' => 'ASC',
+			'tip' => __('The original author of this Tree.')
+		),
+		'sequence' => array(
+			'display' => __('Order'),
+			'align' => 'center',
+			'sort' => 'ASC',
+			'tip' => __('To change the order of the trees, first sort by this column, press the up or down arrows once they appear.')
+		),
+		'last_modified' => array(
+			'display' => __('Last Edited'),
+			'align' => 'right',
+			'sort' => 'ASC',
+			'tip' => __('The date that this Tree was last edited.')
+		),
+		'modified_by' => array(
+			'display' => __('Edited By'),
+			'align' => 'right',
+			'sort' => 'ASC',
+			'tip' => __('The last user to have modified this Tree.')
+		),
+		'sites' => array(
+			'display' => __('Sites'),
+			'align' => 'right',
+			'sort' => 'DESC',
+			'tip' => __('The total number of Site Branches in this Tree.')
+		),
+		'branches' => array(
+			'display' => __('Branches'),
+			'align' => 'right',
+			'sort' => 'DESC',
+			'tip' => __('The total number of Branches in this Tree.')
+		),
+		'hosts' => array(
+			'display' => __('Devices'),
+			'align' => 'right',
+			'sort' => 'DESC',
+			'tip' => __('The total number of individual Devices in this Tree.')
+		),
+		'graphs' => array(
+			'display' => __('Graphs'),
+			'align' => 'right',
+			'sort' => 'DESC',
+			'tip' => __('The total number of individual Graphs in this Tree.')
+		)
+	);
 
 	html_header_sort_checkbox($display_text, get_request_var('sort_column'), get_request_var('sort_direction'), false);
 
